// View.h
#pragma once

enum FilerMessage
{
	WM_GOABSOLUTE	= WM_APP,	// BOOL OnGoAbsolute(CShellItem* lParam);
};

class CView;
typedef CShellMenuHandler< MENU_SHELL_FIRST, MENU_SHELL_LAST,
		CFileMenuHandler < MENU_FILE_FIRST , MENU_FILE_LAST,
		CMenuHandler<CView> > >
		CViewMenuHandler;
typedef CWinTraits<WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, 0>	CViewTraits;

class CView :
	public CFileListCtrlImpl<CView, CWindow, CViewTraits>,
	public CViewCommands<CView>,
	public CViewMenuHandler
{
	typedef CFileListCtrlImpl<CView, CWindow, CViewTraits>	baseClass;
	typedef CViewCommands<CView>	baseCommands;
	typedef CViewMenuHandler	baseMenu;

private:
	CIcon	m_Icon;
	static HACCEL s_hAccelTable;

public:
	DECLARE_WND_SUPERCLASS(_T("FileView"), GetWndClassName())

	static CView* New(const CShellItem* pFullPath, IShellFolder* pShellFolderHint, WindowID window)
	{
		if(pFullPath == NULL)
			return NULL;
		CComPtr<IShellFolder> pShellFolder;
		if(!pShellFolderHint)
		{
			if FAILED(pFullPath->GetSelfFolder(&pShellFolder))
				return NULL;
			pShellFolderHint = pShellFolder;
		}
		CView* view = new CView();
		HWND hView = view->Create(theTablet->GetWindow(window));
		if(!hView)
			return false;
		if FAILED(view->GoAbsolute(pFullPath, pShellFolderHint))
		{
			view->DestroyWindow();
			return NULL;
		}
		// pXǉB
		TCHAR path[MAX_PATH];
		if(view->GetCurrentFolderItem()->GetPath(path))
			theTablet->AddHistory(path);
		return view;
	}

	static bool ReferToCreateNewView(const CShellItem* pFullPath, IShellFolder* pShellFolder)
	{
		if(CKeyboard::IsPressed(VK_CONTROL))
		{	// Ctrl => V^uŊJ
			CView::New(pFullPath, pShellFolder, WINDOW_PANE_ACTIVE);
			return true;
		}
		else if(CKeyboard::IsPressed(VK_SHIFT))
		{	// Shift => VyCŊJ
			UINT nPane = theTablet->CountElements(WINDOW_PANESPLITTER);
			if(nPane < MAX_PANE_COUNT)
			{
				CView::New(pFullPath, pShellFolder, (WindowID)(WINDOW_PANE_FIRST+nPane));
			}
			else
			{	// ɃyC͍ő吔JĂBdȂ̂ŁAy̐V^uŊJB
				CView::New(pFullPath, pShellFolder, WINDOW_PANE_ACTIVE);
			}
			return true;
		}
		else
			return false;
	}

	static bool IsInstance(HWND hWnd)
	{
		TCHAR className[MAX_PATH];
		return hWnd && ::GetClassName(hWnd, className, MAX_PATH) != 0 && lstrcmp(className, GetWndClassInfo().m_wc.lpszClassName) == 0;
	}

	static void Init()
	{
		s_hAccelTable = AtlLoadAccelerators(IDR_VIEW_ACCEL);
	}

	static void Term()
	{
		::DestroyAcceleratorTable(s_hAccelTable);
		s_hAccelTable = NULL;
	}

	void OnFinalMessage(HWND)
	{
		// Ƃ̃pXǉB
		TCHAR path[MAX_PATH];
		if(!IsEmpty() && GetCurrentFolderItem()->GetPath(path))
			theTablet->AddHistory(path);
		delete this;
	}

	////////////////////////////////////////////////////////////////////////////////
	// CViewCommands

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		if(baseClass::PreTranslateMessage(pMsg))
			return TRUE;
		if(::TranslateAccelerator(m_hWnd, s_hAccelTable, pMsg))
			return TRUE;
		return FALSE;
	}

	HICON DoGetIcon(UINT type)
	{
		return m_Icon;
	}

	BOOL DoGetNames(LPTSTR szShortName, LPTSTR szLongName)
	{
		TCHAR name[MAX_PATH];
		GetWindowText(name, MAX_PATH);
		if(szShortName)
			lstrcpy(szShortName, PathFindFileName(name));
		if(szLongName)
			lstrcpy(szLongName, name);
		return TRUE;
	}

	BOOL DoBeginGesture(UINT key, CPoint ptScreen)
	{
		CListViewCtrl list = GetListCtrl();
		if(!list)
			return FALSE;
		LVHITTESTINFO hit = { ptScreen.x, ptScreen.y, 0, -1, 0 };
		list.ScreenToClient(&hit.pt);
		list.HitTest(&hit);
		// ACełȂ΃WFX`󂯓
		return hit.iItem < 0;
	}

	void DoEndGesture(UINT length, const Gesture* gesture)
	{
		if(length == 1)
		{
			switch(gesture[0])
			{
			case GESTURE_NORTH:
				GoUp();
				break;
			case GESTURE_WEST:
				GoBack();
				break;
			case GESTURE_EAST:
				GoForward();
				break;
			case GESTURE_SOUTH:
				PostMessage(WM_CLOSE);
				break;
			}
		}
	}

	struct MenuInserter
	{
		HMENU	hMenu;
		UINT	cmd;
		UINT	index;
		void operator () (const CFileListCtrlTravelEntry* entry)
		{
			if(entry->Path[0] != _T('\0'))
				::InsertMenu(hMenu, index, MF_BYPOSITION, cmd, entry->Path);
			else
				::InsertMenu(hMenu, index, MF_BYPOSITION, cmd, entry->Name);
			++index;
			++cmd;
		}
	};

	BOOL QueryTravelLogToMenu(HMENU hMenu, INT num, UINT nCmdFirst, UINT nInsertIndex)
	{
		MenuInserter op = { hMenu, nCmdFirst, nInsertIndex };
		return SUCCEEDED(QueryTravelLog(op, num));
	}

	BOOL QueryUpMenu(HMENU hMenu, UINT nCmdFirst, UINT nInsertIndex)
	{
		// ƗpɕB
		CAutoPtr<CShellItemTask> item = CShellItemTask::Duplicate(GetTravelEntry()->pShellItem);
		CShellItem* pLeaf = item->FindLeaf();
		for(int i = 0; i < 16 && pLeaf && !pLeaf->IsRoot(); i++)
		{
			TCHAR name[MAX_PATH];
			item->RemoveLeaf(pLeaf);
			if(!item->IsRoot() && item->GetPath(name))
				pLeaf = item->FindLeaf();
			else if(FAILED(item->GetProperty(name, NULL, &pLeaf)))
				break;
			::InsertMenu(hMenu, nInsertIndex+i, MF_BYPOSITION, nCmdFirst+i, name);
		}
		return TRUE;
	}

#define ID_FWF_AUTOARRANGE	13
#define ID_FWF_CHECKSELECT	14

	BOOL QueryShowMenu(HMENU hMenu, UINT nCmdFirst, UINT nInsertIndex)
	{
		OSVERSIONINFO ovi;
		ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		if(::GetVersionEx(&ovi) && ovi.dwPlatformId == VER_PLATFORM_WIN32_NT && ovi.dwMajorVersion >= 4 && ovi.dwMinorVersion >= 1) 
		{	// for XP or later
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_THUMBNAIL	, _T("k(&H)"));
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_TILE		, _T("ׂĕ\(&S)"));
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_ICON		, _T("ACR(&N)"));
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_LIST		, _T("ꗗ(&L)"));
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_DETAILS	, _T("ڍ(&D)"));
		}
		else
		{	// for 9x, 2000
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_ICON		, _T("傫ȃACR(&L)"));
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_SMALLICON	, _T("ȃACR(&S)"));
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_LIST		, _T("ꗗ(&L)"));
			::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+FVM_DETAILS	, _T("ڍ(&D)"));
		}
		//
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION | MF_SEPARATOR, 0, 0);
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+ID_FWF_AUTOARRANGE	, _T("ACR̎(&A)"));
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst+ID_FWF_CHECKSELECT	, _T("`FbN{bNX(&C)"));
		// 
		FOLDERSETTINGS fs;
		if SUCCEEDED(GetFolderSettings(&fs))
		{
			::CheckMenuItem(hMenu, nCmdFirst+fs.ViewMode, MF_CHECKED);
			if(fs.fFlags & FWF_AUTOARRANGE)
				::CheckMenuItem(hMenu, nCmdFirst+ID_FWF_AUTOARRANGE, MF_CHECKED);
			if(fs.fFlags & FWF_CHECKSELECT)
				::CheckMenuItem(hMenu, nCmdFirst+ID_FWF_CHECKSELECT, MF_CHECKED);
		}
		return TRUE;
	}

	void OnViewShow(UINT index)
	{
		FOLDERSETTINGS settings;
		if FAILED(GetFolderSettings(&settings))
			return;
		if(FVM_FIRST <= index && index <= FVM_LAST && settings.ViewMode != index)
			settings.ViewMode = index;
		else if(index == ID_FWF_AUTOARRANGE)
			settings.fFlags ^= FWF_AUTOARRANGE;
		else if(index == ID_FWF_CHECKSELECT)
			settings.fFlags ^= FWF_CHECKSELECT;
		else
			return;
		SetFolderSettings(&settings);
	}

	BOOL DoQueryMenu(HMENU hMenu, UINT cmd, UINT insert)
	{
		switch(cmd)
		{
		case ID_VIEW_BACK:
			return QueryTravelLogToMenu(hMenu, -16, cmd, insert);
		case ID_VIEW_FORWARD:
			return QueryTravelLogToMenu(hMenu, +16, cmd, insert);
		case ID_VIEW_UP:
			return QueryUpMenu(hMenu, cmd, insert);
		case ID_VIEW_SHOW:
			return QueryShowMenu(hMenu, cmd, insert);
		default:
			return FALSE;
		}
	}

	////////////////////////////////////////////////////////////////////////////////
	// Message Map

	BEGIN_MSG_MAP(CView)
		MESSAGE_HANDLER  (WM_CONTEXTMENU	, OnContextMenu)
		MESSAGE_HANDLER  (WM_TABLET_TABMOUSE, OnTabMouse)
		MESSAGE_LAMBDA   (WM_GOABSOLUTE		, { GoAbsolute((CShellItem*)lParam); })
//		COMMAND_ID_LAMBDA(ID_VIEW_STOP		, { Stop(); })
		COMMAND_ID_LAMBDA(ID_VIEW_REFRESH	, { Refresh(); })
		COMMAND_ID_LAMBDA(ID_VIEW_HOME		, { GoHome(); })
		COMMAND_CATEGORY_ID_HANDLER(ID_VIEW_FORWARD	, GoForward , +1)
		COMMAND_CATEGORY_ID_HANDLER(ID_VIEW_BACK	, GoBack    , +1)
		COMMAND_CATEGORY_ID_HANDLER(ID_VIEW_UP		, GoUp      , +1)
		COMMAND_CATEGORY_ID_HANDLER(ID_VIEW_SHOW	, OnViewShow,  0)
		COMMAND_ID_HANDLER(ID_EDIT_SELECT_CHECKED,	OnEditSelectChecked)
		CHAIN_MSG_MAP(baseCommands)
		CHAIN_MSG_MAP(baseMenu)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	// ݃`FbNĂACeIB
	LRESULT OnEditSelectChecked(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CAutoPtr<CShellItemListGlobal> all = GetAllItems();
		CListViewCtrl list = GetListCtrl();
		IShellView* pShellView = GetShellView();
		if(!all || !list || !pShellView)
			return 0;
		ATLASSERT(list.GetItemCount() == all->GetCount());
		DeselectAll();
		UINT nCount = all->GetCount();
		for(UINT i = 0; i < nCount; i++)
		{
			if(list.GetCheckState(i))
				pShellView->SelectItem(all->GetAt(i), SVSI_SELECT | SVSI_CHECK);
		}
		return 0;
	}

	HRESULT OnGetMenuItemName(DWORD wID, HMENU hMenu, LPTSTR name, INT cch)
	{
		if SUCCEEDED(Filer::OnGetMenuItemName(wID, hMenu, name, cch))
			return S_OK;
		else
			return __super::OnGetMenuItemName(wID, hMenu, name, cch);
	}

	void OnContextMenuSelected(CMenuHandle popup, CPoint ptScreen)
	{
		CAutoPtr<CShellItemListGlobal> items = GetSelectedItems();
		// I𒆂̃ACeTutH_̃tH_ۂ𒲂ׂB
		if(items && items->GetCount() > 0)
		{
			CAutoPtr<CShellItemTask> pSelected = CShellItemTask::Concat(items->GetParent(), items->GetAt(0));
			DWORD attr = SFGAO_FOLDER;
			TCHAR name[MAX_PATH];
			pSelected->GetProperty(name, &attr);
			if(attr & SFGAO_FOLDER)
			{	// tH_̏ꍇ̓t@Cj[}B
				InsertFolderMenu(popup, name, pSelected, 0);
				::InsertMenu(popup, 1, MF_BYPOSITION | MF_SEPARATOR, 0, 0);
				::SetMenuDefaultItem(popup, 0, TRUE);
			}
			else
			{	// ʏt@C̏ꍇ̓vCx[gj[}B
				Filer::InsertPrivateMenu(popup);
			}
		}
		UINT cmd = theTablet->TrackPopupMenu(popup, ptScreen, m_hWnd);
		if(MENU_FILER <= cmd)
		{	// vCx[gR}h
			ATLASSERT(items);
			theTablet->OpenFile(FileEntry(items), Filer::CommandToWindow(cmd));
		}
		else if(MENU_FILE_FIRST <= cmd && cmd <= MENU_FILE_LAST)
		{	// t@CR}h
			const CShellItem* pItem = LookupShellItem(cmd);
			if(pItem)
			{
				CComPtr<IShellFolder> pShellFolder;
				if SUCCEEDED(pItem->GetSelfFolder(&pShellFolder))
					ReferToCreateNewView(pItem, pShellFolder) || GoAbsolute(pItem, pShellFolder);
				else
					theTablet->OpenFile(pItem, WINDOW_PANE_ACTIVE);
			}
		}
		else
		{	// VFR}h
			InvokeCommand(cmd, m_hWnd);
		}
		DisposeMenuResource();
	}

	void OnContextMenuBlank(CMenuHandle popup, CPoint ptScreen)
	{
		const CFileListCtrlTravelEntry* entry = GetTravelEntry();
		UINT nCount = popup.GetMenuItemCount();
		MENUITEMINFO info = { sizeof(MENUITEMINFO), MIIM_TYPE };
		popup.GetMenuItemInfo(nCount-1, TRUE, &info);
		if((info.fType & MFT_SEPARATOR) == 0)
		{	// j[̍ŌオZp[^łȂ΁AZp[^}B
			// ȂXP̃obNOEhReLXgł́AŌオZp[^ɂȂĂBB
			popup.AppendMenu(MF_SEPARATOR);
		}
		popup.AppendMenu(0, ID_FILER_SYNC, _T("c[𓯊(&T)"));
		UINT cmd = theTablet->TrackPopupMenu(popup, ptScreen, m_hWnd);
		switch(cmd)
		{
		case ID_FILER_SYNC:
			// FolderTree.EnsureVisible(pCurrentFolder);
			break;
		default:
			InvokeCommand(cmd, m_hWnd);
		}
		DisposeMenuResource();
	}

	LRESULT OnContextMenu(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		CPoint ptScreen(lParam);
		// j[L[̏ꍇ̓ʏ
		if(ptScreen.x == -1 || ptScreen.y == -1)
		{
			CListViewCtrl list = GetListCtrl();
			int index = list.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
			RECT rc;
			if(index >= 0 && list.GetItemRect(index, &rc, LVIR_BOUNDS))
			{
				ptScreen.x = rc.left;
				ptScreen.y = rc.bottom;
			}
			else
			{	// IACeȂ
				ptScreen.x = ptScreen.y = 0;
			}
			ClientToScreen(&ptScreen);
		}
		CMenu popup;
		popup.CreatePopupMenu();
		if SUCCEEDED(InsertShellMenu(popup, 0, GetShellView(), SVGIO_SELECTION, CMF_NORMAL))
		{	// IACeꍇ
			OnContextMenuSelected(popup.m_hMenu, ptScreen);
		}
		else if SUCCEEDED(InsertShellMenu(popup, 0, GetShellView(), SVGIO_BACKGROUND, CMF_NORMAL))
		{	// IACeꍇ
			OnContextMenuBlank(popup.m_hMenu, ptScreen);
		}
		return 0;
	}

	LRESULT OnTabMouse(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		switch(wParam)
		{
		case WM_LBUTTONDBLCLK:
            Refresh();
			break;
		}
		return FALSE;
	}
	
/*	// 
	void OnDefaultExcecute(const CShellItem* pFullPath)
	{
	// CtrlĂΊOD`݂
		if(theTablet->OpenFile(pFullPath, WINDOW_PANE_ACTIVE, -1, SUIT_HIGH))
		{
			// ΉvOCŊJꂽ
		}
		else
		{	// VFɔCB
			__super::OnDefaultExcecute(pFullPath);
		}
	}
*/

	// ftHgR}hŃfBNgړꍇɌĂ΂܂B
	void OnDefaultDirectoryChange(const CShellItem* pFullPath, IShellFolder* pShellFolder)
	{
		if(!ReferToCreateNewView(pFullPath, pShellFolder))
		{	// r[ŃtH_ړ
			GoAbsolute(pFullPath, pShellFolder);
			if(CKeyboard::IsPressed(VK_RETURN))
			{	// L[{[hɂړ̏ꍇ́Aŏ̃ACeIB
				CAutoPtr<CShellItemListGlobal> items = GetAllItems();
				if(items && items->GetCount() > 0)
				{	// ŏ̃ACeIB
					GetShellView()->SelectItem(items->GetAt(0), SVSI_FOCUSED | SVSI_SELECT | SVSI_ENSUREVISIBLE);
				}
			}
		}
	}

	bool OnDirectoryChanging(const CShellItem* pFullPath)
	{
		// ̏ꍇ̓ʏȂƁAɐVEBhE葱Ă܂B
		return IsEmpty() || !ReferToCreateNewView(pFullPath, NULL);
	}

	void OnDirectoryChanged(const CFileListCtrlTravelEntry* entry, IShellFolder* pShellFolder)
	{
		if(entry->Path[0] != _T('\0'))
			SetWindowText(entry->Path);
		else
			SetWindowText(entry->Name);
		// ACR擾
		SHFILEINFO file = {0};
		entry->pShellItem->GetFileInfo(&file, SHGFI_ICON);
		m_Icon = file.hIcon;
		// O̕ύXʒm.
		NotifyNameChanged();
	}

	STDMETHODIMP GetControlWindow(UINT id, HWND* phWnd)
	{
		ATLASSERT(phWnd);
		if(!phWnd)
			return E_POINTER;
		// ₢킹̂́AFCW_STATUS ̂݁B
		switch(id)
		{
		case FCW_STATUS:
			*phWnd = theTablet->GetWindow(WINDOW_STATUSBAR);
			return S_OK;
		default:
			return E_INVALIDARG;
		}
	}

	STDMETHODIMP SendControlMsg(UINT id, UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT *pret)
	{
		if(theTablet->GetWindow(WINDOW_VIEW_ACTIVE) != m_hWnd)
			return E_FAIL;
		LRESULT lResult = 0;
		// bZ[Ŵ́AFCW_STATUS, FCW_TOOLBAR ̂݁B
		switch(id)
		{
		case FCW_STATUS:
			lResult = ::SendMessage(theTablet->GetWindow(WINDOW_STATUSBAR), uMsg, wParam, lParam);
			break;
		default:
			return E_INVALIDARG;
		}
		if(pret) *pret = lResult;
		return S_OK;
	}
};
