// View.h
#pragma once

typedef CWinTraits<
	WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL,
	WS_EX_CLIENTEDGE>
	CViewTraits;

class CView:
	public CScrollWindowImpl<CView, CWindow, CViewTraits>,
	public CViewCommands<CView>
{
private:
	typedef CView thisClass;
	typedef CScrollWindowImpl<CView, CWindow, CViewTraits> baseClass;

private:
	static HACCEL s_hAccelTable;
	TCHAR m_FileName[MAX_PATH];
	CPicture m_Picture;
	CIcon m_Icon;

public:
	DECLARE_WND_CLASS_EX(_T("ImageView"), 0, -1)

	static void Init()
	{
		s_hAccelTable = AtlLoadAccelerators(IDR_VIEW_ACCEL);
	}

	static void Term()
	{
		::DestroyAcceleratorTable(s_hAccelTable);
		s_hAccelTable = NULL;
	}

	CView(LPCTSTR filename, IPicture* pPixture) : m_Picture(pPixture)
	{
		lstrcpy(m_FileName, filename);
		SHFILEINFO file;
		SHGetFileInfo(filename, 0, &file, sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_LARGEICON | SHGFI_SMALLICON);
		m_Icon.Attach(file.hIcon);
	}

	void OnFinalMessage(HWND)
	{
		delete this;
	}

	////////////////////////////////////////////////////////////////////////////////
	// CViewCommands

	BOOL PreTranslateMessage(MSG* pMsg)
	{
//		if(baseClass::PreTranslateMessage(pMsg))
//			return TRUE;
		return ::TranslateAccelerator(m_hWnd, s_hAccelTable, pMsg);
	}

	HICON DoGetIcon(UINT type)
	{
		return m_Icon;
	}

	BOOL DoGetNames(LPTSTR szShortName, LPTSTR szLongName)
	{
		if(szShortName)
			lstrcpy(szShortName, PathFindFileName(m_FileName));
		if(szLongName)
			lstrcpy(szLongName, m_FileName);
		return TRUE;
	}

	////////////////////////////////////////////////////////////////////////////////
	// Message map

	BEGIN_MSG_MAP(CView)
		MESSAGE_HANDLER(WM_CREATE		, OnCreate)
		MESSAGE_HANDLER(WM_ERASEBKGND	, OnEraseBkgnd)
		CHAIN_MSG_MAP(CViewCommands<CView>)
		CHAIN_MSG_MAP(baseClass)
		DEFAULT_REFLECTION_HANDLER()
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		__super::OnCreate(uMsg, wParam, lParam, bHandled);
		bHandled = TRUE;
		SetScrollSize(m_Picture.Size());
		SetScrollOffset(0, 0, FALSE);
		return 0;
	}

	LRESULT OnEraseBkgnd(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		return 1;
	}

	void DoPaint(CDCHandle dc)
	{
		CRect rect;
		GetClientRect(&rect);
		if(m_Picture)
		{
			CRect rcRight (m_Picture.Width(), 0, rect.right, rect.bottom);
			CRect rcBottom(0, m_Picture.Height(), m_Picture.Width(), rect.bottom);
			dc.FillRect(&rcRight , ::GetSysColorBrush(COLOR_APPWORKSPACE));
			dc.FillRect(&rcBottom, ::GetSysColorBrush(COLOR_APPWORKSPACE));
			m_Picture.Draw(dc, CPoint(0, 0));
		}
		else
		{
			dc.FillRect(&rect, ::GetSysColorBrush(COLOR_APPWORKSPACE));
		}
	}
};
