// View.h
#pragma once

typedef CWinTraits<
	WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL,
	WS_EX_CLIENTEDGE>
	CViewTraits;

class CView :
	public CWindowImpl<CView, CAxWindow, CViewTraits>,
	public CViewCommands<CView>
{
private:
	typedef CView thisClass;
	typedef CWindowImpl<CView, CAxWindow, CViewTraits> baseClass;

private:
	static HACCEL s_hAccelTable;

public:
	DECLARE_WND_SUPERCLASS(NULL, CAxWindow::GetWndClassName())

	static void Init()
	{
//		s_hAccelTable = AtlLoadAccelerators(IDR_VIEW_ACCEL);
	}

	static void Term()
	{
//		::DestroyAcceleratorTable(s_hAccelTable);
//		s_hAccelTable = NULL;
	}

	void OnFinalMessage(HWND)
	{
		delete this;
	}

	////////////////////////////////////////////////////////////////////////////////
	// CViewCommands

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		if((pMsg->message < WM_KEYFIRST || pMsg->message > WM_KEYLAST) &&
		   (pMsg->message < WM_MOUSEFIRST || pMsg->message > WM_MOUSELAST))
			return FALSE;

		// give HTML page a chance to translate this message
		return (BOOL)SendMessage(WM_FORWARDMSG, 0, (LPARAM)pMsg);
//		return ::TranslateAccelerator(m_hWnd, s_hAccelTable, pMsg);
	}

//	HICON DoGetIcon(UINT type)
//	{
//		return m_Icon;
//	}

//	BOOL DoGetNames(LPTSTR szShortName, LPTSTR szLongName)
//	{
//		if(szShortName)
//			lstrcpy(szShortName, PathFindFileName(m_FileName));
//		if(szLongName)
//			lstrcpy(szLongName, m_FileName);
//		return TRUE;
//	}

	////////////////////////////////////////////////////////////////////////////////
	// Message map

	BEGIN_MSG_MAP(CView)
		CHAIN_MSG_MAP(CViewCommands<CView>)
		DEFAULT_REFLECTION_HANDLER()
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
};

const LPCTSTR NEW_NAME[] =
{
	_T("URL [Browser]"),
};

const LPCTSTR FILTER_NAME[] =
{
	_T("HTMLt@C [Browser]|*.htm;*.html|"),
};

const LPCTSTR SUITABLE_EXTENSIONS =
{
	_T("*.htm;*.html")
};

class CDocument : public CDocumentImpl<CDocument>
{
public:
	void OnFinalMessage(HWND)
	{
		delete this;
	}

	////////////////////////////////////////////////////////////////////////////////
	// CDocumentCommands

	BOOL DoNew(UINT what, WindowID parent)
	{
		HWND hView = (new CView)->Create(theTablet->GetWindow(parent), rcDefault, _T("http://www.microsoft.com"));
		return hView != NULL;
	}
	INT  DoQuerySuitable(const FileEntry* entry)
	{
		if(entry->IsFile())
		{
			if(PathMatchSpec(entry->path, SUITABLE_EXTENSIONS))
				return SUITABLE_HIGH;
			else
				return SUITABLE_LOW;
		}
		else
			return SUITABLE_NONE;
	}
	BOOL DoOpen(const FileEntry* entry, WindowID parent)
	{
		HWND hView = (new CView())->Create(theTablet->GetWindow(parent), rcDefault, entry->path);
		return hView != NULL;
	}
	UINT DoGetNewCount()
	{
		return sizeof(NEW_NAME) / sizeof(NEW_NAME[0]);
	}
	BOOL DoGetNewName(UINT what, LPTSTR buffer)
	{
		lstrcpyn(buffer, NEW_NAME[what], MAX_PATH);
		return TRUE;
	}
	UINT DoGetFilterCount()
	{
		return sizeof(FILTER_NAME) / sizeof(FILTER_NAME[0]);
	}
	BOOL DoGetFilterName(UINT what, LPTSTR buffer)
	{
		lstrcpyn(buffer, FILTER_NAME[what], MAX_PATH);
		return TRUE;
	}
};

////////////////////////////////////////////////////////////////////////////////

HACCEL CView::s_hAccelTable;

BOOL TabletInitialize(ITablet* pTablet)
{
	theTablet = pTablet;
	CView::Init();

	(new CDocument)->Create(pTablet->GetWindow(WINDOW_DOCUMENT));

	return TRUE;
}

void TabletTerminate()
{
	CView::Term();
}
