// ReBar.h
#pragma once

class CReBarBandInfo : public REBARBANDINFO
{
public:
	CReBarBandInfo(HWND hWndBand, LPTSTR title)
	{
		// Get number of buttons on the toolbar
		int nBtnCount = (int)::SendMessage(hWndBand, TB_BUTTONCOUNT, 0, 0L);

		// Set band info structure
		this->cbSize = sizeof(REBARBANDINFO);
		this->fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE | RBBIM_IDEALSIZE;
		if(title != NULL)
			this->fMask |= RBBIM_TEXT;
		this->fStyle = RBBS_BREAK;
		if(nBtnCount > 0)
			this->fStyle |= RBBS_USECHEVRON;
		this->lpText = title;
		this->hwndChild = hWndBand;
		this->wID = ::GetDlgCtrlID(hWndBand);

		// Calculate the size of the band
		BOOL bRet;
		RECT rcTmp;
		if(nBtnCount > 0)
		{
			bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, nBtnCount - 1, (LPARAM)&rcTmp);
			ATLASSERT(bRet);
			this->cx = rcTmp.right;
			this->cyMinChild = rcTmp.bottom - rcTmp.top;
			if(title == 0)
			{
				bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, 0, (LPARAM)&rcTmp);
				ATLASSERT(bRet);
				this->cxMinChild = rcTmp.right;
			}
			else
			{
				this->cxMinChild = 0;
			}
		}
		else	// no buttons, either not a toolbar or really has no buttons
		{
			bRet = ::GetWindowRect(hWndBand, &rcTmp);
			ATLASSERT(bRet);
			this->cx = rcTmp.right - rcTmp.left;
			this->cxMinChild = 0;
			this->cyMinChild = rcTmp.bottom - rcTmp.top;
		}
		this->cxIdeal = rcTmp.right;
		DWORD dwExStyle = (DWORD)::SendMessage(hWndBand, TB_GETEXTENDEDSTYLE, 0, 0L);
		::SendMessage(hWndBand, TB_SETEXTENDEDSTYLE, 0, dwExStyle | TBSTYLE_EX_HIDECLIPPEDBUTTONS);
	}
};

class CReBar : public CReBarCtrl
{
public:
	bool IsBandVisible(UINT nID) const
	{
		REBARBANDINFO info = { sizeof(REBARBANDINFO), RBBIM_STYLE };
		GetBandInfo(IdToIndex(nID), &info);
		return (info.fStyle & RBBS_HIDDEN) == 0;
	}
	bool SetBandVisible(UINT nID, bool visible)
	{
		REBARBANDINFO info = { sizeof(REBARBANDINFO), RBBIM_STYLE };
		ShowBand(IdToIndex(nID), visible);
		return visible;
	}

	HWND Create(HWND hWndParent, WORD wID, DWORD dwStyle = ATL_SIMPLE_REBAR_NOBORDER_STYLE)
	{
		// Ensure style combinations for proper rebar painting
		if(dwStyle & CCS_NODIVIDER && dwStyle & WS_BORDER)
			dwStyle &= ~WS_BORDER;
		else if(!(dwStyle & WS_BORDER) && !(dwStyle & CCS_NODIVIDER))
			dwStyle |= CCS_NODIVIDER;

		// Create rebar window
		HWND hWndReBar = __super::Create(hWndParent, NULL, _T("WINDOW_REBAR"), dwStyle, 0, wID);
		if(hWndReBar == NULL)
		{
			ATLTRACE2(atlTraceUI, 0, _T("Failed to create rebar.\n"));
			return NULL;
		}

		// Initialize and send the REBARINFO structure
		REBARINFO rbi;
		rbi.cbSize = sizeof(REBARINFO);
		rbi.fMask  = 0;
		if(!::SendMessage(hWndReBar, RB_SETBARINFO, 0, (LPARAM)&rbi))
		{
			ATLTRACE2(atlTraceUI, 0, _T("Failed to initialize rebar.\n"));
			::DestroyWindow(hWndReBar);
			return NULL;
		}

		return hWndReBar;
	}
};
