// TabbedPane.h
#pragma once

#include "mew/CustomTabCtrl.h"
#include "mew/DotNetTabCtrl.h"
#include "mew/TabbedContainer.h"

typedef CWinTraits<WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN> CTabbedPaneTraits;
typedef CWinTraits<WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CTCS_MULTISELECT | CTCS_BOLDSELECTEDTAB | CTCS_TOOLTIPS | CTCS_SCROLL | CTCS_CLOSEBUTTON, 0> CTabbedPaneTabTraits;
typedef CDotNetTabCtrl<>	CTabbedPaneTabCtrl;

class ATL_NO_VTABLE CTabbedPane :
	public CTabbedContainerImpl<CTabbedPane, CTabbedPaneTabCtrl, CWindow, CTabbedPaneTraits >,
	public CPaneCommands<CTabbedPane>
{
	typedef CTabbedContainerImpl<CTabbedPane, CTabbedPaneTabCtrl, CWindow, CTabbedPaneTraits >	baseClass;
	typedef CPaneCommands<CTabbedPane>	baseCommands;
public:
	CTabbedPane()
	{
	}

public: // I[o[Ch
	BOOL PreTranslateMessage(MSG* pMsg)
	{
		if(m_hWndActive && (m_hWndActive == pMsg->hwnd || ::IsChild(m_hWndActive, pMsg->hwnd)))
			return ::SendMessage(m_hWndActive, WM_TABLET_PRETRANSLATE, 0, (LPARAM)pMsg);
		else
			return FALSE;
	}

	UINT DoCountElements()
	{
		return GetTabCtrl().GetItemCount();
	}

	HWND DoGetElement(UINT index)
	{
		if(index == -1)
			index = GetActiveTab();
		if(index == -1)
			index = 0;
		if(index < CountElements())
		{
			ItemType* item = GetItem(index);
			if(item)
				return item->GetTabView();
		}
		return NULL;
	}

	INT  DoGetActiveIndex()
	{
		return GetActiveTab();
	}

	INT  DoSetActiveIndex(INT index)
	{
		INT nCount = DoCountElements();
		if(nCount == 0)
			return -1;
		INT i = (index >= 0) ? (index % nCount) : (nCount - (-index) % nCount);
		SetCurSel(i);
		return GetActiveTab();
	}

	virtual BOOL DoAddElement(CCommonWindow view, UINT index) = 0;
	virtual BOOL DoMoveElement(CCommonWindow view, UINT index) = 0;
	virtual BOOL DoDeleteElement(UINT index) = 0;
	virtual void OnNameChanged(CCommonWindow view) = 0;

	virtual void OnTabMouse(const NMCTCMOUSE& mouse, CCommonWindow view, CPoint ptScreen) = 0;

public: // bZ[W}bv
	BEGIN_MSG_MAP(CTabbedPane)
		MESSAGE_HANDLER(WM_PARENTNOTIFY	, OnParentNotify)
		MESSAGE_HANDLER(WM_COMMAND		, OnCommand)
		NOTIFY_CODE_HANDLER(CTCN_MOUSE	, OnTabButton);
		CHAIN_MSG_MAP(baseCommands)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCommand(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if(m_hWndActive)
		{
			::SendMessage(m_hWndActive, uMsg, wParam, lParam);
			bHandled = TRUE;
		}
		else
		{
			bHandled = FALSE;
		}
		return bHandled;
	}

	LRESULT OnParentNotify(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
	{
		UINT what = LOWORD(wParam);
		if(what == WM_CREATE)
		{
			HWND hChild = (HWND)lParam;
			if(::GetParent(hChild) == m_hWnd && hChild != GetTabCtrl())
				DoAddElement(hChild, -1);
		}
		else if(what == WM_DESTROY)
		{	// WM_DESTROY ͐eNXŏ
			int index = GetTab((HWND)lParam);
			if(index >= 0)
				DoDeleteElement(index);
		}
		return 0;
	}

	LRESULT OnTabButton(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{
		NMCTCMOUSE* mouse = (NMCTCMOUSE*)pnmh;
		CPoint ptScreen = mouse->hit.pt;
		GetTabCtrl().ClientToScreen(&ptScreen);
		if(mouse->iItem >= 0)
		{
			HWND hView = GetElement(mouse->iItem);
			if(!::SendMessage(hView, WM_TABLET_TABMOUSE, mouse->uMsg, MAKELPARAM(ptScreen.x, ptScreen.y)))
				this->OnTabMouse(*mouse, hView, ptScreen);
		}
		else
		{
			this->OnTabMouse(*mouse, NULL, ptScreen);
		}
		return 0;
	}
};
