// Main.cpp

#include "stdafx.h"

#include "Tablet.h"
using namespace Tablet;

ITablet* theTablet;

#include "resource.h"
#include "View.h"
#include <atlfile.h>

const LPCTSTR NEW_NAME[] =
{
	_T("eLXgt@C (.txt) [TextEdit]"),
};

const LPCTSTR FILTER_NAME[] =
{
	_T("eLXgt@C [TextEdit]|*.*|"),
};

const LPCTSTR SUIT_EXTENSIONS =
{
	_T("*.txt;*.c;*.cpp;*.h;*.hpp")
};

class CFactory : public CFactoryImpl<CFactory>
{
public:
	void OnFinalMessage(HWND)
	{
		delete this;
	}

	////////////////////////////////////////////////////////////////////////////////
	// CFactoryCommands

	BOOL DoNew(UINT what, WindowID parent)
	{
		HWND hView = (new CView)->Create(theTablet->GetWindow(parent));
		return hView != NULL;
	}
	INT  DoQuerySuitable(const FileNode* entry)
	{
		if(entry->IsFile())
		{
			if(PathMatchSpec(entry->path, SUIT_EXTENSIONS))
				return SUIT_HIGH;
			else
				return SUIT_LOW;
		}
		else
			return SUIT_NONE;
	}
	BOOL DoOpen(const FileNode* entry, WindowID parent)
	{
		CAtlFile file;
		if FAILED(file.Create(entry->path, GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING))
		{
			return FALSE;
		}
		ULONGLONG size = 0;
		file.GetSize(size);
		CHeapPtr<CHAR> buffer;
		DWORD bufsize = (DWORD)size+1;
		buffer.Allocate(bufsize);
		file.Read(buffer, bufsize);
		buffer[size] = '\0';
		HWND hView = (new CView(entry->path))->Create(theTablet->GetWindow(parent));
		if(hView)
			::SetWindowText(hView, CA2CT(buffer));
		return hView != NULL;
	}
	UINT DoGetNewCount()
	{
		return sizeof(NEW_NAME) / sizeof(NEW_NAME[0]);
	}
	BOOL DoGetNewName(UINT what, LPTSTR buffer)
	{
		lstrcpyn(buffer, NEW_NAME[what], MAX_PATH);
		return TRUE;
	}
	UINT DoGetFilterCount()
	{
		return sizeof(FILTER_NAME) / sizeof(FILTER_NAME[0]);
	}
	BOOL DoGetFilterName(UINT what, LPTSTR buffer)
	{
		lstrcpyn(buffer, FILTER_NAME[what], MAX_PATH);
		return TRUE;
	}
};

////////////////////////////////////////////////////////////////////////////////

HACCEL CView::s_hAccelTable;

extern "C" __declspec(dllexport) BOOL TabletInitialize(ITablet* pTablet)
{
	theTablet = pTablet;
	CView::Init();

	(new CFactory)->Create(pTablet->GetWindow(WINDOW_FACTORY));

	return TRUE;
}

extern "C" __declspec(dllexport) void TabletTerminate()
{
	CView::Term();
}

BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID)
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
		break;
	case DLL_PROCESS_DETACH:
		break;
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	}
    return TRUE;
}
