// View.h
#pragma once

typedef CWinTraits<
	WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | ES_NOHIDESEL,
	WS_EX_CLIENTEDGE>
	CViewTraits;

#if 0

#include "WTL/EditImpl.h"

class CView : public CViewImpl<CView, CEditEx, CViewTraits>
{
private:
	typedef CView thisClass;
	typedef CViewImpl<CView, CEditEx, CViewTraits>	baseClass;

private:
	static HACCEL s_hAccelTable;

	TCHAR m_FileName[MAX_PATH];
	CIcon m_Icon;
	CFont m_font;
	CAtlArray<CString>	m_Strings;

public:
	DECLARE_WND_CLASS(_T("TextView"))

	static void Init()
	{
		s_hAccelTable = AtlLoadAccelerators(IDR_VIEW_ACCEL);
	}

	static void Term()
	{
		::DestroyAcceleratorTable(s_hAccelTable);
		s_hAccelTable = NULL;
	}

	CView()
	{
		lstrcpy(m_FileName, _T("Untilted.txt"));
		SHFILEINFO file;
		SHGetFileInfo(_T("*.txt"), 0, &file, sizeof(SHFILEINFO), SHGFI_USEFILEATTRIBUTES | SHGFI_ICON | SHGFI_LARGEICON | SHGFI_SMALLICON);
		m_Icon.Attach(file.hIcon);
	}

	CView(LPCTSTR filename)
	{
		lstrcpy(m_FileName, filename);
		SHFILEINFO file;
		SHGetFileInfo(filename, 0, &file, sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_LARGEICON | SHGFI_SMALLICON);
		m_Icon.Attach(file.hIcon);
	}

	void OnFinalMessage(HWND)
	{
		delete this;
	}

/*	void SetTabChars(UINT n)
	{
		TEXTMETRIC tm = {0};
		CDCHandle dc = GetDC();
		CFont fontOld = dc.SelectFont(m_font);
		dc.GetTextMetrics(&tm);
		dc.SelectFont(fontOld);
		ReleaseDC(dc);

		int nDialogUnitsX = n * (tm.tmAveCharWidth / LOWORD(GetDialogBaseUnits()));
		int nTabStops = n * nDialogUnitsX;
		SetTabStops(nTabStops);
	}
*/
	////////////////////////////////////////////////////////////////////////////////
	// CViewCommands

	BOOL PreTranslateMessage(MSG* pMsg)
	{
//		if(baseClass::PreTranslateMessage(pMsg))
//			return TRUE;
		return ::TranslateAccelerator(m_hWnd, s_hAccelTable, pMsg);
	}

	HICON DoGetIcon(UINT type)
	{
		return m_Icon;
	}

	BOOL DoGetNames(LPTSTR szShortName, LPTSTR szLongName)
	{
		if(szShortName)
			lstrcpy(szShortName, PathFindFileName(m_FileName));
		if(szLongName)
			lstrcpy(szLongName, m_FileName);
		return TRUE;
	}

	////////////////////////////////////////////////////////////////////////////////
	// Message map

	BEGIN_MSG_MAP(CView)
		MESSAGE_HANDLER(WM_CREATE	, OnCreate)
		MESSAGE_HANDLER(WM_PAINT	, OnPaint)
		MESSAGE_HANDLER(WM_CHAR		, OnChar)
//		CHAIN_MSG_MAP_ALT(CEditCommands<CView>, 1)
		CHAIN_MSG_MAP(baseClass)
		DEFAULT_REFLECTION_HANDLER()
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		// "base::OnCreate()"
		LRESULT lRet = DefWindowProc(uMsg, wParam, lParam);

		// "OnInitialUpdate"
		m_font = AtlGetStockFont(OEM_FIXED_FONT);
		SetFont(m_font);
//		SetTabChars(4);
		m_Strings.Add(_T(""));

		return lRet;
	}

	LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		CPaintDC dc(m_hWnd);
		CFontHandle fontOld = dc.SelectFont(m_font);
		for(size_t i = 0; i < m_Strings.GetCount(); i++)
		{
			dc.TextOut(10, 10+i*20, m_Strings[i]);
		}
		CString& last = m_Strings[m_Strings.GetCount()-1];
		CPoint pt(10+last.GetLength()*8, 10+m_Strings.GetCount()*20-20);
		dc.MoveTo(pt);
		dc.LineTo(pt.x, pt.y+18);
		dc.SelectFont(fontOld);
		return 0;
	}

	LRESULT OnChar(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		CString& last = m_Strings[m_Strings.GetCount()-1];

		switch(wParam)
		{
		case _T('\r'):
			m_Strings.Add(_T(""));
			SetCursorPosition(CPoint(10, 10+m_Strings.GetCount()*20-20));
			break;
		default:
			last.AppendChar((TCHAR)wParam);
			SetCursorPosition(CPoint(10+last.GetLength()*8, 10+m_Strings.GetCount()*20-20));
			break;
		}
		Invalidate();
		return 0;
	}
};

#else

class CView : public CViewImpl<CView, CEdit, CViewTraits>, public CEditCommands<CView>
{
private:
	typedef CView thisClass;
	typedef CViewImpl<CView, CEdit, CViewTraits> baseClass;

private:
	static HACCEL s_hAccelTable;
	TCHAR m_FileName[MAX_PATH];
	CIcon m_Icon;
	CFont m_font;

public:
	DECLARE_WND_SUPERCLASS(_T("TextView"), CEdit::GetWndClassName())

	static void Init()
	{
		s_hAccelTable = AtlLoadAccelerators(IDR_VIEW_ACCEL);
	}

	static void Term()
	{
		::DestroyAcceleratorTable(s_hAccelTable);
		s_hAccelTable = NULL;
	}

	CView()
	{
		lstrcpy(m_FileName, _T("Untilted.txt"));
		SHFILEINFO file;
		SHGetFileInfo(_T("*.txt"), 0, &file, sizeof(SHFILEINFO), SHGFI_USEFILEATTRIBUTES | SHGFI_ICON | SHGFI_LARGEICON | SHGFI_SMALLICON);
		m_Icon.Attach(file.hIcon);
	}

	CView(LPCTSTR filename)
	{
		lstrcpy(m_FileName, filename);
		SHFILEINFO file;
		SHGetFileInfo(filename, 0, &file, sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_LARGEICON | SHGFI_SMALLICON);
		m_Icon.Attach(file.hIcon);
	}

	void OnFinalMessage(HWND)
	{
		delete this;
	}

	void SetTabChars(UINT n)
	{
		TEXTMETRIC tm = {0};
		CDCHandle dc = GetDC();
		CFont fontOld = dc.SelectFont(m_font);
		dc.GetTextMetrics(&tm);
		dc.SelectFont(fontOld);
		ReleaseDC(dc);

		int nDialogUnitsX = n * (tm.tmAveCharWidth / LOWORD(GetDialogBaseUnits()));
		int nTabStops = n * nDialogUnitsX;
		SetTabStops(nTabStops);
	}

	////////////////////////////////////////////////////////////////////////////////
	// CViewCommands

	BOOL PreTranslateMessage(MSG* pMsg)
	{
//		if(baseClass::PreTranslateMessage(pMsg))
//			return TRUE;
		return ::TranslateAccelerator(m_hWnd, s_hAccelTable, pMsg);
	}

	HICON DoGetIcon(UINT type)
	{
		return m_Icon;
	}

	BOOL DoGetNames(LPTSTR szShortName, LPTSTR szLongName)
	{
		if(szShortName)
			lstrcpy(szShortName, PathFindFileName(m_FileName));
		if(szLongName)
			lstrcpy(szLongName, m_FileName);
		return TRUE;
	}

	////////////////////////////////////////////////////////////////////////////////
	// Message map

	BEGIN_MSG_MAP(CView)
		MESSAGE_HANDLER(WM_CREATE		, OnCreate)
		CHAIN_MSG_MAP_ALT(CEditCommands<CView>, 1)
		CHAIN_MSG_MAP(baseClass)
		DEFAULT_REFLECTION_HANDLER()
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		// "base::OnCreate()"
		LRESULT lRet = DefWindowProc(uMsg, wParam, lParam);

		// "OnInitialUpdate"
		m_font = AtlGetStockFont(OEM_FIXED_FONT);
		SetFont(m_font);
		SetTabChars(4);

		return lRet;
	}
};

#endif
