// SideBar.h
#pragma once

#include "mew/FileTreeCtrl.h"

typedef CWinTraitsOR < WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, WS_EX_CLIENTEDGE> CSideBarTraits;

class CSideBar :
	public CFileTreeCtrlImpl<CSideBar, CFileTreeCtrlItem, CTreeViewCtrl, CSideBarTraits>,
	public CViewCommands<CSideBar>
{
	typedef CFileTreeCtrlImpl<CSideBar, CFileTreeCtrlItem, CTreeViewCtrl, CSideBarTraits>	baseClass;

public:
	DECLARE_WND_SUPERCLASS(_T("FavoriteBar"), GetWndClassName())

	static LPCTSTR GetWndCaption()
	{
		return _T("Cɓ");
	}

	void OnFinalMessage(HWND)
	{
		delete this;
	}

public: // CFileTreeCtrlImpl I[o[Ch
	struct CreateNewView
	{
		WindowID Parent;

		CreateNewView(WindowID parent) : Parent(parent) {}
		bool operator () (CSideBar* pThis, HTREEITEM hItem)
		{
			TreeItem item(TVIF_CHILDREN | TVIF_HANDLE | TVIF_PARAM);
			item.hItem = hItem;
			TCHAR path[MAX_PATH];
			if(pThis->GetItem(&item) && !item.cChildren && item.pParam && item.pParam->pShellItem->GetPath(path))
				CView::New(Parent, path);
			return true;
		}
	};

	UINT PopupContextMenu(HTREEITEM hItem, CMenuHandle popup, CPoint ptScreen)
	{
		popup.InsertMenu(0, MF_BYPOSITION | MF_SEPARATOR);
		const UINT COMMAND_FIRST = 0x8000;
		UINT nPanes = theTablet->CountElements(WINDOW_PANESPLITTER);
		for(UINT i = 0; i < nPanes; i++)
		{
			TCHAR buffer[128];
			wsprintf(buffer, _T(" &%d yCŊJ"), i+1);
			popup.InsertMenu(i, MF_BYPOSITION, COMMAND_FIRST + i, buffer);
		}
		if(nPanes < MAX_PANE_COUNT)
			popup.InsertMenu(nPanes, MF_BYPOSITION, COMMAND_FIRST+nPanes, _T("VyCŊJ(&0)"));
		UINT cmd = theTablet->TrackPopupMenu(popup, ptScreen, m_hWnd);
		if(COMMAND_FIRST <= cmd && cmd <= COMMAND_FIRST+nPanes)
		{
			TCHAR path[MAX_PATH];
			TreeItem item(TVIF_CHILDREN | TVIF_HANDLE | TVIF_PARAM);
			item.hItem = hItem;
			if(GetItem(&item) && item.pParam->pShellItem->GetPath(path))
			{
				WindowID parent = (WindowID)(WINDOW_PANE_FIRST + cmd - COMMAND_FIRST);
				if(item.cChildren)
				{
					LoadChildNodes(hItem);
					ForEachChildren(hItem, CreateNewView(parent));
				}
				else
					CView::New(parent, path);
			}
		}
		return cmd;
	}

	void OnItemReturn(HTREEITEM hItem)
	{
		ProcessSelect(hItem);
	}

public: // CViewCommands I[o[Ch
	BOOL PreTranslateMessage(MSG* pMsg)
	{
		if(baseClass::PreTranslateMessage(pMsg))
			return TRUE;
		return FALSE;
	}

	HICON DoGetIcon(UINT type)
	{
		return AtlLoadIconImage(IDR_MAINFRAME, LR_DEFAULTCOLOR | LR_DEFAULTSIZE | LR_SHARED);
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CSideBar)
		MESSAGE_HANDLER(WM_CREATE	, OnCreate)
		REFLECTED_NOTIFY_CODE_HANDLER(NM_DBLCLK			, OnItemSelect)
		REFLECTED_NOTIFY_CODE_HANDLER(TVN_SELCHANGED	, OnSelChanged)
		CHAIN_MSG_MAP(CViewCommands<CSideBar>)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		__super::OnCreate(uMsg, wParam, lParam, bHandled);
		bHandled = TRUE;
		CAutoPtr<CShellItemTask> pRoot = CShellItemTask::FromCSIDL(CSIDL_FAVORITES, m_hWnd);
		if(!pRoot)
			return -1;
		SetFileEnumFlag(FileEnumFlag_IncludeFolders | FileEnumFlag_IncludeFiles | FileEnumFlag_SortFoldersOnTop);
		AddFolder(pRoot);
		return 0;
	}

	BOOL ProcessSelect(HTREEITEM hItem)
	{	// [tm[h̃NbN̏ꍇ́AsB
		TreeItem item(TVIF_CHILDREN | TVIF_HANDLE | TVIF_PARAM);
		item.hItem = hItem;
		TCHAR path[MAX_PATH];
		if(GetItem(&item)
			&& !item.cChildren
			&& item.pParam
			&& item.pParam->pShellItem->GetPath(path))
		{
			return CView::New(WINDOW_PANE_ACTIVE, path) != NULL;
		}
		else
			return FALSE;
	}

	LRESULT OnItemSelect(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
	{	// ^[܂̓_uNbN
		return ProcessSelect(GetSelectedItem());
	}

	LRESULT OnSelChanged(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{
		TreeNotify* nm = (TreeNotify*)pnmh;
		DWORD style = GetStyle();
		// }EXNbN̏ꍇ̂
		if((style & TVS_SINGLEEXPAND) && nm->action == TVC_BYMOUSE)
			return ProcessSelect(nm->itemNew.hItem);
		return FALSE;
	}
};
