// View.h
#pragma once

////////////////////////////////////////////////////////////////////////////////

typedef CWinTraits<	WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, WS_EX_CLIENTEDGE> CViewTraits;

__interface INavigateFilter
{
	bool FilterURL(LPCTSTR strURL);
	bool FilterTitle(LPCTSTR strTitle);
	bool FilterStatusText(LPCTSTR strText);
};

class CView :
	public CWebBrowserImpl<CView, 0, CWindow, CViewTraits>,
	public CViewCommands<CView>
{
	typedef CWebBrowserImpl<CView, 0, CWindow, CViewTraits>	baseClass;
	typedef CViewCommands<CView>	baseCommands;

private:
	static HACCEL s_hAccelTable;
	static INavigateFilter*	s_pFilter;

	CIcon	m_Icon;

private:
	CView()
	{
		// Ƃ肠B
		SHFILEINFO file;
		SHGetFileInfo(_T("*.html"), 0, &file, sizeof(SHFILEINFO), SHGFI_USEFILEATTRIBUTES | SHGFI_ICON | SHGFI_LARGEICON | SHGFI_SMALLICON);
		m_Icon.Attach(file.hIcon);
	}

public:
	DECLARE_WND_CLASS_EX(_T("HtmlView"), 0, -1)

	static void Init(INavigateFilter* pFilter)
	{
		s_hAccelTable = AtlLoadAccelerators(IDR_VIEW_ACCEL);
		s_pFilter = pFilter;
	}

	static void Term()
	{
		::DestroyAcceleratorTable(s_hAccelTable);
		s_hAccelTable = NULL;
		s_pFilter = NULL;
	}

	static CView* New(WindowID parent, LPCTSTR url)
	{
		CView* pView = NULL;
		if(url)
		{	// vtURLw肳ĂB
			TCHAR resolved[MAX_PATH];
			ResolveInternetShortcut(resolved, url);
			pView = new CView();
			if(pView->Create(theTablet->GetWindow(parent)))
			{
				theTablet->AddHistory(resolved);
				pView->Navigate(resolved);
				pView->NotifyNameChanged();
				return pView;
			}
		}
		else
		{	// URLw肳ĂȂBz[y[WJB
			pView = new CView();
			if(pView->Create(theTablet->GetWindow(parent)))
			{
				pView->GoHome();
				return pView;
			}
		}
		return NULL;
	}

// I[o[Ch
	void OnFinalMessage(HWND)
	{
		delete this;
	}

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		if(pMsg->message == WM_MBUTTONDOWN && (m_hWnd == pMsg->hwnd || IsChild(pMsg->hwnd)))
		{
			// from DonutP : N𒆃NbNŕʂ̃EBhEŊJB
			// ̏ԂŃbZ[WƗǂ炵B
			UINT nViews = theTablet->CountElements(WINDOW_PANE_ACTIVE);
			::SendMessage(pMsg->hwnd, WM_LBUTTONDOWN, 0, pMsg->lParam);
			::SendMessage(pMsg->hwnd, WM_COMMAND    , 2137, 0);
			::SendMessage(pMsg->hwnd, WM_LBUTTONUP  , 0, pMsg->lParam);
			if(nViews != theTablet->CountElements(WINDOW_PANE_ACTIVE))
			{	// r[̐ωVEBhEŊJꍇA
				// ȏ̏ȂB
				// ȂƁANbNꂽ܂܁iI[gXN[ԁjɂȂĂ܂B
				return TRUE;
			}
		}

		if(::TranslateAccelerator(m_hWnd, s_hAccelTable, pMsg))
			return TRUE;

		return baseClass::PreTranslateMessage(pMsg);
	}

	HICON DoGetIcon(UINT type)
	{
		return m_Icon;
	}

	BOOL DoGetNames(LPTSTR szShortName, LPTSTR szLongName)
	{
		if(szShortName)
			lstrcpyn(szShortName, this->Title, MAX_PATH);
		if(szLongName)
			lstrcpyn(szLongName, this->URL, MAX_PATH);
		return TRUE;
	}

	BOOL DoBeginGesture(UINT key, CPoint ptScreen)
	{
		// ɃWFX`󂯓B
		return TRUE;
	}

	void DoEndGesture(UINT length, const Gesture* gesture)
	{
		if(length == 1)
		{
			switch(gesture[0])
			{
			case GESTURE_NORTH:
				GoUp();
				break;
			case GESTURE_WEST:
				GoBack();
				break;
			case GESTURE_EAST:
				GoForward();
				break;
			case GESTURE_SOUTH:
				PostMessage(WM_CLOSE);
				break;
			}
		}
	}

	struct MenuInserter
	{
		HMENU	hMenu;
		UINT	cmd;
		UINT	index;
		void operator () (ITravelLogEntry* entry)
		{
			LPOLESTR title;
			if(SUCCEEDED(entry->GetTitle(&title)) && title)
			{
				::InsertMenu(hMenu, index, MF_BYPOSITION, cmd, COLE2CT(title));
				++index;
				++cmd;
				CoTaskMemFree(title);
			}
		}
	};

	BOOL QueryTravelLogToMenu(HMENU hMenu, INT num, UINT nCmdFirst, UINT nInsertIndex)
	{
		MenuInserter op = { hMenu, nCmdFirst, nInsertIndex };
		return SUCCEEDED(QueryTravelLog(op, num));
	}

	BOOL QueryUpMenu(HMENU hMenu, UINT nCmdFirst, UINT nInsertIndex)
	{
		// ƗpɕB
		TCHAR buffer[MAX_PATH];
		lstrcpy(buffer, this->URL);
		if(PathIsURL(buffer))
		{	// URL
			if(UrlIsIndex(buffer))
				UrlRemoveLeaf(buffer);
			while(UrlRemoveLeaf(buffer))
			{
				UrlAddSlash(buffer);
				::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst++, buffer);
			}
		}
		else
		{	// Windows Path
			bool isIndex = PathIsIndex(buffer);
			if(isIndex)
				PathRemoveFileSpec(buffer); // index.html ̏ꍇ͂Qi邽
			while(PathRemoveFileSpec(buffer))
			{
				::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst++, buffer);
			}
		}
		return TRUE;
	}

	BOOL QueryShowMenu(HMENU hMenu, UINT nCmdFirst, UINT nInsertIndex)
	{
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst++, _T("ŏ"));
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst++, _T(""  ));
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst++, _T(""));
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst++, _T(""  ));
		::InsertMenu(hMenu, nInsertIndex++, MF_BYPOSITION, nCmdFirst++, _T("ő"));
		return TRUE;
	}

	BOOL DoQueryMenu(HMENU hMenu, UINT cmd, UINT insert)
	{
		switch(cmd)
		{
		case ID_VIEW_BACK:
			return QueryTravelLogToMenu(hMenu, -16, cmd, insert);
		case ID_VIEW_FORWARD:
			return QueryTravelLogToMenu(hMenu, +16, cmd, insert);
		case ID_VIEW_UP:
			return QueryUpMenu(hMenu, cmd, insert);
		case ID_VIEW_SHOW:
			return QueryShowMenu(hMenu, cmd, insert);
		default:
			return FALSE;
		}
	}

	////////////////////////////////////////////////////////////////////////////////
	// Message map

	BEGIN_MSG_MAP(CView)
		MESSAGE_HANDLER(WM_DESTROY			, OnDestroy)
		MESSAGE_HANDLER(WM_TABLET_TABMOUSE	, OnTabMouse)
		COMMAND_CATEGORY_ID_HANDLER(ID_VIEW_FORWARD	, GoForward, +1)
		COMMAND_CATEGORY_ID_HANDLER(ID_VIEW_BACK	, GoBack   , +1)
		COMMAND_CATEGORY_ID_HANDLER(ID_VIEW_UP		, GoUp     , +1)
		CHAIN_MSG_MAP(baseCommands)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		CString url = this->URL;
		if(PathIsURL(url))
			theTablet->AddHistory(url);
		bHandled = FALSE;
		return 0;
	}

	LRESULT OnTabMouse(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
	{
		CPoint pt(lParam);
		switch(wParam)
		{
		case WM_LBUTTONDBLCLK:
			Refresh();
			break;
		}
		return FALSE;
	}

public: // I[o[Ch : CWebBrowserEvents
	void OnTitleChange(const CString& title)
	{
		if(!s_pFilter->FilterTitle(title))
		{
			ATLTRACE("CLOSE: FilterTitle(%s)\n", (LPCTSTR)title);
			PostMessage(WM_CLOSE);
			return;
		}
		NotifyNameChanged();
	}

	void OnStatusTextChange(const CString& text)
	{
		if(theTablet->GetWindow(WINDOW_VIEW_ACTIVE) != m_hWnd)
			return;
		if(!s_pFilter->FilterStatusText(text))
			return;
		CStatusBarCtrl statusbar = theTablet->GetWindow(WINDOW_STATUSBAR);
		statusbar.SetText(0, text, 0);
	}

	void OnProgressChange(LONG Progress, LONG ProgressMax)
	{
		if(theTablet->GetWindow(WINDOW_VIEW_ACTIVE) != m_hWnd)
			return;
		CStatusBarCtrl statusbar = theTablet->GetWindow(WINDOW_STATUSBAR);
		if(Progress >= 0 && ProgressMax > 0)
		{
			TCHAR text[128];
			wsprintf(text, _T("%d%%"), MulDiv(Progress, 100, ProgressMax));
			statusbar.SetText(1, text, 0);
		}
		else
		{
			statusbar.SetText(1, _T(""), 0);
		}
	}

	void OnNavigateComplete(IDispatch* pDisp, const CString& url)
	{
		NotifyNameChanged();
	}

	bool OnCancelNavigate(IDispatch* pDisp, BSTR bstrURL, BSTR targetFrameName, VARIANT* vtPostData, BSTR header)
	{
		CString url(bstrURL);

		if(!s_pFilter->FilterURL(url))
		{	// TODO: ֎~URLVKEBhEŊĴȂΎIɕ
			ATLTRACE("CLOSE: FilterURL(%s)\n", (LPCTSTR)url);
			return true;
		}
/*
		bool isEqualBrowser = IsEqualBrowser(pDisp);

		if(StrCmpNI(url, _T("javascript"), 10) == 0)
		{	// javascript ŊJEBhÉAOŊJĂ܂B
			// ̂߁ALZAŊJȂB
			*Cancel = VARIANT_TRUE;
			ATLTRACE("javascript [%s] canceled\n", (LPCTSTR)url);
			return;
		}
*/
/*		// irQ[gbN:ׂĐVEBhEŊJ
		if(navigateLocked)
		{
			CView* view = new CView();
			view->Navigate(url);
			*Cancel = VARIANT_TRUE;
			return;
		}
*/
		return false;
	}

	void OnNewWindow(IWebBrowser2** ppNewBrowser)
	{
		// ō쐬ׂr[́ANavigateOłKv邽߁ANew()g킸ڍ쐬B
		CView* pView = new CView();
		CComPtr<IWebBrowser2> pBrowser;
		if(pView->Create(theTablet->GetWindow(WINDOW_PANE_ACTIVE)))
			pView->QueryWebBrowser(ppNewBrowser);
	}

/*
	void OnCommandStateChange(long Command, bool bEnable)
	{
		if (Command == CSC_NAVIGATEBACK) {
			m_bNavigateBack = bEnable ? TRUE : FALSE;
		}		
		else if (Command == CSC_NAVIGATEFORWARD) {
			m_bNavigateForward = bEnable ? TRUE : FALSE;
		}
		else if (Command == CSC_UPDATECOMMANDS) {
		}
	}
*/
};
