// TabletClass.h
#pragma once

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CPaneWindow
// CCommonWindow
// CFactoryWindow
// CCommonCommands<T>
// CViewCommands<T>
// CViewImpl<T, TBase, TWinTraits>
// CFactoryCommands<T>
// CFactoryImpl<T, TBase, TWinTraits>

namespace Tablet
{
	////////////////////////////////////////////////////////////////////////////////
	// NCAgpNX

	// WINDOW_FACTORY, WINDOW_SIDEBAR, WINDOW_PANESPLITTER, WINDOW_PANE_* ŕԂEBhE
	class CPaneWindow : public CWindow
	{
	public:
		CPaneWindow(HWND hWnd = NULL) : CWindow(hWnd)
		{
		}
		WindowID GetID() const
		{	// WINDOW_PANE_* ͊ԈIDԂ܂B
			return (WindowID)::GetWindowLong(m_hWnd, GWL_ID);
		}
		BOOL PreTranslateMessage(MSG* pMsg)
		{
			return (BOOL)SendMessage(WM_TABLET_PRETRANSLATE, 0, (LPARAM)pMsg);
		}
		UINT CountElements()
		{
			return (UINT)SendMessage(WM_TABLET_COUNTELEMENTS);
		}
		HWND GetElement(UINT index)
		{
			return (HWND)SendMessage(WM_TABLET_GETELEMENT, index);
		}
		HWND GetActiveElement()
		{
			return GetElement(-1);
		}
		BOOL AddElement(HWND hWnd, UINT index = -1)
		{
			return (BOOL)SendMessage(WM_TABLET_ADDELEMENT, (WPARAM)index, (LPARAM)hWnd);
		}
		BOOL MoveElement(HWND hWnd, UINT index)
		{
			return (BOOL)SendMessage(WM_TABLET_MOVEELEMENT, (WPARAM)index, (LPARAM)hWnd);
		}
		BOOL DeleteElement(UINT index)
		{
			return (BOOL)SendMessage(WM_TABLET_DELETEELEMENT, (WPARAM)index, 0);
		}
		INT GetActiveIndex()
		{
			return (INT)SendMessage(WM_TABLET_GETACTIVEINDEX, 0, 0);
		}
		INT SetActiveIndex(INT index)
		{
			return (INT)SendMessage(WM_TABLET_SETACTIVEINDEX, index, 0);
		}
	};

	// t@NgAr[ATCho[ʃEBhE
	class CCommonWindow : public CWindow
	{
	public:
		CCommonWindow(HWND hWnd = NULL) : CWindow(hWnd)
		{
		}
		BOOL GetNames(LPTSTR szShortName, LPTSTR szLongName)
		{
			return (BOOL)SendMessage(WM_TABLET_GETNAMES, (WPARAM)szShortName, (LPARAM)szLongName);
		}
		BOOL QueryMenu(HMENU hMenu, UINT cmd, UINT insert = 0)
		{
			return (BOOL)SendMessage(WM_TABLET_QUERYMENU, MAKEWPARAM(cmd, insert), (LPARAM)hMenu);
		}
	};

	// t@NgEEBhE
	class CFactoryWindow : public CCommonWindow
	{
	public:
		CFactoryWindow(HWND hWnd = NULL) : CCommonWindow(hWnd)
		{
		}
		UINT GetNewCount()
		{
			return (UINT)SendMessage(WM_TABLET_GETNEWCOUNT, 0, 0);
		}
		BOOL GetNewName(UINT what, LPTSTR buffer)
		{
			return (BOOL)SendMessage(WM_TABLET_GETNEWNAME, (WPARAM)what, (LPARAM)buffer);
		}
		UINT GetFilterCount()
		{
			return (UINT)SendMessage(WM_TABLET_GETFILTERCOUNT, 0, 0);
		}
		BOOL GetFilterName(UINT what, LPTSTR buffer)
		{
			return (BOOL)SendMessage(WM_TABLET_GETFILTERNAME, (WPARAM)what, (LPARAM)buffer);
		}
		BOOL New(UINT what, WindowID parent = WINDOW_PANE_ACTIVE)
		{
			return (BOOL)SendMessage(WM_TABLET_NEW, (WPARAM)parent, (LPARAM)what);
		}
		INT QuerySuitable(const FileNode* node)
		{
			return (INT)SendMessage(WM_TABLET_QUERYSUITABLE, 0, (LPARAM)node);
		}
		BOOL Open(const FileNode* node, WindowID parent = WINDOW_PANE_ACTIVE)
		{
			return (BOOL)SendMessage(WM_TABLET_OPEN, (WPARAM)parent, (LPARAM)node);
		}
	};

	////////////////////////////////////////////////////////////////////////////////
	// T[opNX

	// ʃbZ[Wp
	template < class T > class ATL_NO_VTABLE CCommonCommands
	{
	public: // Iy[V
    	BOOL GetNames(LPTSTR szShortName, LPTSTR szLongName)	{ return DoGetNames(szShortName, szLongName); }
		BOOL QueryMenu(HMENU hMenu, UINT cmd, UINT insert = 0)	{ return DoQueryMenu(hMenu, cmd, insert); }
		void NotifyNameChanged()
		{
			T* pThis = static_cast<T*>(this);
			::PostMessage(pThis->GetParent(), WM_TABLET_NAMECHANGED, 0, (LPARAM)pThis->m_hWnd);
		}

	public: // I[o[Ch
		BOOL PreTranslateMessage(MSG* pMsg)
		{
			return FALSE;
		}
		HICON DoGetIcon(UINT type)
		{
			T* pThis = static_cast<T*>(this);
			return (HICON)(LONG_PTR)::GetClassLongPtr(pThis->m_hWnd, type == ICON_BIG ? GCL_HICON : GCL_HICONSM);
		}
		BOOL DoGetNames(LPTSTR szShortName, LPTSTR szLongName)
		{
			T* pThis = static_cast<T*>(this);
			if(szShortName)
				pThis->GetWindowText(szShortName, MAX_PATH);
			if(szLongName)
				pThis->GetWindowText(szLongName, MAX_PATH);
			return TRUE;
		}
		BOOL DoQueryMenu(HMENU hMenu, UINT cmd, UINT insert)
		{
			return FALSE;
		}

	public: // bZ[W}bv
		BEGIN_MSG_MAP(CCommonCommands)
			MESSAGE_HANDLER(WM_GETICON				, OnGetIcon)
			MESSAGE_HANDLER(WM_TABLET_PRETRANSLATE	, OnPreTranslateMessage)
			MESSAGE_HANDLER(WM_TABLET_GETNAMES		, OnGetNames)
			MESSAGE_HANDLER(WM_TABLET_QUERYMENU		, OnQueryMenu)
			COMMAND_ID_HANDLER(ID_FILE_CLOSE		, OnFileClose)
		END_MSG_MAP()

		LRESULT OnGetIcon(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)(HICON)pThis->DoGetIcon((UINT)wParam);
		}
		LRESULT OnPreTranslateMessage(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->PreTranslateMessage((MSG*)lParam);
		}
		LRESULT OnGetNames(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoGetNames((LPTSTR)wParam, (LPTSTR)lParam);
		}
		LRESULT OnQueryMenu(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoQueryMenu((HMENU)lParam, LOWORD(wParam), HIWORD(lParam));
		}
		// BOOL OnQueryMenu(HMENU hMenu = lParam, UINT cmd = LOWORD(wParam), UINT insert = HIWORD(wParam))
		LRESULT OnFileClose(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			pThis->PostMessage(WM_CLOSE);
			return 0;
		}
	};

	// r[bZ[Wp
	template < class T > class ATL_NO_VTABLE CViewCommands : public CCommonCommands<T>
	{
	public: // I[o[Ch
		BOOL DoBeginGesture(UINT key, CPoint ptScreen)
		{
			return FALSE;
		}

		void DoEndGesture(UINT length, const Gesture* gesture)
		{
		}

	public: // bZ[W}bv
		BEGIN_MSG_MAP(CViewCommands)
			MESSAGE_HANDLER(WM_TABLET_BEGINGESTURE	, OnBeginGesture)
			MESSAGE_HANDLER(WM_TABLET_ENDGESTURE	, OnEndGesture)
			CHAIN_MSG_MAP(__super)
		END_MSG_MAP()

		LRESULT OnBeginGesture(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoBeginGesture((UINT)wParam, CPoint(lParam));
		}

		LRESULT OnEndGesture(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			pThis->DoEndGesture((UINT)wParam, (const Gesture*)lParam);
			return 0;
		}
	};

	// Pr[p
	template < class T, class TBase = CWindow, class TWinTraits = CControlWinTraits >
	class ATL_NO_VTABLE CViewImpl : public CWindowImpl<T, TBase, TWinTraits>, public CViewCommands<T>
	{
	public:
		BEGIN_MSG_MAP(CViewImpl)
			CHAIN_MSG_MAP(CViewCommands<T>);
		END_MSG_MAP()
	};

	// t@NgbZ[Wp
	template < class T > class ATL_NO_VTABLE CFactoryCommands : public CCommonCommands<T>
	{
	public: // Iy[V
		UINT GetNewCount()								{ return static_cast<T*>(this)->DoGetNewCount(); }
		BOOL GetNewName(UINT what, LPTSTR buffer)		{ return static_cast<T*>(this)->DoNewName(what, buffer); }
		UINT GetFilterCount()							{ return static_cast<T*>(this)->DoGetFilterCount(); }
		BOOL GetFilterName(UINT what, LPTSTR buffer)	{ return static_cast<T*>(this)->DoGetFilterName(what, buffer); }
		BOOL New(UINT what, WindowID parent = WINDOW_PANE_ACTIVE)				{ return static_cast<T*>(this)->DoNew(what, parent); }
		INT  QuerySuitable(const FileNode* node)								{ return static_cast<T*>(this)->DoQuerySuitable(node); }
		BOOL Open(const FileNode* node, WindowID parent = WINDOW_PANE_ACTIVE)	{ return static_cast<T*>(this)->DoOpen(node, parent); }

	public: // I[o[Ch
	//	BOOL DoNew(UINT what, WindowID parent);
	//  INT  DoQuerySuitable(const FileNode* node);
	//	BOOL DoOpen(const FileNode* node, WindowID parent);
	//	UINT DoGetNewCount();
	//	BOOL DoGetNewName(UINT what, LPTSTR buffer);
	//	UINT DoGetFilterCount();
	//	BOOL DoGetFilterName(UINT what, LPTSTR buffer);

	public: // bZ[W}bv
		BEGIN_MSG_MAP(CFactoryCommands)
			MESSAGE_HANDLER(WM_TABLET_NEW			, OnNew)
			MESSAGE_HANDLER(WM_TABLET_QUERYSUITABLE	, OnQuerySuitable)
			MESSAGE_HANDLER(WM_TABLET_OPEN			, OnOpen)
			MESSAGE_HANDLER(WM_TABLET_GETNEWCOUNT	, OnGetNewCount)
			MESSAGE_HANDLER(WM_TABLET_GETNEWNAME	, OnGetNewName)
			MESSAGE_HANDLER(WM_TABLET_GETFILTERCOUNT, OnGetFilterCount)
			MESSAGE_HANDLER(WM_TABLET_GETFILTERNAME	, OnGetFilterName)
			CHAIN_MSG_MAP(__super)
		END_MSG_MAP()

		LRESULT OnNew(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoNew((UINT)lParam, (WindowID)wParam);
		}
		LRESULT OnQuerySuitable(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoQuerySuitable((const FileNode*)lParam);
		}
		LRESULT OnOpen(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoOpen((const FileNode*)lParam, (WindowID)wParam);
		}
		LRESULT OnGetNewCount(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoGetNewCount();
		}
		LRESULT OnGetNewName(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoGetNewName((UINT)wParam, (LPTSTR)lParam);
		}
		LRESULT OnGetFilterCount(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoGetFilterCount();
		}
		LRESULT OnGetFilterName(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
		{
			T* pThis = static_cast<T*>(this);
			return (LRESULT)pThis->DoGetFilterName((UINT)wParam, (LPTSTR)lParam);
		}
	};

	// Pt@Ngp
	template < class T, class TBase = CWindow, class TWinTraits = CControlWinTraits >
	class ATL_NO_VTABLE CFactoryImpl : public CWindowImpl<T, TBase, TWinTraits>, public CFactoryCommands<T>
	{
	public:
		BEGIN_MSG_MAP(CFactoryImpl)
			CHAIN_MSG_MAP(CFactoryCommands<T>);
		END_MSG_MAP()
	};
}
