// Allocator.h
#pragma once

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CGlobalLockAllocator	: GlobalAlloc(GMEM_MOVEABLE), GlobalLock() gpAP[^B
// CTaskAllocator		: CoTaskMemAlloc() gpAP[^BSHGetMalloc == CoGetMalloc 炵B

/////////////////////////////////////////////////////////////////////////////

// #include <atlcore.h>
// ATL::CCRTAllocator    : malloc
// ATL::CLocalAllocator  : LocalAlloc
// ATL::CGlobalAllocator : GlobalAlloc(GMEM_FIXED)

/////////////////////////////////////////////////////////////////////////////

class CGlobalLockAllocator
{
public:
	static void* Allocate(size_t nBytes) throw()
	{
		HGLOBAL hGlobal = ::GlobalAlloc(GMEM_MOVEABLE, nBytes);
		return ::GlobalLock(hGlobal);
	}
	static void* Reallocate(void* p, size_t nBytes) throw()
	{
		::GlobalUnlock(p);
		HGLOBAL hGlobal = ::GlobalReAlloc(p, nBytes, 0);
		return ::GlobalLock(hGlobal);
	}
	static void Free(void* p) throw()
	{
		::GlobalUnlock(p);
		::GlobalFree(p);
	}
};

/////////////////////////////////////////////////////////////////////////////

class CTaskAllocator
{
public:
	static void* Reallocate(void* p, size_t nBytes) throw()
	{
		return CoTaskMemRealloc(p, nBytes);
		//IMalloc* pMalloc;
		//SHGetMalloc(&pMalloc);
		//void* pp = pMalloc->Realloc(p, nBytes);
		//pMalloc->Release();
		//return pp;
	}

	static void* Allocate(size_t nBytes) throw()
	{
		return CoTaskMemAlloc(nBytes);
		//IMalloc* pMalloc;
		//SHGetMalloc(&pMalloc);
		//void* p = pMalloc->Alloc(nBytes);
		//pMalloc->Release();
		//return p;
	}

	static void Free(void* p) throw()
	{
		CoTaskMemFree(p);
		//IMalloc* pMalloc;
		//SHGetMalloc(&pMalloc);
		//pMalloc->Free(p);
		//pMalloc->Release();
	}
};

} // namespace mew
