// DataObject.h
#pragma once

//#include <atlctl.h>

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CFormatEtc			: FORMATETC wrapper
// CEnumFORMATETC
// CDataObjectImpl<T>
// CDataObject_Text
// CDataObject_URL

#ifdef _UNICODE
const CLIPFORMAT CF_LPTSTR	= CF_UNICODETEXT;
#else
const CLIPFORMAT CF_LPTSTR	= CF_TEXT;
#endif

//////////////////////////////////////////////////////////////////////////////

class CFormatEtc : public FORMATETC
{
public:
	static CLIPFORMAT ToCLIPFORMAT(UINT format)		{ return (CLIPFORMAT)format; }
	static CLIPFORMAT ToCLIPFORMAT(LPCTSTR format)	{ return (CLIPFORMAT)::RegisterClipboardFormat(format); }

	CFormatEtc(UINT format, DWORD tymed_)
	{
		cfFormat	= ToCLIPFORMAT(format);
		ptd			= NULL; 
		dwAspect	= DVASPECT_CONTENT; 
		lindex		= -1; 
		tymed		= tymed_; 
	}
	CFormatEtc(LPCTSTR format, DWORD tymed_)
	{
		cfFormat	= ToCLIPFORMAT(format);
		ptd			= NULL; 
		dwAspect	= DVASPECT_CONTENT; 
		lindex		= -1; 
		tymed		= tymed_; 
	}
};

//////////////////////////////////////////////////////////////////////////////

#define BEGIN_FORMAT_MAP(name) \
	static const void GetFormatMap(const CFormatEtc** pp, ULONG* num) { \
		static const CFormatEtc FORMATS[] = { \

#define END_FORMAT_MAP() \
		}; \
		*num = sizeof(FORMATS) / sizeof(FORMATS[0]); \
		*pp = FORMATS; \
	}

#define FORMAT_ENTRY(format, tymed) \
		CFormatEtc(format, tymed),

#define FORMAT_STANDARD_FILES() \
		CFormatEtc(CFSTR_SHELLIDLIST, TYMED_HGLOBAL), \
		CFormatEtc(CFSTR_INETURL    , TYMED_HGLOBAL), \
		CFormatEtc(CF_HDROP         , TYMED_HGLOBAL),

//////////////////////////////////////////////////////////////////////////////

class CEnumFORMATETC : public IEnumFORMATETC
{
private:
	volatile LONG		m_ReferenceCount;
	const FORMATETC*	m_pFormats;
	size_t				m_Count;
	size_t				m_Iterator;

private:
	CEnumFORMATETC(const FORMATETC* formats, size_t count)
		: m_ReferenceCount(1), m_pFormats(formats), m_Count(count), m_Iterator(0)
	{}

public:
	static HRESULT New(IEnumFORMATETC** ppObject, const FORMATETC* formats, size_t count)
	{
		*ppObject = new CEnumFORMATETC(formats, count);
		return S_OK;
	}

	STDMETHOD(QueryInterface)(REFIID iid, void **ppvObject)
	{
		if(ppvObject == NULL)
			return E_POINTER;
		if(iid == IID_IUnknown || iid == IID_IEnumFORMATETC)
			*ppvObject = static_cast<IEnumFORMATETC*>(this);
		else
		{
			*ppvObject = NULL;
			return E_NOINTERFACE;
		}
		AddRef();
		return S_OK;
	}
	STDMETHOD_(ULONG, AddRef)()
	{
		return ::InterlockedIncrement(&m_ReferenceCount);
	}
	STDMETHOD_(ULONG, Release)()
	{
		LONG ref = ::InterlockedDecrement(&m_ReferenceCount);
		if(ref <= 0)
			delete this;
		return ref;
	}
	STDMETHOD(Next)(ULONG n, LPFORMATETC fmts, ULONG FAR* done)
	{
		for(ULONG i = 0; i < n; i++, m_Iterator++)
		{
			if(m_Iterator >= m_Count)
			{
				if(done) *done = i;
				return S_FALSE;
			}
			fmts[i] = m_pFormats[m_Iterator];
		}
		if(done) *done = n;
		return S_OK;
	}
	STDMETHOD(Skip)(ULONG n)
	{
		m_Iterator += n;
		return S_OK;
	}
	STDMETHOD(Reset)(void)
	{
		m_Iterator = 0;
		return S_OK;
	}
	STDMETHOD(Clone)(IEnumFORMATETC** ppObject)
	{
		return New(ppObject, m_pFormats, m_Count);
	}
};

//////////////////////////////////////////////////////////////////////////////
#if 0
class CDataObject : public IDataObjectImpl<CDataObject>
{
private:
	CStdVector<CDataObjectElement>	m_Data;

public:
	HRESULT AddShellIDList(LPCITEMIDLIST pIDList)
	{
		CFormatEtc fmt(CFSTR_SHELLIDLIST, TYMED_HGLOBAL);
		CStgMedium stg();
		return SetData(&fmt, &stg, TRUE);
	}
	HRESULT AddShellIDList(const CIDA* pCIDA)
	{
		CFormatEtc fmt(CFSTR_SHELLIDLIST, TYMED_HGLOBAL);
		CStgMedium stg();
		return SetData(&fmt, &stg, TRUE);
	}
	HRESULT AddText(LPCSTR text)
	{
		CFormatEtc fmt(CF_TEXT, TYMED_HGLOBAL);
		CStgMedium stg();
		return SetData(&fmt, &stg, TRUE);
	}
	HRESULT AddText(LPCWSTR text)
	{
		CFormatEtc fmt(CF_UNICODETEXT, TYMED_HGLOBAL);
		CStgMedium stg();
		return SetData(&fmt, &stg, TRUE);
	}
	HRESULT AddURL(LPCSTR url)
	{
		CFormatEtc fmt(CFSTR_INETURLA, TYMED_HGLOBAL);
		CStgMedium stg();
		return SetData(&fmt, &stg, TRUE);
	}
	HRESULT AddURL(LPCWSTR url)
	{
		CFormatEtc fmt(CFSTR_INETURLW, TYMED_HGLOBAL);
		CStgMedium stg();
		return SetData(&fmt, &stg, TRUE);
	}

	// IUnknown

	STDMETHOD(QueryInterface)(REFIID iid, void **ppvObject)
	{
		if(ppvObject == NULL)
			return E_POINTER;
		if(iid == IID_IUnknown || iid == IID_IDataObject)
			*ppvObject = static_cast<IDataObject*>(this);
		else
		{
			*ppvObject = NULL;
			return E_NOINTERFACE;
		}
		AddRef();
		return S_OK;
	}
	STDMETHOD_(ULONG, AddRef)()  { return 1; }
	STDMETHOD_(ULONG, Release)() { return 1; }

	// IDataObject

	STDMETHOD(GetData)(FORMATETC *fmt, STGMEDIUM *pmedium)
	{
		if(fmt->tymed & (TYMED_HGLOBAL | TYMED_ISTREAM))
		{
			HGLOBAL hGlobal = NULL;
			T* pThis = static_cast<T*>(this);
			HRESULT hr = pThis->GetDataOnHGlobal(fmt, &hGlobal);
			if FAILED(hr)
				return hr;
			if(hGlobal == NULL)
				return E_FAIL;
			pmedium->pUnkForRelease = NULL;
			if(fmt->tymed & TYMED_HGLOBAL)
			{
				pmedium->tymed = TYMED_HGLOBAL;
				pmedium->hGlobal = hGlobal;
				return S_OK;
			}
			else
			{
				pmedium->tymed = TYMED_ISTREAM;
				hr = CreateStreamOnHGlobal(hGlobal, TRUE, &pmedium->pstm);
				if FAILED(hr)
				{
					::GlobalFree(hGlobal);
					return hr;
				}
				return S_OK;
			}
		}
		else
		{
			ATLTRACE("CDataObjectImpl::GetData(0x%X) == DV_E_TYMED\n", fmt->tymed);
			return DV_E_TYMED;
		}
	}
	STDMETHOD(QueryGetData)(FORMATETC* lhs)
	{
		bool matchFORMAT = false, matchTYMED = false;
		const CFormatEtc* formats;
		ULONG num;
		T::GetFormatMap(&formats, &num);
		for(size_t i = 0; i < num; i++)
		{
			bool format = (lhs->cfFormat == formats[i].cfFormat);
			bool tymed  = ((lhs->tymed & formats[i].tymed) != 0);
			if(format && tymed)
				return S_OK;
			matchFORMAT |= format;
			matchTYMED  |= tymed;
		}
		if(!matchFORMAT) DV_E_FORMATETC;
		if(!matchTYMED)  DV_E_TYMED;
		return E_FAIL;
	}
	STDMETHOD(SetData)(FORMATETC* /* pformatetc */, STGMEDIUM* /* pmedium */, BOOL /* fRelease */)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::SetData"));
	}
	STDMETHOD(EnumFormatEtc)(DWORD dwDirection, IEnumFORMATETC** ppenumFormatEtc)
	{
		if(dwDirection == DATADIR_GET)
		{
			const CFormatEtc* formats;
			ULONG num;
			T::GetFormatMap(&formats, &num);
			return CEnumFORMATETC::New(ppenumFormatEtc, formats, num);
		}
		else
		{
			ATLTRACENOTIMPL(_T("CDataObjectImpl::EnumFormatEtc(dwDirection)"));
		}
	}

	HRESULT GetDataOnHGlobal(FORMATETC* fmt, HGLOBAL* phGlobal)
	{
		if(!m_pShellItemList)
			return E_FAIL;
		if(fmt->cfFormat == CFormatEtc::ToCLIPFORMAT(CFSTR_SHELLIDLIST))
		{
			size_t size = m_pShellItemList->GetSize();
			*phGlobal = ::GlobalAlloc(GMEM_MOVEABLE, size);
			m_pShellItemList->CopyTo(GlobalLock(*phGlobal));
			GlobalUnlock(*phGlobal);
		}
		else if(fmt->cfFormat == CF_HDROP)
		{
			*phGlobal = m_pShellItemList->CreateHDROP();
			if(!*phGlobal)
				return E_FAIL;
		}
		else
		{
			ATLTRACE("CDataObject_ShellItemList::GetData(0x%X) DV_E_FORMATETC\n", fmt->cfFormat);
			return DV_E_FORMATETC;
		}
		return S_OK;
	}
};
#endif
template <class T>
class ATL_NO_VTABLE CDataObjectImpl : public IDataObject //, IAsyncOperation
{
public:
	// Overridable

	HRESULT GetDataOnHGlobal(FORMATETC* fmt, HGLOBAL* phGlobal)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::GetData"));
	}

	// IUnknown

	STDMETHOD(QueryInterface)(REFIID iid, void **ppvObject)
	{
		if(ppvObject == NULL)
			return E_POINTER;
		if(iid == IID_IUnknown || iid == IID_IDataObject)
			*ppvObject = static_cast<IDataObject*>(this);
		else
		{
			*ppvObject = NULL;
			return E_NOINTERFACE;
		}
		AddRef();
		return S_OK;
	}
	STDMETHOD_(ULONG, AddRef)()  { return 1; }
	STDMETHOD_(ULONG, Release)() { return 1; }

	// IDataObject

	STDMETHOD(GetData)(FORMATETC *fmt, STGMEDIUM *pmedium)
	{
		if(fmt->tymed & (TYMED_HGLOBAL | TYMED_ISTREAM))
		{
			HGLOBAL hGlobal = NULL;
			T* pThis = static_cast<T*>(this);
			HRESULT hr = pThis->GetDataOnHGlobal(fmt, &hGlobal);
			if FAILED(hr)
				return hr;
			if(hGlobal == NULL)
				return E_FAIL;
			pmedium->pUnkForRelease = NULL;
			if(fmt->tymed & TYMED_HGLOBAL)
			{
				pmedium->tymed = TYMED_HGLOBAL;
				pmedium->hGlobal = hGlobal;
				return S_OK;
			}
			else
			{
				pmedium->tymed = TYMED_ISTREAM;
				hr = CreateStreamOnHGlobal(hGlobal, TRUE, &pmedium->pstm);
				if FAILED(hr)
				{
					::GlobalFree(hGlobal);
					return hr;
				}
				return S_OK;
			}
		}
		else
		{
			ATLTRACE("CDataObjectImpl::GetData(0x%X) == DV_E_TYMED\n", fmt->tymed);
			return DV_E_TYMED;
		}
	}
	STDMETHOD(GetDataHere)(FORMATETC* /* pformatetc */, STGMEDIUM* /* pmedium */)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::GetDataHere"));
	}
	STDMETHOD(QueryGetData)(FORMATETC* lhs)
	{
		bool matchFORMAT = false, matchTYMED = false;
		const CFormatEtc* formats;
		ULONG num;
		T::GetFormatMap(&formats, &num);
		for(size_t i = 0; i < num; i++)
		{
			bool format = (lhs->cfFormat == formats[i].cfFormat);
			bool tymed  = ((lhs->tymed & formats[i].tymed) != 0);
			if(format && tymed)
				return S_OK;
			matchFORMAT |= format;
			matchTYMED  |= tymed;
		}
		if(!matchFORMAT) DV_E_FORMATETC;
		if(!matchTYMED)  DV_E_TYMED;
		return E_FAIL;
	}
	STDMETHOD(GetCanonicalFormatEtc)(FORMATETC* /* pformatectIn */,FORMATETC* /* pformatetcOut */)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::GetCanonicalFormatEtc"));
	}
	STDMETHOD(SetData)(FORMATETC* /* pformatetc */, STGMEDIUM* /* pmedium */, BOOL /* fRelease */)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::SetData"));
	}
	STDMETHOD(EnumFormatEtc)(DWORD dwDirection, IEnumFORMATETC** ppenumFormatEtc)
	{
		if(dwDirection == DATADIR_GET)
		{
			const CFormatEtc* formats;
			ULONG num;
			T::GetFormatMap(&formats, &num);
			return CEnumFORMATETC::New(ppenumFormatEtc, formats, num);
		}
		else
		{
			ATLTRACENOTIMPL(_T("CDataObjectImpl::EnumFormatEtc(dwDirection)"));
		}
	}
	STDMETHOD(DAdvise)(FORMATETC *pformatetc, DWORD advf, IAdviseSink *pAdvSink, DWORD *pdwConnection)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::DAdvise\n"));
	}
	STDMETHOD(DUnadvise)(DWORD dwConnection)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::DUnadvise\n"));
	}
	STDMETHOD(EnumDAdvise)(IEnumSTATDATA **ppenumAdvise)
	{
		ATLTRACENOTIMPL(_T("CDataObjectImpl::EnumDAdvise\n"));
	}
};

/*
/////////////////////////////////////////////////////////////////////////////
// eLXg
// CR-LF sR[hƂAkI[łȂ΂ȂȂB

class CDataObject_Text : public CDataObjectImpl< CDataObject_Text >
{
public:
	BEGIN_FORMAT_MAP(CDataObject_Text)
		FORMAT_ENTRY(CF_TEXT       , TYMED_HGLOBAL | TYMED_ISTREAM)
		FORMAT_ENTRY(CF_UNICODETEXT, TYMED_HGLOBAL | TYMED_ISTREAM)
		FORMAT_ENTRY(CF_OEMTEXT    , TYMED_HGLOBAL | TYMED_ISTREAM)
	END_FORMAT_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// URL

class CDataObject_URL : public CDataObjectImpl< CDataObject_URL >
{
public:
	BEGIN_FORMAT_MAP(CDataObject_URL)
		FORMAT_ENTRY(CFSTR_INETURLA, TYMED_HGLOBAL | TYMED_ISTREAM)
		FORMAT_ENTRY(CFSTR_INETURLW, TYMED_HGLOBAL | TYMED_ISTREAM)
		FORMAT_ENTRY(CFSTR_SHELLURL, TYMED_HGLOBAL | TYMED_ISTREAM)
		FORMAT_ENTRY(CF_TEXT       , TYMED_HGLOBAL | TYMED_ISTREAM)
		FORMAT_ENTRY(CF_UNICODETEXT, TYMED_HGLOBAL | TYMED_ISTREAM)
		FORMAT_ENTRY(CF_OEMTEXT    , TYMED_HGLOBAL | TYMED_ISTREAM)
	END_FORMAT_MAP()
};
*/

} // namespace mew
