#ifndef __DOTNET_TABCTRL_H__
#define __DOTNET_TABCTRL_H__

#pragma once

#ifndef __CUSTOMTABCTRL_H__
  #error DotNetTabCtrl.h requires CustomTabCtrl.h to be included first
#endif

// NOTE: If you are compiling under VC7, be sure to put the following in
// your precompiled header:
//
//extern "C" const int _fltused = 0;

/////////////////////////////////////////////////////////////////////////////
//
// CDotNetTabItem
//  Derived Tab Item class that supports an HWND identifying a "tab view"
//
/////////////////////////////////////////////////////////////////////////////

class CDotNetTabItem : public CCustomTabItem
{
protected:
	typedef CDotNetTabItem	thisClass;

// Member variables (in addition to CCustomTabItem ones)
protected:
	HWND  m_hWndTabView;
	HICON m_hIcon;
	CSize m_IconSize;

// Constructors/Destructors
public:
	CDotNetTabItem() : m_hWndTabView(NULL), m_hIcon(NULL), m_IconSize(16, 16)
	{
	}

// Accessors
public:
	HICON GetIcon() const			{ return m_hIcon; }
	void  SetIcon(HICON hIcon)		{ m_hIcon = hIcon; }
	CSize GetIconSize() const		{ return m_IconSize; }
	void  SetIconSize(CSize sz)		{ m_IconSize = sz; }
	HWND  GetTabView() const		{ return m_hWndTabView; }
	void  SetTabView(HWND hWnd)		{ m_hWndTabView = hWnd; }

// Methods:
public:
	bool UsingTabView() const	 	{ return (m_hWndTabView != NULL); }
	bool UsingIcon() const			{ return m_hIcon != NULL; }
};

////////////////////////////////////////////////////////////////////////////////

template < class T, class TItem >
class ATL_NO_VTABLE CDotNetTabLookImpl : public CTabCtrlLookImpl<T, TItem>
{
private:
	struct Settings
	{
		signed char iPadding;
		signed char iMargin;
		signed char iSelMargin;
		signed char iIndent;
		signed char iFontSizeTextTopOffset;
		signed char iMinWidthToDisplayText;
		signed char iReserved[2];
	};

	CBrush m_hbrBackground;
	COLORREF m_clrBackground, m_clrInactiveTab;
	Settings m_settings;

// Constructor
public:
	CDotNetTabLookImpl() :
		m_clrBackground(::GetSysColor(COLOR_APPWORKSPACE)),
		m_clrInactiveTab(::GetSysColor(COLOR_GRAYTEXT))
	{
		::ZeroMemory(&m_settings, sizeof(Settings));
		m_settings.iMinWidthToDisplayText = 12;
	}

public:
	BEGIN_MSG_MAP(CDotNetTabLookImpl)
		MESSAGE_HANDLER(WM_SETTINGCHANGE, OnSettingChange)
		CHAIN_MSG_MAP(__super)
	END_MSG_MAP()

	LRESULT OnSettingChange(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		DWORD dwStyle = pT->GetStyle();

		// Initialize/Reinitialize font
		// Visual Studio.Net seems to use the "icon" font for the tabs
		LOGFONT lfIcon = { 0 };
		::SystemParametersInfo(SPI_GETICONTITLELOGFONT, sizeof(lfIcon), &lfIcon, 0);

		bool bResetFont = true;
		if( !m_font.IsNull() )
		{
			LOGFONT lf = {0};
			if(m_font.GetLogFont(&lf))
			{
				if(lstrcmpi(lf.lfFaceName, lfIcon.lfFaceName) == 0 &&
					lf.lfHeight == lfIcon.lfHeight)
				{
					bResetFont = false;
				}
			}
		}

		if(bResetFont)
		{
			if(!m_font.IsNull()) m_font.DeleteObject();
			if(!m_fontSel.IsNull()) m_fontSel.DeleteObject();

			HFONT font = m_font.CreateFontIndirect(&lfIcon);
			if(font==NULL)
			{
				m_font.Attach(AtlGetDefaultGuiFont());
			}

			if(CTCS_BOLDSELECTEDTAB == (dwStyle & CTCS_BOLDSELECTEDTAB))
			{
				lfIcon.lfWeight = FW_BOLD;
			}

			font = m_fontSel.CreateFontIndirect(&lfIcon);
			if(font==NULL)
			{
				m_fontSel.Attach(AtlGetDefaultGuiFont());
			}
		}

		// Background brush
		if( !m_hbrBackground.IsNull() ) m_hbrBackground.DeleteObject();
		CWindowDC dcWindow(NULL);
		int nBitsPerPixel = dcWindow.GetDeviceCaps(BITSPIXEL);
		if( nBitsPerPixel > 8 )
		{
			//COLORREF clrBtnHilite = ::GetSysColor(COLOR_BTNHILIGHT);
			//COLORREF clrBtnFace = ::GetSysColor(COLOR_BTNFACE);
			//m_clrBackground =
			//	RGB( GetRValue(clrBtnFace) + ((GetRValue(clrBtnHilite) - GetRValue(clrBtnFace)) / 3 * 2),
			//		GetGValue(clrBtnFace) + ((GetGValue(clrBtnHilite) - GetGValue(clrBtnFace)) / 3 * 2),
			//		GetBValue(clrBtnFace) + ((GetBValue(clrBtnHilite) - GetBValue(clrBtnFace)) / 3 * 2),
			//	);
			//m_hbrBackground.CreateSolidBrush(m_clrBackground);

			COLORREF clrBtnFace = ::GetSysColor(COLOR_BTNFACE);

			// This is a brave attempt to mimic the algorithm that Visual Studio.Net
			// uses to calculate the tab's background color and inactive tab color.
			// The other colors that VS.Net uses seems to be standard ones,
			// but these two colors are calculated.
			BYTE nRed = 0, nGreen = 0, nBlue = 0, nMax = 0;

			// Early experiments seemed to reveal that the background color is dependant
			// on COLOR_BTNFACE.  The following algorithm is just an attempt
			// to match several empirical results.  I tested with 20 variations
			// on COLOR_BTNFACE and kept track of what the tab background became.
			// I then brought the numbers into Excel, and started crunching on the numbers
			// until I came up with a formula that seems to pretty well match.

			nRed = GetRValue(clrBtnFace);
			nGreen = GetGValue(clrBtnFace);
			nBlue = GetBValue(clrBtnFace);

			nMax = (nRed > nGreen) ? ((nRed > nBlue) ? nRed : nBlue) : ((nGreen > nBlue) ? nGreen : nBlue);
			const BYTE nMagicBackgroundOffset = (nMax > (0xFF - 35)) ? (BYTE)(0xFF - nMax) : (BYTE)35;
			if(nMax == 0)
			{
				nRed = (BYTE)(nRed + nMagicBackgroundOffset);
				nGreen = (BYTE)(nGreen + nMagicBackgroundOffset);
				nBlue = (BYTE)(nBlue + nMagicBackgroundOffset);
			}
			else
			{
				nRed = (BYTE)(nRed + (nMagicBackgroundOffset*(nRed/(double)nMax) + 0.5));
				nGreen = (BYTE)(nGreen + (nMagicBackgroundOffset*(nGreen/(double)nMax) + 0.5));
				nBlue = (BYTE)(nBlue + (nMagicBackgroundOffset*(nBlue/(double)nMax) + 0.5));
			}

			m_clrBackground = RGB(nRed, nGreen, nBlue);
			m_hbrBackground.CreateSolidBrush(m_clrBackground);


			// The inactive tab color seems to be calculated in a similar way to
			// the tab background, only instead of lightening BNTFACE, it darkens GRAYTEXT.
			COLORREF clrGrayText = ::GetSysColor(COLOR_GRAYTEXT);

			nRed   = GetRValue(clrGrayText);
			nGreen = GetGValue(clrGrayText);
			nBlue  = GetBValue(clrGrayText);

			nMax = (nRed > nGreen) ? ((nRed > nBlue) ? nRed : nBlue) : ((nGreen > nBlue) ? nGreen : nBlue);
			const BYTE nMagicInactiveOffset = 43;
			if(nMax != 0)
			{
				if(nRed < nMagicInactiveOffset)
					nRed = (BYTE)(nRed / 2);
				else
					nRed  = (BYTE)(nRed - (nMagicInactiveOffset*(nRed/(double)nMax) + 0.5));

				if(nGreen < nMagicInactiveOffset)
					nGreen = (BYTE)(nGreen / 2);
				else
					nGreen = (BYTE)(nGreen - (nMagicInactiveOffset*(nGreen/(double)nMax) + 0.5));

				if(nBlue < nMagicInactiveOffset)
					nBlue = (BYTE)(nBlue / 2);
				else
					nBlue = (BYTE)(nBlue - (nMagicInactiveOffset*(nBlue/(double)nMax) + 0.5));
			}

			m_clrInactiveTab = RGB(nRed, nGreen, nBlue);
		}
		else
		{
			m_hbrBackground =  CDCHandle::GetHalftoneBrush();
			m_clrInactiveTab = ::GetSysColor(COLOR_GRAYTEXT);
		}

		m_settings.iIndent    = 5;
		m_settings.iPadding   = 4;
		m_settings.iMargin    = 3;
		m_settings.iSelMargin = 3;

		int nHeightLogicalUnits = -lfIcon.lfHeight;
		// In MSDN for "LOGFONT", they give the following formula for calculating
		// the log font height given a point size.
		//long lfHeight = -MulDiv(PointSize, GetDeviceCaps(hDC, LOGPIXELSY), 72);

		const int nNominalFontLogicalUnits = 11;	// 8 point Tahoma with 96 DPI
		m_settings.iFontSizeTextTopOffset = (BYTE)((nHeightLogicalUnits - nNominalFontLogicalUnits) / 2);

		pT->UpdateLayout();
		pT->Invalidate();
		return 0;
	}

public:
	void CalcComponentSize(const CRect& rcClient)
	{
		DWORD dwStyle = static_cast<const T*>(this)->GetStyle();
		m_rcTabItemArea = rcClient;
		CalcSize_NonClient(&m_rcTabItemArea, dwStyle);
		if(CTCS_CLOSEBUTTON == (dwStyle & CTCS_CLOSEBUTTON))
			CalcSize_CloseButton(m_rcTabItemArea, dwStyle);
		if(CTCS_SCROLL == (dwStyle & CTCS_SCROLL))
			CalcSize_ScrollButtons(m_rcTabItemArea, dwStyle);
	}
private:
	void CalcSize_NonClient(LPRECT prcTabItemArea, DWORD dwStyle)
	{
		// account for "non-client" areas
		// TODO: For the short term, we will use this
		//  for the non-client areas on the left and right.
		//  The drawing code for the tabs already accounts
		//  for the "non-client" areas on the top and bottom, and
		//  would need to be updated if we account for it here.
		//  Tab item rect methods also would need to be
		//  updated to account for the non-client areas
		//  on top and bottom (and effected drawing code
		//  would need to be updated).

		if(CTCS_BOTTOM == (dwStyle & CTCS_BOTTOM))
		{
			// TODO: Update to actually specify the
			//  non-client areas, and adjust all of the
			//  effected drawing code, as well as
			//  tab item rect related things
			//prcTabItemArea->top += 3;
		}
		else
		{
			prcTabItemArea->left += 2;
			prcTabItemArea->right -= 2;

			// TODO: Update to actually specify the top and bottom
			//  non-client areas, and adjust all of the
			//  effected drawing code, as well as
			//  tab item rect related things
			//prcTabItemArea->top += 1;
			//// We would have bottom as 3, but we want the
			//// selected tab to actually paint over highlight part
			//prcTabItemArea->bottom -= 2;
		}
	}

	void CalcSize_CloseButton(LPRECT prcTabItemArea, DWORD dwStyle)
	{
		//int nButtonSizeX = ::GetSystemMetrics(SM_CXSMSIZE);
		//int nButtonSizeY = ::GetSystemMetrics(SM_CYSMSIZE);
		// NOTE: After several tests, VS.Net does NOT depend on
		//  any system metric for the button size, so neither will we.
		int nButtonSizeX = 15;
		int nButtonSizeY = 15;

		if((prcTabItemArea->right - prcTabItemArea->left) < nButtonSizeX)
		{
			::SetRectEmpty(&m_rcCloseButton);
			return;
		}

		m_rcCloseButton = *prcTabItemArea;

		if (CTCS_BOTTOM == (dwStyle & CTCS_BOTTOM))
		{
			m_rcCloseButton.top += 3;
			m_rcCloseButton.right -= 3;
		}
		else
		{
			m_rcCloseButton.top += 1;
			m_rcCloseButton.bottom -= 2;
			m_rcCloseButton.right -= 2;
		}
		m_rcCloseButton.top = (m_rcCloseButton.bottom + m_rcCloseButton.top - nButtonSizeY) / 2;
		m_rcCloseButton.bottom = m_rcCloseButton.top + nButtonSizeY;

		m_rcCloseButton.left = m_rcCloseButton.right - (nButtonSizeX);

		// Adjust the tab area
		prcTabItemArea->right = m_rcCloseButton.left;
	}

	void CalcSize_ScrollButtons(LPRECT prcTabItemArea, DWORD dwStyle)
	{
		//int nButtonSizeX = ::GetSystemMetrics(SM_CXSMSIZE);
		//int nButtonSizeY = ::GetSystemMetrics(SM_CYSMSIZE);
		// NOTE: After several tests, VS.Net does NOT depend on
		//  any system metric for the button size, so neither will we.
		int nButtonSizeX = 15;
		int nButtonSizeY = 15;

		if((prcTabItemArea->right - prcTabItemArea->left) < nButtonSizeX)
		{
			::SetRectEmpty(&m_rcScrollRight);
			::SetRectEmpty(&m_rcScrollLeft);
			return;
		}

		RECT rcScroll = *prcTabItemArea;

		if (CTCS_BOTTOM == (dwStyle & CTCS_BOTTOM))
		{
			rcScroll.top += 3;
			if(0 == (dwStyle & CTCS_CLOSEBUTTON))
			{
				rcScroll.right -= 3;
			}
		}
		else
		{
			rcScroll.top += 1;
			rcScroll.bottom -= 2;
			if(0 == (dwStyle & CTCS_CLOSEBUTTON))
			{
				rcScroll.right -= 2;
			}
		}
		rcScroll.top = (rcScroll.bottom + rcScroll.top - nButtonSizeY) / 2;
		rcScroll.bottom = rcScroll.top + nButtonSizeY;

		m_rcScrollRight = rcScroll;
		m_rcScrollLeft = rcScroll;

		m_rcScrollRight.left = m_rcScrollRight.right - nButtonSizeX;

		m_rcScrollLeft.right = m_rcScrollRight.left;
		m_rcScrollLeft.left = m_rcScrollLeft.right - nButtonSizeX;

		// Adjust the tab area
		prcTabItemArea->right = m_rcScrollLeft.left;
	}

public:
	void InitializeDrawStruct(LPNMCTCCUSTOMDRAW lpNMCustomDraw, DWORD dwStateBits)
	{
		lpNMCustomDraw->hFontInactive = m_font;
		lpNMCustomDraw->hFontSelected = m_fontSel;
		lpNMCustomDraw->hBrushBackground = m_hbrBackground;
		lpNMCustomDraw->clrTextSelected = ::GetSysColor(COLOR_BTNTEXT);
		lpNMCustomDraw->clrTextInactive = m_clrInactiveTab;
		lpNMCustomDraw->clrSelectedTab = ::GetSysColor(COLOR_BTNFACE);
		lpNMCustomDraw->clrBtnFace = ::GetSysColor(COLOR_BTNFACE);
		lpNMCustomDraw->clrBtnShadow = ::GetSysColor(COLOR_BTNSHADOW);
		lpNMCustomDraw->clrBtnHighlight = ::GetSysColor(COLOR_BTNHIGHLIGHT);
		lpNMCustomDraw->clrBtnText = ::GetSysColor(COLOR_BTNTEXT);
		lpNMCustomDraw->clrHighlight = ::GetSysColor(COLOR_HIGHLIGHT);
#if WINVER >= 0x0500 || _WIN32_WINNT >= 0x0500
		lpNMCustomDraw->clrHighlightHotTrack = ::GetSysColor(COLOR_HOTLIGHT);
#else
		lpNMCustomDraw->clrHighlightHotTrack = ::GetSysColor(COLOR_HIGHLIGHT);
#endif
	}

	void DoPrePaint(RECT rcClient, LPNMCTCCUSTOMDRAW lpNMCustomDraw, DWORD dwStateBits)
	{
		CDCHandle dc( lpNMCustomDraw->nmcd.hdc );

		// Set up the text color and background mode
		dc.SetTextColor(lpNMCustomDraw->clrBtnText);
		dc.SetBkMode(TRANSPARENT);

		RECT rcClip = {0};
		dc.GetClipBox(&rcClip);

		if(::EqualRect(&rcClip, &m_rcCloseButton) ||
			::EqualRect(&rcClip, &m_rcScrollRight) ||
			::EqualRect(&rcClip, &m_rcScrollLeft))
		{
			// Paint needed in only "other button" area

			HBRUSH hOldBrush = dc.SelectBrush(lpNMCustomDraw->hBrushBackground);
			dc.PatBlt(rcClip.left, rcClip.top, rcClip.right-rcClip.left, rcClip.bottom-rcClip.top, PATCOPY);
			dc.SelectBrush(hOldBrush);
		}
		else
		{
			// Paint needed in tab item area or more

			// Erase Background
			//  (do it here instead of a handler for WM_ERASEBKGND
			//   so that we can do flicker-free drawing with the help
			//   of COffscreenDrawRect that's in the base class)

			// TODO: Don't "erase" entire client area.
			//  Do a smarter erase of just what needs it

			RECT rc = rcClient;

			HBRUSH hOldBrush = dc.SelectBrush(lpNMCustomDraw->hBrushBackground);
			dc.PatBlt(rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top, PATCOPY);
			dc.SelectBrush(hOldBrush);

			// Connect with the client area.
			DWORD dwStyle = static_cast<T*>(this)->GetStyle();

			if(CTCS_BOTTOM == (dwStyle & CTCS_BOTTOM))
			{
				rc.bottom = rc.top + 3;
				dc.FillSolidRect(&rc, lpNMCustomDraw->clrBtnFace);

				CPen penText;
				penText.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrBtnText);
				CPenHandle penOld = dc.SelectPen(penText);

				dc.MoveTo(rc.left, rc.bottom);
				dc.LineTo(rc.right, rc.bottom);

				dc.SelectPen(penOld);
			}
			else
			{
				int nOrigTop = rc.top;
				rc.top = rc.bottom - 2;
				dc.FillSolidRect(&rc, lpNMCustomDraw->clrBtnFace);

				CPen penHilight;
				penHilight.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrBtnHighlight);
				CPenHandle penOld = dc.SelectPen(penHilight);

				dc.MoveTo(rc.left, rc.top-1);
				dc.LineTo(rc.right, rc.top-1);

				rc.top = nOrigTop;

				CPen penShadow, pen3D;
				penShadow.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrBtnShadow);
				pen3D.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrBtnFace);
				dc.SelectPen(penShadow);

				dc.MoveTo(rc.left, rc.bottom);
				dc.LineTo(rc.left, rc.top);
				dc.LineTo(rc.right-1, rc.top);
						
				if(0 == (dwStyle & CTCS_FLATEDGE))
				{
					dc.SelectPen(penHilight);
				}
				dc.LineTo(rc.right-1, rc.bottom);

				dc.SelectPen(pen3D);
				dc.MoveTo(rc.right-2, rc.bottom-3);
				dc.LineTo(rc.right-2, rc.top);
				dc.MoveTo(rc.left+1, rc.bottom-3);
				dc.LineTo(rc.left+1, rc.top);

				dc.SelectPen(penOld);
			}

		}
	}

	void DoItemPaint(LPNMCTCCUSTOMDRAW lpNMCustomDraw, DWORD dwStateBits)
	{
		T* pT = static_cast<T*>(this);

		CDCHandle dc( lpNMCustomDraw->nmcd.hdc );
		bool bSelected = (CDIS_SELECTED == (lpNMCustomDraw->nmcd.uItemState & CDIS_SELECTED));
		bool bHot = (CDIS_HOT == (lpNMCustomDraw->nmcd.uItemState & CDIS_HOT));
		// NOTE: lpNMCustomDraw->nmcd.rc is in logical coordinates
		RECT &rc = lpNMCustomDraw->nmcd.rc;
		int nItem = lpNMCustomDraw->nmcd.dwItemSpec;
		TItem* pItem = pT->GetItem(nItem);
		DWORD dwStyle = pT->GetStyle();

		RECT rcTab = rc;
		RECT rcText = rc;

		if(CTCS_BOTTOM == (dwStyle & CTCS_BOTTOM))
		{
			rcTab.top += 3;
			rcTab.bottom -= 2;

			rcText.top = rcTab.top+1 + m_settings.iFontSizeTextTopOffset;
			rcText.bottom = rc.bottom;
		}
		else
		{
			rcTab.top += 3;
			rcTab.bottom -= 2;

			rcText.top = rc.top+1 + m_settings.iFontSizeTextTopOffset;
			rcText.bottom = rc.bottom;
		}

		int nIconVerticalCenter = (rc.bottom + rc.top) / 2 + rcTab.top / 2;

		if( bSelected )
		{
			// Tab is selected, so paint tab folder

			rcTab.right--;
			dc.FillSolidRect(&rcTab, lpNMCustomDraw->clrSelectedTab);

			CPen penText, penHilight;
			penText.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrBtnText);
			penHilight.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrBtnHighlight);

			if(CTCS_BOTTOM == (dwStyle & CTCS_BOTTOM))
			{
				CPenHandle penOld = dc.SelectPen(penText);

				dc.MoveTo(rcTab.right, rcTab.top);
				dc.LineTo(rcTab.right, rcTab.bottom);
				dc.LineTo(rcTab.left, rcTab.bottom);
				dc.SelectPen(penHilight);
				dc.LineTo(rcTab.left, rcTab.top-1);

				dc.SelectPen(penOld);
			}
			else
			{
				CPenHandle penOld = dc.SelectPen(penHilight);

				dc.MoveTo(rcTab.left, rcTab.bottom-1);
				dc.LineTo(rcTab.left, rcTab.top);
				dc.LineTo(rcTab.right, rcTab.top);
				dc.SelectPen(penText);
				dc.LineTo(rcTab.right, rcTab.bottom);

				dc.SelectPen(penOld);
			}
		}
		else
		{
			// Tab is not selected

			// Draw division line on right, unless we're the item
			// on the left of the selected tab
			if(pT->GetSel(nItem + 1))
			{
				// Item just left of selected tab
			}
			else
			{
				CPen pen;
				pen.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrBtnShadow);
				CPenHandle penOld = dc.SelectPen(pen);
				if(CTCS_BOTTOM == (dwStyle & CTCS_BOTTOM))
				{
					// Important!  Be sure and keep within "our" tab area horizontally
					dc.MoveTo(rcTab.right-1, rcTab.top + 3);
					dc.LineTo(rcTab.right-1, rcTab.bottom - 1);
				}
				else
				{
					// Important!  Be sure and keep within "our" tab area horizontally
					dc.MoveTo(rcTab.right-1, rcTab.top + 2);
					dc.LineTo(rcTab.right-1, rcTab.bottom - 2);
				}
				dc.SelectPen(penOld);
			}
		}

		HFONT hOldFont = NULL;
		if( bSelected )
		{
			hOldFont = dc.SelectFont(lpNMCustomDraw->hFontSelected);
		}
		else
		{
			hOldFont = dc.SelectFont(lpNMCustomDraw->hFontInactive);
		}

		COLORREF crPrevious = 0;
		if( bHot )
		{
			crPrevious = dc.SetTextColor(lpNMCustomDraw->clrHighlightHotTrack);
		}
		else if( bSelected )
		{
			crPrevious = dc.SetTextColor(lpNMCustomDraw->clrTextSelected);
		}
		else
		{
			crPrevious = dc.SetTextColor(lpNMCustomDraw->clrTextInactive);
		}

		//--------------------------------------------
		// This is how CDotNetTabCtrlImpl interprets padding, margin, etc.:
		//
		//  M - Margin
		//  P - Padding
		//  I - Image
		//  Text - Tab Text
		//
		// With image:
		//     __________________________
		//
		//    | M | I | P | Text | P | M |
		//     --------------------------
		//
		// Without image:
		//     ______________________
		//
		//    | M | P | Text | P | M |
		//     ----------------------

		//rcText.left += (bSelected ? m_settings.iSelMargin : m_settings.iMargin);
		rcText.left += m_settings.iMargin;
		rcText.right -= m_settings.iMargin;
		if(pItem->UsingIcon())
		{
			CSize szIcon = pItem->GetIconSize();
			// Draw the image.
			if( szIcon.cx < (rcTab.right - rcTab.left) )
			{
				int nImageHalfHeight = szIcon.cy / 2;
				dc.DrawIconEx(rcText.left, nIconVerticalCenter - nImageHalfHeight + m_settings.iFontSizeTextTopOffset, pItem->GetIcon(), szIcon.cx, szIcon.cy, 0, 0, DI_NORMAL);
			}

			// Offset on the right of the image.
			rcText.left += szIcon.cx;
		}

		if(pItem->UsingText() && rcText.left + m_settings.iMinWidthToDisplayText < rcText.right)
		{
			::InflateRect(&rcText, -m_settings.iPadding, 0);
			dc.DrawText(pItem->GetText(), -1, &rcText, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		}

		dc.SetTextColor(crPrevious);
		dc.SelectFont(hOldFont);
	}

	void DoPostPaint(RECT /*rcClient*/, LPNMCTCCUSTOMDRAW lpNMCustomDraw, DWORD dwStateBits)
	{
		DWORD dwStyle = static_cast<T*>(this)->GetStyle();

		if(0 == (dwStyle & (CTCS_CLOSEBUTTON | CTCS_SCROLL)))
		{
			return;
		}

		CDCHandle dc( lpNMCustomDraw->nmcd.hdc );

		CPen penButtons;
		penButtons.CreatePen(PS_SOLID, 1, lpNMCustomDraw->clrTextInactive);
		CBrush brushArrow;
		brushArrow.CreateSolidBrush(lpNMCustomDraw->clrTextInactive);

		CPenHandle penOld = dc.SelectPen(penButtons);
		CBrushHandle brushOld = dc.SelectBrush(brushArrow);

		// Close Button
		if(CTCS_CLOSEBUTTON == (dwStyle & CTCS_CLOSEBUTTON))
		{
			RECT rcX = m_rcCloseButton;

			if(ectcMouseDown_CloseButton == (dwStateBits & ectcMouseDown))
			{
				if(ectcMouseOver_CloseButton == (dwStateBits & ectcMouseOver))
				{
					::OffsetRect(&rcX, 1, 1);
				}
			}

			const int sp = 4;

			dc.MoveTo(rcX.left+sp+ -1, rcX.top+sp);
			dc.LineTo(rcX.right-sp -1, rcX.bottom-sp);
			dc.MoveTo(rcX.left+sp, rcX.top+sp);
			dc.LineTo(rcX.right-sp, rcX.bottom-sp);

			dc.MoveTo(rcX.left+sp -1, rcX.bottom-sp -1);
			dc.LineTo(rcX.right-sp -1, rcX.top+sp -1 );
			dc.MoveTo(rcX.left+sp, rcX.bottom-sp -1);
			dc.LineTo(rcX.right-sp, rcX.top+sp -1);

			if(ectcMouseDown_CloseButton == (dwStateBits & ectcMouseDown))
			{
				if(ectcMouseOver_CloseButton == (dwStateBits & ectcMouseOver))
				{
					dc.DrawEdge(&m_rcCloseButton, BDR_SUNKENOUTER, BF_RECT);
				}
			}
			else if(ectcHotTrack_CloseButton == (dwStateBits & ectcHotTrack))
			{
				dc.DrawEdge(&m_rcCloseButton, BDR_RAISEDINNER, BF_RECT);
			}
		}

		// Scroll Buttons
		if(CTCS_SCROLL == (dwStyle & CTCS_SCROLL))
		{
			RECT rcArrowRight = m_rcScrollRight;
			RECT rcArrowLeft = m_rcScrollLeft;

			if(ectcMouseDown_ScrollRight == (dwStateBits & ectcMouseDown))
			{
				if(ectcMouseOver_ScrollRight == (dwStateBits & ectcMouseOver))
				{
					if(ectcOverflowRight == (dwStateBits & ectcOverflowRight))
					{
						::OffsetRect(&rcArrowRight, 1, 1);
					}
				}
			}
			if(ectcMouseDown_ScrollLeft == (dwStateBits & ectcMouseDown))
			{
				if(ectcMouseOver_ScrollLeft == (dwStateBits & ectcMouseOver))
				{
					if(ectcOverflowLeft == (dwStateBits & ectcOverflowLeft))
					{
						::OffsetRect(&rcArrowLeft, 1, 1);
					}
				}
			}

			const int spRight = 5;
			const int spLeft = 6;

			POINT ptsArrowRight[] = {
				{rcArrowRight.left+spRight, rcArrowRight.top+spRight -2},
				{rcArrowRight.left+spRight, rcArrowRight.bottom-spRight +1},
				{rcArrowRight.right-spRight -1, (rcArrowRight.bottom + m_rcScrollRight.top) / 2},
				{rcArrowRight.left+spRight, rcArrowRight.top+spRight -2}
			};
			if(ectcOverflowRight != (dwStateBits & ectcOverflowRight))
			{
				dc.Polyline(ptsArrowRight, 4);
			}
			else
			{
				dc.Polygon(ptsArrowRight, 4);

				if(ectcMouseDown_ScrollRight == (dwStateBits & ectcMouseDown))
				{
					if(ectcMouseOver_ScrollRight == (dwStateBits & ectcMouseOver))
					{
						dc.DrawEdge(&m_rcScrollRight, BDR_SUNKENOUTER, BF_RECT);
					}
				}
				else if(ectcHotTrack_ScrollRight == (dwStateBits & ectcHotTrack))
				{
					dc.DrawEdge(&m_rcScrollRight, BDR_RAISEDINNER, BF_RECT);
				}
			}

			POINT ptsArrowLeft[] = {
				{rcArrowLeft.right-spLeft, rcArrowLeft.top+spLeft -3},
				{rcArrowLeft.right-spLeft, rcArrowLeft.bottom-spLeft +2},
				{rcArrowLeft.left+spLeft -1, (rcArrowLeft.bottom + m_rcScrollLeft.top) / 2},
				{rcArrowLeft.right-spLeft, rcArrowLeft.top+spLeft -3}
			};
			if(ectcOverflowLeft != (dwStateBits & ectcOverflowLeft))
			{
				dc.Polyline(ptsArrowLeft, 4);
			}
			else
			{
				dc.Polygon(ptsArrowLeft, 4);

				if(ectcMouseDown_ScrollLeft == (dwStateBits & ectcMouseDown))
				{
					if(ectcMouseOver_ScrollLeft == (dwStateBits & ectcMouseOver))
					{
						dc.DrawEdge(&m_rcScrollLeft, BDR_SUNKENOUTER, BF_RECT);
					}
				}
				else if(ectcHotTrack_ScrollLeft == (dwStateBits & ectcHotTrack))
				{
					dc.DrawEdge(&m_rcScrollLeft, BDR_RAISEDINNER, BF_RECT);
				}
			}

		}

		dc.SelectBrush(brushOld);
		dc.SelectPen(penOld);
	}

// Overrides from CCustomTabCtrl
public:
	void UpdateLayout_Default(RECT rcTabItemArea)
	{
		T* pT = static_cast<T*>(this);
		long nMinInactiveWidth = 0x7FFFFFFF;
		long nMaxInactiveWidth = 0;

		CClientDC dc(pT->m_hWnd);
		//HFONT hOldFont = dc.SelectFont(lpNMCustomDraw->hFontInactive);
		HFONT hOldFont = dc.SelectFont(m_font);

		RECT rcItem = rcTabItemArea;
		// rcItem.top and rcItem.bottom aren't really going to change

		// Recalculate tab positions and widths
		// See OnItemPrePaint for a discussion of how CDotNetTabCtrlImpl
		//  interprets margin, padding, etc.
		size_t nCount = pT->GetItemCount();
		int xpos = m_settings.iIndent;
		HFONT hRestoreNormalFont = NULL;
		for( size_t i=0; i<nCount; ++i )
		{
			bool bSelected = pT->GetSel(i);
			if(bSelected)
			{
				//hRestoreNormalFont = dc.SelectFont(lpNMCustomDraw->hFontSelected);
				hRestoreNormalFont = dc.SelectFont(m_fontSel);
			}

			rcItem.left = rcItem.right = xpos;
			int width = UpdateTabRect(pT->GetItem(i), dc.m_hDC, rcItem, m_settings);
			xpos += width;

			if(hRestoreNormalFont != NULL)
			{
				dc.SelectFont(hRestoreNormalFont);
				hRestoreNormalFont = NULL;
			}

			if(!bSelected)
			{
				if(width < nMinInactiveWidth)
				{
					nMinInactiveWidth = width;
				}
				if(width > nMaxInactiveWidth)
				{
					nMaxInactiveWidth = width;
				}
			}
		}
		xpos += m_settings.iIndent;

		LONG nTabAreaWidth = (rcTabItemArea.right - rcTabItemArea.left);
		if(xpos > nTabAreaWidth && nCount > 0 && pT->GetCurSel() >= 0)
		{
			// Our desired widths are more than the width of the client area.
			// We need to have some or all of the tabs give up some real estate

			// We'll try to let the selected tab have its fully desired width.
			// If it can't, we'll make all the tabs the same width.

			RECT rcSelected = pT->GetItem(pT->GetCurSel())->GetRect();
			LONG nSelectedWidth = (rcSelected.right - rcSelected.left);

			long cxClientInactiveTabs = nTabAreaWidth - (m_settings.iIndent * 2) - nSelectedWidth;

			double nRatioWithSelectionFullSize = (double) (cxClientInactiveTabs) / (double)(xpos - (m_settings.iIndent * 2) - nSelectedWidth);

			long nInactiveSameSizeWidth = (m_settings.iMinWidthToDisplayText + (m_settings.iMargin*2) + (m_settings.iPadding));

			if(cxClientInactiveTabs > (nInactiveSameSizeWidth * (long)(nCount-1)))
			{
				//  There should be enough room to display the entire contents of
				//  the selected tab plus something for the inactive tabs

				bool bMakeInactiveSameSize = ((nMinInactiveWidth * nRatioWithSelectionFullSize) < nInactiveSameSizeWidth);

				xpos = m_settings.iIndent;
				for(size_t i=0; i<nCount; ++i )
				{
					TItem* pItem = pT->GetItem(i);
					RECT rcItemDesired = pItem->GetRect();
					rcItem.left = rcItem.right = xpos;
					if(pT->GetSel(i))
					{
						rcItem.right += (rcItemDesired.right - rcItemDesired.left);
					}
					else
					{
						if(bMakeInactiveSameSize)
						{
							rcItem.right += (long)((cxClientInactiveTabs / (nCount-1)) + 0.5);
						}
						else
						{
							rcItem.right += (long)(((rcItemDesired.right - rcItemDesired.left) * nRatioWithSelectionFullSize) + 0.5);
						}
					}
					pItem->SetRect(rcItem);
					xpos += (rcItem.right-rcItem.left);
				}
			}
			else
			{
				// We're down pretty small, so just make all the tabs the same width
				int cxItem = (nTabAreaWidth - (m_settings.iIndent*2)) / (int)nCount;

				xpos = m_settings.iIndent;

				for(size_t i=0; i<nCount; ++i)
				{
					rcItem.left = rcItem.right = xpos;
					rcItem.right += cxItem;
					pT->GetItem(i)->SetRect(rcItem);
					xpos += (rcItem.right-rcItem.left);
				}
			}
		}

		dc.SelectFont(hOldFont);
	}

	void UpdateLayout_ScrollToFit(RECT rcTabItemArea)
	{
		T* pT = static_cast<T*>(this);
		// When we scroll to fit, we ignore what's passed in for the
		// tab item area rect, and use the client rect instead
		RECT rcClient;
		pT->GetClientRect(&rcClient);

		CClientDC dc(pT->m_hWnd);
		//HFONT hOldFont = dc.SelectFont(lpNMCustomDraw->hFontInactive);
		HFONT hOldFont = dc.SelectFont(m_font);

		RECT rcItem = rcClient;
		// rcItem.top and rcItem.bottom aren't really going to change

		// Recalculate tab positions and widths
		// See OnItemPrePaint for a discussion of how CDotNetTabCtrlImpl
		//  interprets margin, padding, etc.
		size_t nCount = pT->GetItemCount();
		int xpos = m_settings.iIndent;
		HFONT hRestoreNormalFont = NULL;
		for( size_t i=0; i<nCount; ++i )
		{
			bool bSelected = pT->GetSel(i);
			if(bSelected)
			{
				//hRestoreNormalFont = dc.SelectFont(lpNMCustomDraw->hFontSelected);
				hRestoreNormalFont = dc.SelectFont(m_fontSel);
			}

			rcItem.left = rcItem.right = xpos;
			TItem* pItem = pT->GetItem(i);
			int width = UpdateTabRect(pItem, dc.m_hDC, rcItem, m_settings);
			xpos += width;

			if(hRestoreNormalFont != NULL)
			{
				dc.SelectFont(hRestoreNormalFont);
				hRestoreNormalFont = NULL;
			}
		}
		xpos += m_settings.iIndent;

		// If we've been scrolled to the left, and resize so
		// there's more client area to the right, adjust the
		// scroll offset accordingly.
		if((xpos + pT->GetScrollOffset()) < rcTabItemArea.right)
		{
			pT->SetScrollOffset(rcTabItemArea.right - xpos);
		}

		dc.SelectFont(hOldFont);
	}

private:
	static int UpdateTabRect(TItem* pItem, CDCHandle dc, CRect rcItem, const Settings& setting)
	{
		//rcItem.right += ((bSelected ? setting.iSelMargin : setting.iMargin));
		rcItem.right += setting.iMargin;
		if(pItem->UsingIcon())
		{
			rcItem.right += pItem->GetIconSize().cx;
		}
		if(pItem->UsingText())
		{
			RECT rcText = {0};
			dc.DrawText(pItem->GetText(), -1, &rcText, DT_SINGLELINE | DT_CALCRECT);
			rcItem.right += (rcText.right - rcText.left) + (setting.iPadding * 2);
		}
		rcItem.right += setting.iMargin;
		pItem->SetRect(rcItem);
		return rcItem.Width();
	}
};

typedef CWinTraits < WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CTCS_BOLDSELECTEDTAB | CTCS_TOOLTIPS | CTCS_SCROLL | CTCS_CLOSEBUTTON, 0> CDotNetTabTraits;

template <
	class TItem = CDotNetTabItem,
	class TBase = CWindow,
	class TWinTraits = CDotNetTabTraits>
class CDotNetTabCtrl :
	public CCustomTabCtrl<CDotNetTabCtrl, TItem, TBase, TWinTraits, CDotNetTabLookImpl>
{
public:
	DECLARE_WND_CLASS(_T("WTL_DotNetTabCtrl"))
};


// Overrides for painting from CCustomTabCtrl
#endif // __DOTNET_TABCTRL_H__

