// DragDrop.h
#pragma once

#include "ShellUtil.h"

namespace mew {

// IӁI
// CoInitialize() ł͂ȂA
// OleInitialize() gKv邱ƂɒӂĂB

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CDropTargetImpl<T> : OLE Drag&Drop Target
// CDropSourceImpl<T> : OLE Drag&Drop Source

/////////////////////////////////////////////////////////////////////////////

template < class T, DWORD t_dwDefaultOperation = DROPEFFECT_COPY >
class CDropTargetImpl : public IDropTarget
{
private:
	typedef CDropTargetImpl<T, t_dwDefaultOperation>	ThisClass;

protected:
	// IDropTargetHelpergƁAGNXv[̃t@C̃hbOɉ摜\B
	CComPtr<IDropTargetHelper> m_pDropTargetHelper;
	DWORD	m_dwDefaultOperation;

public: // Iy[V
	HRESULT RegisterDragDrop()
	{
		HWND hWnd = static_cast<HWND>(*static_cast<T*>(this));
		HRESULT hr = ::RegisterDragDrop(hWnd, this);
		if FAILED(hr)
			ATLTRACE("RegisterDragDrop failed.\n");
		return hr;
	}
	HRESULT RevokeDragDrop()
	{
		HWND hWnd = static_cast<HWND>(*static_cast<T*>(this));
		return ::RevokeDragDrop(hWnd);
	}
	void SetDefaultOperation(DWORD dwOperation)
	{
		m_dwDefaultOperation = dwOperation;
	}
	// Pȃt@Chbv󂯓ꍇ́AOnDrop()ł̃\bhĂłB
	// etH[}bgAOnDropFile() R[obN܂B
	// CFSTR_SHELLIDLIST, CFSTR_INETURL, CF_HDROP ɑΉĂ܂B
	HRESULT ProcessDropFiles(IDataObject* pDataObject, DWORD key, CPoint ptScreen, DWORD& effect)
	{
		static CFormatEtc FORMAT_IDLIST (CFSTR_SHELLIDLIST, TYMED_HGLOBAL);
		static CFormatEtc FORMAT_INETURL(CFSTR_INETURL	  , TYMED_HGLOBAL);
		static CFormatEtc FORMAT_HDROP  (CF_HDROP	      , TYMED_HGLOBAL);
		T* pThis = static_cast<T*>(this);
		STGMEDIUM medium;
		if SUCCEEDED(pDataObject->GetData(&FORMAT_IDLIST, &medium))
		{
			ATLTRACE(_T("ProcessDropFiles() as SHELLIDLIST\n"));
			const CShellItemList* pItems = (const CShellItemList*)::GlobalLock(medium.hGlobal);
			if(pItems)
			{
				UINT cFiles = pItems->GetCount();
				for(UINT i = 0; i < cFiles; i++)
				{
					CAutoPtr<CShellItemTask> pFullPath = CShellItemTask::Concat(pItems->GetParent(), pItems->GetAt(i));
					pThis->OnDropFile(NULL, pFullPath, key, ptScreen, effect);
				}
				::GlobalUnlock(medium.hGlobal);
			}
			ReleaseStgMedium(&medium);
			return S_OK;
		}
		else if SUCCEEDED(pDataObject->GetData(&FORMAT_INETURL, &medium))
		{
			ATLTRACE(_T("ProcessDropFiles() as INETURL\n"));
			LPCTSTR url = (LPCTSTR)::GlobalLock(medium.hGlobal);
			if(url)
			{
				pThis->OnDropFile(url, NULL, key, ptScreen, effect);
				::GlobalUnlock(medium.hGlobal);
			}
			ReleaseStgMedium(&medium);
			return S_OK;
		}
		else if SUCCEEDED(pDataObject->GetData(&FORMAT_HDROP, &medium))
		{
			ATLTRACE(_T("ProcessDropFiles() as HDROP\n"));
			HDROP hDrop = (HDROP)::GlobalLock(medium.hGlobal);
			if(hDrop)
			{
				UINT cFiles = ::DragQueryFile(hDrop, 0xFFFFFFFF, 0, 0);
				for(UINT i = 0; i < cFiles; i++)
				{
					TCHAR path[MAX_PATH];
					if(::DragQueryFile(hDrop, i, path, MAX_PATH) > 0)
						pThis->OnDropFile(path, NULL, key, ptScreen, effect);
				}
				::GlobalUnlock(medium.hGlobal);
			}
			ReleaseStgMedium(&medium);
			return S_OK;
		}
		else
			return E_FAIL;
	}

public: // IDropTarget
	STDMETHODIMP QueryInterface(REFIID riid, void** ppvObject)
	{
		if(!ppvObject)
			return E_POINTER;
		if(riid == __uuidof(IDropTarget) || riid == __uuidof(IUnknown))
		{
			*ppvObject = static_cast<IDropTarget*>(this);
			AddRef();
			return S_OK;
		}
		*ppvObject = NULL;
		return E_NOINTERFACE;
	}
	STDMETHODIMP_(ULONG) AddRef()	{ return 1; }
	STDMETHODIMP_(ULONG) Release()	{ return 1; }
	STDMETHODIMP DragEnter(IDataObject* pDataObject, DWORD key, POINTL pt, DWORD* pdwEffect)
	{
		T* pThis = static_cast<T*>(this);
		if(!m_pDropTargetHelper)
			CoCreateInstance(CLSID_DragDropHelper, NULL, CLSCTX_INPROC_SERVER, IID_IDropTargetHelper, (LPVOID*)&m_pDropTargetHelper);
		if(m_pDropTargetHelper)
			m_pDropTargetHelper->DragEnter(static_cast<HWND>(*pThis), pDataObject, (POINT*)&pt, *pdwEffect);
		pThis->ModifierToDropEffect(key, pdwEffect);
		pThis->OnDragEnter(pDataObject, key, (CPoint&)pt, *pdwEffect);
		return (*pdwEffect) ? S_OK : E_FAIL;
	}
	STDMETHODIMP DragOver(DWORD key, POINTL pt, DWORD* pdwEffect)
	{
		T* pThis = static_cast<T*>(this);
		if(m_pDropTargetHelper)
			m_pDropTargetHelper->DragOver((POINT*)&pt, *pdwEffect);
		pThis->ModifierToDropEffect(key, pdwEffect);
		pThis->OnDragOver(key, (CPoint&)pt, *pdwEffect);
		return S_OK;
	}
	STDMETHODIMP DragLeave()
	{
		T* pThis = static_cast<T*>(this);
		if(m_pDropTargetHelper)
			m_pDropTargetHelper->DragLeave();
		pThis->OnDragLeave();
		return S_OK;
	}
	STDMETHODIMP Drop(IDataObject* pDataObject, DWORD key, POINTL pt, DWORD * pdwEffect)
	{
		T* pThis = static_cast<T*>(this);
		if(m_pDropTargetHelper)
			m_pDropTargetHelper->Drop(pDataObject, (POINT*)&pt, *pdwEffect);
		pThis->ModifierToDropEffect(key, pdwEffect);
		pThis->OnDrop(pDataObject, key, (CPoint&)pt, *pdwEffect);
		*pdwEffect = DROPEFFECT_NONE;
		return S_OK;
	}

public: // overridable
	void ModifierToDropEffect(DWORD key, DWORD* pdwEffect)
	{
		// ALT or CTRL+SHIFT -- DROPEFFECT_LINK
		// CTRL              -- DROPEFFECT_COPY
		// SHIFT             -- DROPEFFECT_MOVE
		// no modifier       -- m_dwDefaultOperation
		DWORD op;
		if(key & MK_CONTROL)
			if(key & MK_SHIFT)
				op = DROPEFFECT_LINK;
			else
				op = DROPEFFECT_COPY;
		else if(key & MK_SHIFT)
			op = DROPEFFECT_MOVE;
		else if(key & MK_ALT)
			op = DROPEFFECT_LINK;
		else
			op = m_dwDefaultOperation;
		*pdwEffect = (op & *pdwEffect);
	}
	void OnDragEnter(IDataObject* pDataObject, DWORD key, CPoint pt, DWORD& effect)
	{
		const CFormatEtc* fmt;
		ULONG num;
		T::GetFormatMap(&fmt, &num);
		CComPtr<IEnumFORMATETC> pEnum;
		pDataObject->EnumFormatEtc(DATADIR_GET, &pEnum);
		FORMATETC etc;
		ULONG fetched;
		while(pEnum->Next(1, &etc, &fetched) == S_OK)
		{
			for(size_t i = 0; i < num; i++)
			{
				if(etc.cfFormat == fmt[i].cfFormat && (etc.tymed & fmt[i].tymed) != 0)
				{
					if(etc.cfFormat == ::RegisterClipboardFormat(CFSTR_INETURLA)
						|| etc.cfFormat == ::RegisterClipboardFormat(CFSTR_INETURLW))
					{	// C^lbgV[gJbg̏ꍇ́AN󂯕tȂB
						m_dwDefaultOperation = DROPEFFECT_LINK | DROPEFFECT_SCROLL;
					}
					else
					{	// ȊȌꍇ̓ftHg̃Iy[VB
						m_dwDefaultOperation = t_dwDefaultOperation | DROPEFFECT_SCROLL;
					}
					effect = m_dwDefaultOperation;
					return;
				}
			}
		}
		effect = DROPEFFECT_NONE;
	}
	void OnDragOver(DWORD key, CPoint pt, DWORD& effect) {}
	void OnDragLeave() {}
	void OnDrop(IDataObject* pDataObject, DWORD key, CPoint ptScreen, DWORD& effect)
	{
		ATLTRACE("CDropTargetImpl::OnDrop()\n");
	}
	void OnDropFile(LPCTSTR filename, const CShellItem* pItem, DWORD key, CPoint ptScreen, DWORD& effect)
	{
#ifdef _DEBUG
		if(filename)
		{
			ATLTRACE(_T("WARNING: OnDropFile(%s)\n"), filename);
		}
		else if(pItem)
		{
			SHFILEINFO file;
			pItem->GetFileInfo(&file, SHGFI_DISPLAYNAME);
			ATLTRACE(_T("WARNING: OnDropFile(%s)\n"), file.szDisplayName);
		}
#endif
	}
};

/////////////////////////////////////////////////////////////////////////////

template < class T >
class CDropSourceImpl : public IDropSource
{
private:
	CComPtr<IDragSourceHelper> m_pDragSourceHelper;

public:
	DWORD DoDragDrop(IDataObject * pDataObject, DWORD dwOKEffect)
	{
		ATLTRACE("DoDragDrop Begin\n");
		DWORD dwResult = 0;
		::DoDragDrop(pDataObject, this, dwOKEffect, &dwResult);
		ATLTRACE("DoDragDrop End\n");
		return dwResult;
	}

	DWORD DoDragDrop(IDataObject * pDataObject, DWORD dwOKEffect, HWND hWndDrag, POINT pt)
	{
		if(!m_pDragSourceHelper)
			CoCreateInstance(CLSID_DragDropHelper, NULL, CLSCTX_INPROC_SERVER, IID_IDragSourceHelper, (void**)&m_pDragSourceHelper);
		if(m_pDragSourceHelper)
			m_pDragSourceHelper->InitializeFromWindow(hWndDrag, &pt, pDataObject);
		DWORD dwResult = DoDragDrop(pDataObject, dwOKEffect);
		return dwResult;
	}

	DWORD DoDragDrop(IDataObject * pDataObject, DWORD dwOKEffect, LPSHDRAGIMAGE pshdi)
	{
		if(!m_pDragSourceHelper)
			CoCreateInstance(CLSID_DragDropHelper, NULL, CLSCTX_INPROC_SERVER, IID_IDragSourceHelper, (void**)&m_pDragSourceHelper);
		if(m_pDragSourceHelper)
			m_pDragSourceHelper->InitializeFromBitmap(pshdi, pDataObject);
		DWORD dwResult = DoDragDrop(pDataObject, dwOKEffect);
		return dwResult;
	}

	STDMETHODIMP QueryInterface(REFIID riid, void** ppvObject)
	{
		if(!ppvObject)
			return E_POINTER;
		if(riid == __uuidof(IDropSource) || riid == __uuidof(IUnknown))
		{
			*ppvObject = static_cast<IDropSource*>(this);
			AddRef();
			return S_OK;
		}
		*ppvObject = NULL;
		return E_NOINTERFACE;
	}
	STDMETHODIMP_(ULONG) AddRef()	{ return 1; }
	STDMETHODIMP_(ULONG) Release()	{ return 1; }

	STDMETHODIMP QueryContinueDrag(BOOL fEscapePressed, DWORD grfKeyState)
	{
		// uQȏ̃}EX{^ĂvƃX}[gɔłȂH
		if (fEscapePressed
		|| ((grfKeyState & MK_LBUTTON) && (grfKeyState & (MK_MBUTTON | MK_RBUTTON)))
		|| ((grfKeyState & MK_MBUTTON) && (grfKeyState & (MK_LBUTTON | MK_RBUTTON)))
		|| ((grfKeyState & MK_RBUTTON) && (grfKeyState & (MK_LBUTTON | MK_MBUTTON))))
			return DRAGDROP_S_CANCEL;
        if(!(grfKeyState & (MK_LBUTTON | MK_MBUTTON | MK_RBUTTON)))
			return DRAGDROP_S_DROP;
		return S_OK;
	}

	STDMETHODIMP GiveFeedback(DWORD dwEffect)
	{
		return DRAGDROP_S_USEDEFAULTCURSORS;
	}
};

} // namespace mew
