// DropDownToolBar.h
#pragma once

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CDropDownToolBarImpl<T>
// CDropDownToolBar

/////////////////////////////////////////////////////////////////////////////
// ITEM.lParam  HMENU ێĂB

template < class T, class TBase = CToolBarCtrl, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CDropDownToolBarImpl : public CWindowImpl<T, TBase, TWinTraits>
{
public:
	DECLARE_WND_SUPERCLASS(_T("DropDownToolBar"), TBase::GetWndClassName())

public: // I[o[Ch
	void OnDropDown(WORD wID, CMenuHandle popup, INT x, INT y)
	{
		popup.TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, x, y, GetTopLevelParent());
	}

public:	// Iy[V
	HWND Create(
		HWND hWndParent,
		UINT nResourceID,
		LPCTSTR lpszTitle,
		DWORD dwStyle = ATL_SIMPLE_TOOLBAR_STYLE,
		UINT nID = ATL_IDW_TOOLBAR)
	{
		struct _AtlToolBarData
		{
			WORD wVersion;
			WORD wWidth;
			WORD wHeight;
			WORD wItemCount;
			WORD wItems[1]; // [wItemCount]
		};

		HINSTANCE hResInst = _AtlBaseModule.GetResourceInstance();
		HRSRC hRsrc = ::FindResource(hResInst, MAKEINTRESOURCE(nResourceID), RT_TOOLBAR);
		if (hRsrc == NULL)
			return NULL;

		_AtlToolBarData* pData = (_AtlToolBarData*)::LockResource(::LoadResource(hResInst, hRsrc));
		if (pData == NULL)
			return NULL;
		ATLASSERT(pData->wVersion == 1);

		WORD* pItems = pData->wItems;
		int nItems = pData->wItemCount;
		TBBUTTON* pTBBtn = (TBBUTTON*)_alloca(nItems * sizeof(TBBUTTON));

		int nBmp = 0;
		for(int i = 0; i < pData->wItemCount; i++)
		{
			if(pItems[i] != 0)
			{
				pTBBtn[i].iBitmap = nBmp++;
				pTBBtn[i].idCommand = pItems[i];
				pTBBtn[i].fsState = TBSTATE_ENABLED;
				pTBBtn[i].fsStyle = TBSTYLE_BUTTON;
				pTBBtn[i].dwData = 0;
				pTBBtn[i].iString = 0;
			}
			else
			{
				pTBBtn[i].iBitmap = 8;
				pTBBtn[i].idCommand = 0;
				pTBBtn[i].fsState = 0;
				pTBBtn[i].fsStyle = TBSTYLE_SEP;
				pTBBtn[i].dwData = 0;
				pTBBtn[i].iString = 0;
			}
		}

		HWND hWnd = __super::Create(hWndParent, rcDefault, lpszTitle, dwStyle, 0, nID, NULL);
		SetExtendedStyle(TBSTYLE_EX_DRAWDDARROWS);

		::SendMessage(hWnd, TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0L);

		// check if font is taller than our bitmaps
		CFontHandle font = (HFONT)::SendMessage(hWnd, WM_GETFONT, 0, 0L);
		if(font.IsNull())
			font = AtlGetDefaultGuiFont();
		LOGFONT lf;
		font.GetLogFont(lf);
		WORD cyFontHeight = (WORD)abs(lf.lfHeight);

		if(AtlIsAlphaBitmapResource(nResourceID))	// alpha channel bitmap (valid for Windows XP only)
		{
			HIMAGELIST hImageList = ImageList_LoadImage(hResInst, MAKEINTRESOURCE(nResourceID), pData->wWidth, 1, CLR_NONE, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_DEFAULTSIZE);
			ATLASSERT(hImageList != NULL);
			::SendMessage(hWnd, TB_SETIMAGELIST, 0, (LPARAM)hImageList);
		}
		else
		{
			TBADDBITMAP tbab;
			tbab.hInst = hResInst;
			tbab.nID = nResourceID;
			::SendMessage(hWnd, TB_ADDBITMAP, nBmp, (LPARAM)&tbab);
		}

		::SendMessage(hWnd, TB_ADDBUTTONS, nItems, (LPARAM)pTBBtn);
		::SendMessage(hWnd, TB_SETBITMAPSIZE, 0, MAKELONG(pData->wWidth, max(pData->wHeight, cyFontHeight)));
		::SendMessage(hWnd, TB_SETBUTTONSIZE, 0, MAKELONG(pData->wWidth, max(pData->wHeight, cyFontHeight)));
//		::SendMessage(hWnd, TB_SETBUTTONSIZE, 0, MAKELONG(pData->wWidth + 7, max(pData->wHeight, cyFontHeight) + 7)); // too large

		return hWnd;
	}

	// hMenu ̏L͌ĂяoɂBic[o[jɍ폜Ȃj
	BOOL SetDropDownMenu(UINT idCmd, HMENU hMenu)
	{
		TBBUTTONINFO info = { sizeof(TBBUTTONINFO), TBIF_COMMAND | TBIF_LPARAM | TBIF_STYLE };
		if(!GetButtonInfo(idCmd, &info))
			return FALSE;
		info.lParam = (LPARAM)hMenu;
		if(info.lParam)
			info.fsStyle |= TBSTYLE_DROPDOWN;
		else
			info.fsStyle &= ~TBSTYLE_DROPDOWN;
		return SetButtonInfo(idCmd, &info);
	}

	HMENU GetDropDownMenu(UINT idCmd)
	{
		TBBUTTONINFO info = { sizeof(TBBUTTONINFO), TBIF_COMMAND | TBIF_LPARAM | TBIF_STYLE };
		if(!GetButtonInfo(idCmd, &info))
			return NULL;
		return (HMENU)info.lParam;
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CDropDownToolBarImpl)
		REFLECTED_NOTIFY_CODE_HANDLER(TBN_DROPDOWN, OnDropDown)
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnDropDown(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{
		NMTOOLBAR* notify = (NMTOOLBAR*)pnmh;
		TBBUTTONINFO info = { sizeof(TBBUTTONINFO), TBIF_COMMAND | TBIF_LPARAM };
		CRect rc;
		if(GetButtonInfo(notify->iItem, &info) && info.lParam && GetRect(notify->iItem, &rc))
		{
			CMenuHandle popup = (HMENU)info.lParam;
			ClientToScreen(&rc);
			T* pT = static_cast<T*>(this);
			pT->OnDropDown(notify->iItem, popup, rc.left, rc.bottom);
		}
		return 0;
	}
};

/////////////////////////////////////////////////////////////////////////////

class CDropDownToolBar : public CDropDownToolBarImpl<CDropDownToolBar>
{
};

} // namespace mew
