// CFileTreeCtrl.h
#pragma once

#include "DragDrop.h"
#include "MenuHandler.h"
#include "TreeImpl.h"
#include "Collection.h"

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CFileTreeCtrlItem
// CFileTreeCtrlImpl<T, TItem, TBase, TWinTraits>
// CFileTreeCtrl

////////////////////////////////////////////////////////////////////////

struct CFileTreeCtrlItem
{
	CShellItemTask*	pShellItem;
	bool isValid;

	CFileTreeCtrlItem()
	{
		pShellItem = NULL;
		isValid = false;
	}
	~CFileTreeCtrlItem()
	{
		delete pShellItem;
	}
};

template <class T, class TItem = CFileTreeCtrlItem, class TBase = CTreeViewCtrl, class TWinTraits = CControlWinTraits>
class ATL_NO_VTABLE CFileTreeCtrlImpl :
	public CTreeImpl<T, TItem, TBase, TWinTraits>,
	public CDropTargetImpl<T, DROPEFFECT_MOVE>,
	public CDropSourceImpl<T>,
	public CShellMenuHandler<0x0001, 0x3FFF, CMenuHandler<T> >
{
	typedef CTreeImpl<T, TItem, TBase, TWinTraits>	baseClass;
	typedef CShellMenuHandler<0x0001, 0x3FFF, CMenuHandler<T> >	baseMenu;


private: // ϐ
	DWORD	m_FileEnumFlag;

public:
	CFileTreeCtrlImpl()
	{
		m_FileEnumFlag = FileEnumFlag_IncludeFolders | FileEnumFlag_SortFoldersOnTop;
	}

public: // I[o[Ch
	// |bvAbvj[ɓƎ̍ڂǉȂǂ̏ꍇɃI[o[Ch܂B
	// TPM_RETURNCMD ܂݁ATPM_NONOTIFY ܂܂Ȃ悤ɂĂB
	// TrackPopupMenu()̕ԒliR}hIDjԂ܂B
	// ftHg̃VFj[ڂ́AĂяoŏ܂B
	UINT PopupContextMenu(HTREEITEM hItem, CMenuHandle popup, CPoint ptScreen)
	{
		return popup.TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD | TPM_RIGHTBUTTON, ptScreen.x, ptScreen.y, m_hWnd);
	}

	void OnItemContextMenu(HTREEITEM hItem, CPoint ptScreen)
	{
		ParamType* pItem = GetItemParam(hItem);
		if(!pItem || !pItem->pShellItem)
			return;
		CMenu popup;
		popup.CreatePopupMenu();
		if SUCCEEDED(this->InsertShellMenu(popup, 0, pItem->pShellItem, CMF_EXPLORE, m_hWnd))
		{
			T* pT = static_cast<T*>(this);
			UINT cmd = pT->PopupContextMenu(hItem, popup.m_hMenu, ptScreen);
			this->InvokeCommand(cmd, m_hWnd);
			this->DisposeMenuResource();
		}
	}

public: // Iy[V
	DWORD GetFileEnumFlag() const		{ return m_FileEnumFlag; }
	void SetFileEnumFlag(DWORD style)	{ m_FileEnumFlag = style; }

	HTREEITEM AddFolder(CAutoPtr<CShellItemTask> pRoot, HTREEITEM hParent = NULL)
	{
		if(!pRoot)
			return NULL;
		SHFILEINFO file;
		pRoot->GetFileInfo(&file, SHGFI_PIDL | SHGFI_DISPLAYNAME);
		HTREEITEM hItem = TreeInsertItem(hParent, pRoot, file.szDisplayName, TRUE, FALSE);
		SelectItem(hItem);
		Expand(hItem);
		return hItem;
	}

	bool LoadChildNodes(HTREEITEM hItem, bool reload = false)
	{
		ParamType* pItem = GetItemParam(hItem);
		if(!pItem || !pItem->pShellItem)
			return false;
		if(!reload && pItem->isValid)
			return true;
		if(reload)
			DeleteChildren(hItem);
		CWaitCursor wait;
		CComPtr<IShellFolder> pSelfFolder;
		if FAILED(pItem->pShellItem->GetSelfFolder(&pSelfFolder))
			return false;
		SetRedraw(FALSE);
		AddChildren(hItem, pSelfFolder, pItem->pShellItem);
		pItem->isValid = true;
		SetRedraw(TRUE);
		return true;
	}

	struct FindItemOp
	{
		const CShellItem* pShellItem;
		HTREEITEM hFound;

		FindItemOp(const CShellItem* pItem) : pShellItem(pItem), hFound(NULL)
		{
		}
		bool operator () (T* pThis, HTREEITEM hChild)
		{
			ParamType* pParam = pThis->GetItemParam(hChild);
			if(pShellItem->Equals(pParam->pShellItem))
			{
				hFound = hChild;
				return false;
			}
			return true;
		}
	};

	HTREEITEM FindItem(const CShellItem* pShellItem, HTREEITEM hParent)
	{
		if(pShellItem == NULL)
			return NULL;
		FindItemOp op(pShellItem);
		ForEachChildren(hParent, op);
		return op.hFound;
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CFileTreeCtrlImpl)
		MESSAGE_HANDLER(WM_CREATE	, OnCreate)
		MESSAGE_HANDLER(WM_DESTROY	, OnDestroy)
		REFLECTED_NOTIFY_CODE_HANDLER(TVN_ITEMEXPANDING	, OnItemExpanding)
		REFLECTED_NOTIFY_CODE_HANDLER(TVN_BEGINDRAG		, OnBeginDrag)
		REFLECTED_NOTIFY_CODE_HANDLER(TVN_BEGINRDRAG	, OnBeginDrag)
		CHAIN_MSG_MAP(baseMenu)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
	{
		// __super::OnCreate()
		LRESULT lRet = DefWindowProc(uMsg, wParam, lParam);

		// OnInitialUpdate()
		SHFILEINFO  file;
		CImageList imagelist = (HIMAGELIST)SHGetFileInfo(_T("C:\\"), 0, &file, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX | SHGFI_SMALLICON); // SHGFI_LARGEICON
		SetImageList(imagelist, TVSIL_NORMAL);
		RegisterDragDrop();
		return lRet;
	}

	LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		// c[r[Ɋ֘AtꂽC[WXg͎IɔjȂB
		CImageList imagelist = GetImageList(TVSIL_NORMAL);
		if(imagelist)
			imagelist.Destroy();
		bHandled = FALSE;
		return 0;
	}

	LRESULT OnBeginDrag(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{
		TreeNotify* notify = (TreeNotify*)pnmh;
		ParamType* item = notify->itemNew.pParam;
		if(!item || !item->pShellItem || item->pShellItem->IsRoot()) // [g̃fXNgbv͏O
			return 0;
//		CComPtr<CDataObject> pDataObject = CDataObject::Create();
//		pDataObject->AddShelItemLise(item->pShellItem);
//		DoDragDrop(pDataObject, DROPEFFECT_COPY | DROPEFFECT_MOVE | DROPEFFECT_LINK | DROPEFFECT_SCROLL, notify->hdr.hwndFrom, notify->ptDrag);
		CDataObject_ShellItemList data(item->pShellItem);
		DoDragDrop(&data, DROPEFFECT_COPY | DROPEFFECT_MOVE | DROPEFFECT_LINK | DROPEFFECT_SCROLL, notify->hdr.hwndFrom, notify->ptDrag);
		return 0;
	}

	LRESULT OnItemExpanding(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{
		// TODO: t@CVXeXVꂽꍇ̔fB
		TreeNotify* notify = (TreeNotify*)pnmh;
		if(notify->action == TVE_COLLAPSE)
		{
			Expand(notify->itemNew.hItem, TVE_COLLAPSE);
		}
		else if(notify->action == TVE_EXPAND)
		{
			LoadChildNodes(notify->itemNew.hItem);
			return 0;
		}
		return 0;
	}

public: // CDropTargetImpl
	BEGIN_FORMAT_MAP(CFileTreeCtrlImpl)
		FORMAT_STANDARD_FILES()
	END_FORMAT_MAP()

	void OnDragOver(DWORD key, CPoint pt, DWORD& effect)
	{
		ScreenToClient(&pt);
		TVHITTESTINFO hit;
		hit.pt = pt;
		HTREEITEM hItem = HitTest(&hit);
		if(hItem != NULL)
		{
			SelectDropTarget(hItem);
		}
	}
	void OnDragLeave()
	{
		SelectDropTarget(NULL);
	}
	void OnDrop(IDataObject* pDataObject, DWORD key, CPoint ptScreen, DWORD& effect)
	{
		ProcessDropFiles(pDataObject, key, ptScreen, effect);
		SelectDropTarget(NULL);
	}
/*	void OnDropFile(LPCTSTR filename, const CShellItem* pItem, DWORD key, CPoint ptScreen, DWORD& effect)
	{
		// TODO: t@C̃Rs[EړENB
	}
*/
private:
	HTREEITEM TreeInsertItem(HTREEITEM hParent, CAutoPtr<CShellItemTask>& pFullPath, LPCTSTR name, BOOL hasChildren, BOOL shared)
	{
		SHFILEINFO file;

		T* pThis = static_cast<T*>(this);
		CAutoPtr<ParamType> pItem = pThis->AllocItem();
		pItem->pShellItem = pFullPath.Detach();
		pItem->isValid    = false;

		TVINSERTSTRUCT insert = { hParent, TVI_LAST };
		insert.item.mask      = TVIF_PARAM | TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_CHILDREN;
		insert.item.pszText   = const_cast<LPTSTR>(name);
		pItem->pShellItem->GetFileInfo(&file, SHGFI_PIDL | SHGFI_SYSICONINDEX | SHGFI_SMALLICON | SHGFI_LINKOVERLAY);
		insert.item.iImage    = file.iIcon;
		pItem->pShellItem->GetFileInfo(&file, SHGFI_PIDL | SHGFI_SYSICONINDEX | SHGFI_SMALLICON | SHGFI_OPENICON);
		insert.item.iSelectedImage = file.iIcon;
		insert.item.lParam    = (LPARAM)pItem.Detach();
		insert.item.cChildren = hasChildren;
		if(shared)
		{
			insert.item.mask      |= TVIF_STATE;
			insert.item.stateMask |= TVIS_OVERLAYMASK;
			insert.item.state     |= INDEXTOOVERLAYMASK(1);
		}
		return InsertItem(&insert);
	}

	bool AddChildren(HTREEITEM hParentItem, IShellFolder* pParentFolder, CShellItem* pidlParent)
	{
		// q̐グBグ̏ԂGNXv[Ił͂Ȃ̂ŁA\[gKvB
		CShellItemTask::EnumResult items;
		items.reserve(64);
		HRESULT hr = CShellItemTask::EnumChildren(pParentFolder, m_FileEnumFlag, items);
		if(hr != S_OK)
			return false;

		// ǉ
		for(size_t i = 0; i < items.size(); i++)
		{
			DWORD dwAttrs = items[i]->Attr;
			BOOL hasChildren = (m_FileEnumFlag & SHCONTF_NONFOLDERS) ? (dwAttrs & SFGAO_FOLDER) : (dwAttrs & SFGAO_HASSUBFOLDER);
			HTREEITEM hItem = TreeInsertItem(
				hParentItem,
				CShellItemTask::Concat(pidlParent, items[i]->pLeaf),
				items[i]->Name,
				hasChildren,
				dwAttrs & SFGAO_SHARE);
		}

		return true;
	}
};

////////////////////////////////////////////////////////////////////////

class CFileTreeCtrl : public CFileTreeCtrlImpl<CFileTreeCtrl>
{
public:
	DECLARE_WND_SUPERCLASS(_T("FileTreeCtrl"), GetWndClassName())
};

} // namespace mew
