// FileMenu.h
#pragma once

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CMenuHandler<T>
// CStringTableMenuHandler<TBase>		: e[uR}hIDƈv镶擾B
// CFileMenuHandler<UINT, UINT, TBase>	: t@CKwj[𓮓IɐB
// CShellMenuHandler<UINT, UINT, TBase>	: IContextMenu3 B

////////////////////////////////////////////////////////////////////////

template < class T >
class ATL_NO_VTABLE CMenuHandler
{
public: // I[o[Ch
	HRESULT OnGetMenuItemName(DWORD wID, HMENU hMenu, LPTSTR name, INT cch)
	{
		return E_NOTIMPL;
	}

	STDMETHODIMP GetControlWindow(UINT id, HWND* phWnd)
	{
		return E_NOTIMPL;
	}

public: // Iy[V
	void DisposeMenuResource()
	{
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CMenuHandler)
		MESSAGE_HANDLER(WM_MENUSELECT	, OnMenuSelect)
	END_MSG_MAP()

	LRESULT OnMenuSelect(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		T* pT = static_cast<T*>(this);

		// Xe[^Xo[擾
		CStatusBarCtrl statusbar;
		if(FAILED(pT->GetControlWindow(FCW_STATUS, &statusbar.m_hWnd)) || !statusbar.IsWindow())
			return 1;

		WORD wFlags = HIWORD(wParam);
		if(wFlags == 0xFFFF && lParam == NULL)	// menu closing
			statusbar.SetSimple(FALSE);
		else
 		{
			DWORD wID = LOWORD(wParam);
			TCHAR szItemName[MAX_PATH];
			szItemName[0] = 0;
			if(wFlags & MF_POPUP)
				pT->OnGetMenuItemName(wID, ::GetSubMenu((HMENU)lParam, wID), szItemName, MAX_PATH);
			else
				pT->OnGetMenuItemName(wID, 0, szItemName, MAX_PATH);
			statusbar.SetSimple(TRUE);
			statusbar.SetText(255, szItemName, SBT_NOBORDERS);
		}

		return 1;
	}
};

///////////////////////////////////////////////////////////////////////

template < class TBase >
class ATL_NO_VTABLE CStringTableMenuHandler : public TBase
{
public:
	CStringTableMenuHandler()
	{
	}

public: // I[o[Ch
	HRESULT OnGetMenuItemName(DWORD wID, HMENU hMenu, LPTSTR name, INT cch)
	{
		if(hMenu == NULL)
		{
			// check for special cases
			if(wID >= 0xF000 && wID < 0xF1F0)	// system menu IDs
				wID = (WORD)(((wID - 0xF000) >> 4) + ATL_IDS_SCFIRST);
			else if(wID >= ID_FILE_MRU_FIRST && wID <= ID_FILE_MRU_LAST)	// MRU items
				wID = ATL_IDS_MRU_FILE;
			else if(wID >= ATL_IDM_FIRST_MDICHILD)	// MDI child windows
				wID = ATL_IDS_MDICHILD;

			int nRet = ::LoadString(_AtlBaseModule.GetResourceInstance(), wID, name, cch);
			for(int i = 0; i < nRet; i++)
			{
				if(name[i] == _T('\n'))
				{
					name[i] = 0;
					return S_OK;
				}
			}
		}
		return __super::OnGetMenuItemName(wID, hMenu, name, cch);
	}
};

///////////////////////////////////////////////////////////////////////

template < UINT t_uCommandFirst, UINT t_uCommandLast, class TBase >
class ATL_NO_VTABLE CFileMenuHandler : public TBase
{
private:
	typedef CSimpleMap<HMENU, const CShellItemTask*>	CMenuMap; // QȂ̂SimpleMapŉƂȂ邾낤B
	typedef CAutoPtrArray<const CShellItemTask>		CItemArray;
	CMenuMap	m_MenuToItem;
	CItemArray	m_IdToItem;
	DWORD		m_FileEnumFlag;

public:
	CFileMenuHandler()
	{
		m_FileEnumFlag = FileEnumFlag_IncludeFolders | FileEnumFlag_SortFoldersOnTop;
	}
	~CFileMenuHandler()
	{
		_Dispose();
	}

public: // I[o[Ch
	HRESULT OnGetMenuItemName(DWORD wID, HMENU hMenu, LPTSTR name, INT cch)
	{
		size_t index;
		if(hMenu == NULL)
			index = wID - t_uCommandFirst;
		else if(EnsureFolderMenu(hMenu, wID == 1))
			index = ::GetMenuItemID(hMenu, 0) - t_uCommandFirst;
		else
			return __super::OnGetMenuItemName(wID, hMenu, name, cch);
		//
		if(index < m_IdToItem.GetCount())
		{
			m_IdToItem[index]->GetPath(name);
			return S_OK;
		}
		else
			return __super::OnGetMenuItemName(wID, hMenu, name, cch);
	}

public: // Iy[V
	DWORD GetFileEnumFlag() const		{ return m_FileEnumFlag; }
	void SetFileEnumFlag(DWORD style)	{ m_FileEnumFlag = style; }

	BOOL InsertFolderMenu(HMENU hMenu, LPCTSTR name, const CShellItem* pShellItem, UINT uPosition = -1, UINT uFlags = MF_BYPOSITION)
	{
		CAutoPtr<const CShellItemTask> item = CShellItemTask::Duplicate(pShellItem);
		HMENU hSubMenu = InsertSubMenu(hMenu, name, item, uPosition, uFlags);
		EnsureFolderMenu(hSubMenu, true);
		return TRUE;
	}

	const CShellItem* LookupShellItem(UINT wID)
	{
		UINT index = wID - t_uCommandFirst;
		if(index >= m_IdToItem.GetCount())
			return NULL;
		else
			return m_IdToItem[index];
	}

	void DisposeMenuResource()
	{
		ATLTRACE("CFileMenuHandler.DisposeMenuResource() : MenuToItem=%d, IdToItem=%d\n", m_MenuToItem.GetSize(), m_IdToItem.GetCount());
		_Dispose();
		__super::DisposeMenuResource();
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CFileMenuHandler)
		MESSAGE_HANDLER(WM_INITMENUPOPUP, OnInitMenuPopup)
		CHAIN_MSG_MAP(__super)
	END_MSG_MAP()

	LRESULT OnInitMenuPopup(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		HMENU hMenu = (HMENU)wParam;
		UINT index = LOWORD(lParam);
		// gbL[B̊Kwւ̃j[́AKQԖځiindex==1jł邱Ƃ𗘗pĂB
		EnsureFolderMenu(hMenu, index == 1);
		bHandled = FALSE;
		return 0;
	}

private: // Cve[V
	void _Dispose()
	{
		int size = m_MenuToItem.GetSize();
		for(int i = 0; i < size; i++)
			delete m_MenuToItem.GetValueAt(i);
		m_MenuToItem.RemoveAll();
		m_IdToItem.RemoveAll();
	}

	HMENU InsertSubMenu(HMENU hMenu, LPCTSTR name, CAutoPtr<const CShellItemTask>& pItem, UINT uPosition = -1, UINT uFlags = MF_BYPOSITION)
	{
		HMENU hSubMenu = ::CreateMenu();
		m_MenuToItem.Add(hSubMenu, pItem.Detach());
		::InsertMenu(hMenu, uPosition, MF_POPUP | uFlags, (UINT_PTR)hSubMenu, name);
		return hSubMenu;
	}

	UINT InsertMenuItem(HMENU hMenu, LPCTSTR name, CAutoPtr<const CShellItemTask>& pItem, UINT uPosition = -1, UINT uFlags = MF_BYPOSITION)
	{
		UINT nCount = (UINT)m_IdToItem.GetCount();
		UINT nCmd = t_uCommandFirst + nCount;
		if(nCmd > t_uCommandLast)
		{
			::InsertMenu(hMenu, uPosition, uFlags, 0, _T("iȏ\ł܂j"));
			return FALSE;
		}
		m_IdToItem.Add(pItem);
		::InsertMenu(hMenu, uPosition, uFlags, nCmd, name);
		return nCmd;
	}

	bool EnsureFolderMenu(HMENU hMenu, bool upper)
	{
		// Ɏq or MenuToItem̂ȂɃj[nhȂ ȂΒǉ̕KvȂƂ݂ȂB
		int count = ::GetMenuItemCount(hMenu);
		if(count > 0)
			return true;
		const CShellItemTask* pItem = m_MenuToItem.Lookup(hMenu);
		if(!pItem)
			return true;
		// j[쐬.
		CWaitCursor wait;
		m_MenuToItem.Remove(hMenu); // Ȃ
		TCHAR name[MAX_PATH];
		HRESULT hr;
		// 
		if FAILED(hr = pItem->GetProperty(name))
			return false;
		InsertMenuItem(hMenu, name, CAutoPtr<const CShellItemTask>(pItem));
		::SetMenuDefaultItem(hMenu, 0, TRUE);
		// éAŏʂ̃j[ɂΏ\
		if(!pItem->IsRoot() && upper)
		{
			CAutoPtr<const CShellItemTask> item = pItem->GetParent();
			InsertSubMenu(hMenu, _T(".."), item);
		}
		// q
		CComPtr<IShellFolder> pSelfFolder;
		if FAILED(hr = pItem->GetSelfFolder(&pSelfFolder))
			return false;
		CShellItemTask::EnumResult items;
		items.reserve(64);
		hr = CShellItemTask::EnumChildren(pSelfFolder, m_FileEnumFlag, items);
		if(hr == S_OK)
		{
			::AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
			for(CShellItemTask::EnumResult::iterator i = items.begin(); i != items.end(); i++)
			{
				DWORD dwAttrs = (*i)->Attr;
				BOOL hasChildren = (m_FileEnumFlag & SHCONTF_NONFOLDERS) ? (dwAttrs & SFGAO_FOLDER) : (dwAttrs & SFGAO_HASSUBFOLDER);
				CAutoPtr<const CShellItemTask> item = CShellItemTask::Concat(pItem, (*i)->pLeaf);
				if(hasChildren)
					InsertSubMenu(hMenu, (*i)->Name, item);
				else
					InsertMenuItem(hMenu, (*i)->Name, item);
			}
		}
		return true;
	}
};

////////////////////////////////////////////////////////////////////////

template < UINT t_uCommandFirst, UINT t_uCommandLast, class TBase >
class ATL_NO_VTABLE CShellMenuHandler : public TBase
{
private:
	CComPtr<IContextMenu3> m_pContextMenu3;

public: // I[o[Ch
	HRESULT OnGetMenuItemName(DWORD wID, HMENU hMenu, LPTSTR name, INT cch)
	{
		if(hMenu == NULL && t_uCommandFirst <= wID && wID <= t_uCommandLast)
		{
			// XP ɂāAGCS_HELPTEXTA 擾悤ƂƕB
			// dȂ̂ŁA_UNICODE ɂ炸 GCS_HELPTEXTW Ŏ擾邱ƂɂB
			//m_pContextMenu3->GetCommandString(wID - t_uCommandFirst, GCS_HELPTEXT, NULL, (LPSTR)name, 255);
#ifdef _UNICODE
			m_pContextMenu3->GetCommandString(wID - t_uCommandFirst, GCS_HELPTEXTW, NULL, (LPSTR)name, 255);
#else
			WCHAR buffer[128];
			if SUCCEEDED(m_pContextMenu3->GetCommandString(wID - t_uCommandFirst, GCS_HELPTEXTW, NULL, (LPSTR)buffer, 127))
			{
				WideCharToMultiByte(CP_ACP, 0, buffer, -1, name, cch, 0, 0);
				return S_OK;
			}
#endif
		}
		return __super::OnGetMenuItemName(wID, hMenu, name, cch);
	}

public: // Iy[V
	HRESULT InsertShellMenu(HMENU hMenu, UINT insertIndex, const CShellItem* pItem, UINT flags, HWND hWnd = ::GetActiveWindow())
	{
		CComPtr<IContextMenu> pContextMenu1;
		HRESULT hr = pItem->GetUIObjectOf(hWnd, &pContextMenu1, IID_IContextMenu);
		if FAILED(hr)
			return hr;
		return InsertShellMenu(hMenu, insertIndex, pContextMenu1, flags);
	}

	HRESULT InsertShellMenu(HMENU hMenu, UINT insertIndex, IShellView* pShellView, UINT uItem, UINT flags)
	{
		CComPtr<IContextMenu> pContextMenu1;
		HRESULT hr = pShellView->GetItemObject(uItem, IID_IContextMenu, (void**)&pContextMenu1);
		if FAILED(hr)
			return hr;
		return InsertShellMenu(hMenu, insertIndex, pContextMenu1, flags);
	}

	HRESULT InsertShellMenu(HMENU hMenu, UINT insertIndex, IContextMenu* pContextMenu1, UINT flags)
	{
		ATLASSERT(!m_pContextMenu3);
		m_pContextMenu3.Release(); //  ʂ͉ς݂Ȃ̂ŕKvȂB
		pContextMenu1->QueryInterface(IID_IContextMenu3, (void**)&m_pContextMenu3);
		HRESULT hr = pContextMenu1->QueryContextMenu(hMenu, 0, t_uCommandFirst, t_uCommandLast, CMF_EXPLORE);
		if FAILED(hr)
		{
			m_pContextMenu3.Release();
			return hr;
		}
		return S_OK;
	}

	HRESULT InvokeCommand(UINT cmd, HWND hWnd = ::GetActiveWindow())
	{
		HRESULT hr = S_FALSE;
		if(m_pContextMenu3 && t_uCommandFirst <= cmd && cmd <= t_uCommandLast)
		{
			CMINVOKECOMMANDINFO cmi;
			cmi.cbSize       = sizeof(CMINVOKECOMMANDINFO);
			cmi.fMask        = 0;
			cmi.hwnd         = hWnd;
			cmi.lpVerb       = MAKEINTRESOURCEA(cmd - t_uCommandFirst);
			cmi.lpParameters = NULL;
			cmi.lpDirectory  = NULL;
			cmi.nShow        = SW_SHOWNORMAL;
			cmi.dwHotKey     = 0;
			cmi.hIcon        = NULL;
			hr = m_pContextMenu3->InvokeCommand(&cmi);
		}
		return hr;
	}

	void DisposeMenuResource()
	{
		m_pContextMenu3.Release();
		__super::DisposeMenuResource();
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CShellMenuHandler)
		MESSAGE_HANDLER(WM_INITMENUPOPUP, OnHandleMenuMsg)
		MESSAGE_HANDLER(WM_DRAWITEM		, OnHandleMenuMsg)
		MESSAGE_HANDLER(WM_MENUCHAR		, OnHandleMenuMsg)
		MESSAGE_HANDLER(WM_MEASUREITEM	, OnHandleMenuMsg)
		CHAIN_MSG_MAP(__super)
	END_MSG_MAP()

	LRESULT OnHandleMenuMsg(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		LRESULT lResult = 0;
		if(m_pContextMenu3)
		{
			m_pContextMenu3->HandleMenuMsg2(uMsg, wParam, lParam, &lResult);
		}
		bHandled = FALSE;
		return lResult;
	}
};

////////////////////////////////////////////////////////////////////////

} // namespace mew
