// MouseGesture.h : from DonutP
#pragma once

#include "Collection.h"

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CMouseGestureImpl<T, t_dwMsgMapID> : }EXWFX`̃x[XNX

enum Gesture
{
	GESTURE_NORTH,	// 00
	GESTURE_SOUTH,	// 01
	GESTURE_WEST,	// 10
	GESTURE_EAST,	// 11
};

const LPCTSTR GESTURE_CHAR[] =
{
	_T(""), _T(""), _T(""), _T("")
};

const int GESTURE_SKIP_DISTANCE	= 10; // [px] ̋ȉ̃}EXړ͊mȂ
const int GESTURE_ALT_TIME = 300;	// [msec] ȏ㎞Ԃoꍇ͐VȃWFX`Ƃ݂ȂiȂǁj

template < class T, DWORD t_dwMsgMapID >
class CMouseGestureImpl
{
private:
	MSG		m_msgOrginal;
	CPoint	m_ptMouse;
	DWORD	m_dwLastTime;
	CStdVector<Gesture>	m_Gesture;
	bool	m_isGesturing;

public:
	CMouseGestureImpl()
	{
		m_isGesturing = false;
	}

public: // I[o[Ch
	// WFX`̊JnɌĂ΂B
	void OnGestureBegin()
	{
	}

	// WFX`̏Ii~܂ށjɌĂ΂B
	void OnGestureEnd()
	{
	}

	// WFX`V[PXωɌĂ΂B
	void OnGestureUpdate(UINT length, const Gesture* gesture)
	{
#ifdef _DEBUG
		ATLTRACE(_T("OnGestureUpdate : "));
		for(UINT i = 0; i < length; i++)
		{
			ATLTRACE(GESTURE_CHAR[gesture[i]]);
		}
		ATLTRACE(_T("\n"));
#endif
	}

	// WFX`ɊꍇɌĂ΂B
	void OnGestureFinish(UINT length, const Gesture* gesture)
	{
#ifdef _DEBUG
		ATLTRACE(_T("OnGestureFinish : "));
		for(UINT i = 0; i < length; i++)
		{
			ATLTRACE(GESTURE_CHAR[gesture[i]]);
		}
		ATLTRACE(_T("\n"));
#endif
	}

public: // Iy[V
	// pMsg->message == WM_RBUTTONDOWN ł邱ƁB
	void BeginGesture(MSG* pMsg)
	{
		ATLASSERT(pMsg->message == WM_RBUTTONDOWN);
		T* pThis = static_cast<T*>(this);
		HWND hThisWnd = pThis->m_hWnd;
		::SetCapture(hThisWnd);

		m_isGesturing = true;
		m_Gesture.clear();
		m_msgOrginal = *pMsg;
		m_ptMouse.x = GET_X_LPARAM(m_msgOrginal.lParam);
		m_ptMouse.y = GET_Y_LPARAM(m_msgOrginal.lParam);
		::ClientToScreen(m_msgOrginal.hwnd, &m_ptMouse);
		m_dwLastTime = ::GetTickCount();

		pThis->OnGestureBegin();
		pThis->OnGestureUpdate(0, 0);

		while(m_isGesturing)
		{
			MSG msg;
			if(!::GetMessage(&msg, NULL, 0, 0)) 
			{	// WM_QUIT
				EndGesture();
				PostQuitMessage(msg.wParam);
				return;
			}
			if (::GetCapture() != hThisWnd)
			{
				EndGesture();
				break;
			}

			LRESULT lResult = 0;
			BOOL bHandled = pThis->ProcessWindowMessage(msg.hwnd, msg.message, msg.wParam, msg.lParam, lResult, t_dwMsgMapID);
			if(!bHandled)
			{
				::TranslateMessage(&msg);
				::DispatchMessage(&msg);
			}
		}
	}

	BOOL EndGesture()
	{
		if(m_isGesturing)
		{
			T* pThis = static_cast<T*>(this);
			if(::GetCapture() == pThis->m_hWnd)
			{
				::ReleaseCapture();
			}
			m_Gesture.clear();
			m_isGesturing = false;
			pThis->OnGestureEnd();
		}
		return FALSE;
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CMouseGestureImpl)
	ALT_MSG_MAP(t_dwMsgMapID)
		MESSAGE_HANDLER(WM_MOUSEMOVE, OnGestureMouseMove)
		MESSAGE_HANDLER(WM_RBUTTONUP, OnGestureRButtonUp)
	END_MSG_MAP()

//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	static Gesture CalcDirection(int dx, int dy)
	{
		static const Gesture DIR[2][2] =
		{
			{ GESTURE_NORTH, GESTURE_WEST },
			{ GESTURE_EAST, GESTURE_SOUTH }
		};
		return DIR[dy+dx>0 ? 1 : 0][dy-dx>0 ? 1 : 0];
	}

	LRESULT OnGestureMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
	{
		T* pThis = static_cast<T*>(this);
		CPoint pt;
		::GetCursorPos(&pt);

		int dx = pt.x-m_ptMouse.x, dy = pt.y-m_ptMouse.y;
		if((dx*dx+dy*dy) < GESTURE_SKIP_DISTANCE*GESTURE_SKIP_DISTANCE)
		{	// s
			return 0;
		}

		Gesture dir = CalcDirection(dx, dy);
		m_ptMouse = pt;

		DWORD dwTimeNow = ::GetTickCount();
		if(m_Gesture.empty() || m_Gesture.back() != dir || (dwTimeNow - m_dwLastTime) > GESTURE_ALT_TIME)
		{	// VWFX`
			m_Gesture.push_back(dir);
			pThis->OnGestureUpdate(m_Gesture.size(), m_Gesture.GetData());
		}

		m_dwLastTime = dwTimeNow;
		return 0;
	}

	LRESULT OnGestureRButtonUp(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		if(m_isGesturing)
		{
			T* pThis = static_cast<T*>(this);
			::ReleaseCapture();
			if(!m_Gesture.empty())
			{
				pThis->OnGestureFinish(m_Gesture.size(), m_Gesture.GetData());
			}
			else
			{
				CPoint pt = m_ptMouse;
				::ScreenToClient(m_msgOrginal.hwnd, &pt);
				LPARAM lParamClient = MAKELPARAM(pt.x, pt.y);
				LPARAM lParamScreen = MAKELPARAM(m_ptMouse.x, m_ptMouse.y);
				// eNjbN
				// WM_RBUTTONUP bZ[WL[ɓꂽAWM_RBUTTONDOWN 𒼐SendMessageB
				// ۂɎ󂯎鏇Ԃ́ARDOWN => RUP => CONTEXTMENUB
				// SendMessage()̑OɃ|XgĂKv̂́A
				// EhbOLȃ^CṽRg[iXgAc[Ȃǁj
				// WM_RBUTTONDOWN AĂȂȂ邽߁B
				// WM_RBUTTONUP ɒǉ WM_CONTEXTMENU 𑗂̂́A
				// VFXgr[ RUPł̓j[oȂ߁B
				// |bvAbvReLXgj[ɂ WM_CONTEXTMENU ĂA
				// ͖Ȃ߁A̎ɂB
				// Ȃ݂ɁAWM_RBUTTONDOWN PostMessageƁA}EXWFX`ƗŖ[vɂȂB
				::PostMessage(m_msgOrginal.hwnd, WM_RBUTTONUP  , wParam, lParamClient);
				::PostMessage(m_msgOrginal.hwnd, WM_CONTEXTMENU, (WPARAM)m_msgOrginal.hwnd, lParamScreen);
				::SendMessage(m_msgOrginal.hwnd, WM_RBUTTONDOWN, wParam, lParamClient);
			}
			EndGesture();
		}
		return 0;
	}
};

} // namespace mew
