// MultiSplitter.h
#pragma once

#include "atlsplit.h"

// TODO: v|[ViCAEgB

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CMultiSplitterImplBase
// CMultiSplitterImpl<T>

/////////////////////////////////////////////////////////////////////////////
// CMultiSplitterImplBase

class CMultiSplitterImplBase
{
protected:
	static HCURSOR m_hCursor[2];

	CMultiSplitterImplBase()
	{
		if(m_hCursor[0] == NULL)
		{
			::EnterCriticalSection(&_pAtlModule->m_csStaticDataInitAndTypeInfo.m_sec);
			if(m_hCursor[0] == NULL)
			{
				m_hCursor[0] = ::LoadCursor(NULL, IDC_SIZEWE);
				m_hCursor[1] = ::LoadCursor(NULL, IDC_SIZENS);
			}
			::LeaveCriticalSection(&_pAtlModule->m_csStaticDataInitAndTypeInfo.m_sec);
		}
	}
};

__declspec(selectany) HCURSOR CMultiSplitterImplBase::m_hCursor[2];

/////////////////////////////////////////////////////////////////////////////
// CMultiSplitterImpl

template <class T>
class CMultiSplitterImpl : public CMultiSplitterImplBase
{
private:
	enum { m_nPropMax = 10000 };

	struct Pane
	{
		CWindow	window;
		int size;

		Pane()
		{
			size = 0;
		}
	};

	CAtlArray<Pane> m_Panes;
	CRect m_rcClientCache;	// {Iɂ́ANCAg`̃LbV
	int   m_nActivePane;	// ANeBuyCFSetFocus()ɗD悳
	int   m_nSinglePane;	// VOyC[hBSPLIT_PANE_NONE ̏ꍇ͒ʏ탂[hB
	int   m_nDraggingBar;	// hbÕo[BhbOĂȂ-1B
	int   m_cxyDragOffset;	// hbO̍ۂ̍WItZbg
	int   m_cxySplitBar;	// splitter bar width/height
	int   m_cxyMin;			// minimum pane size
	int   m_cxyBarEdge;		// splitter bar edge
	int   m_PaneEdge;		// ̃yCꍇɏGbW̕B
	DWORD m_dwExtendedStyle;// splitter specific extended styles
	int   m_nProportionalPos; // ???
	bool  m_bVertical;		// c or 
	bool  m_bFullDrag;		// EBhE̕\܂܃hbO邩ۂ
	bool  m_bUpdateProportionalPos;

private:
	void _SetActivePane(INT index)
	{
		bool isChanged = m_nActivePane != index;
		m_nActivePane = index;
		if(isChanged)
		{
			T* pT = static_cast<T*>(this);
			pT->OnActivePaneChange(m_nActivePane);
		}
	}

public: // I[o[Ch
	void OnActivePaneChange(INT index)
	{
	}
	bool OnSetPane(INT index, HWND hWnd)
	{	// yC̑ꍇfalseB
		return true;
	}
	bool OnRemovePane(INT index)
	{	// yC폜ꍇfalseB
		return true;
	}

public:

// Constructor
	CMultiSplitterImpl() :
		m_nActivePane(SPLIT_PANE_NONE), 
		m_cxySplitBar(0), m_cxyMin(0), m_cxyBarEdge(0), m_bVertical(true), m_bFullDrag(true),
		m_cxyDragOffset(0), m_nProportionalPos(0), m_bUpdateProportionalPos(true),
		m_nDraggingBar(-1), m_PaneEdge(2),
		m_dwExtendedStyle(0),//(SPLIT_PROPORTIONAL),
		m_nSinglePane(SPLIT_PANE_NONE)
	{
		m_rcClientCache.SetRectEmpty();
	}

// Attributes
	bool IsVertical() const	{ return m_bVertical; }
	void SetVertical(bool vert)
	{
		m_bVertical = vert;
		RecalcLayout(true);
	}
	int GetSplitBarSize() const
	{
		return m_cxySplitBar + m_cxyBarEdge;
	}

	UINT GetPaneCount() const
	{
		return m_Panes.GetCount();
	}

	UINT SetPaneCount(UINT count)
	{
		m_Panes.SetCount(count);
		for(UINT i = 0; i < m_Panes.GetCount(); i++)
		{
			if(m_Panes[i].size < m_cxyMin + m_cxyBarEdge)
				m_Panes[i].size = m_cxyMin;
		}
		if(m_nActivePane == SPLIT_PANE_NONE)
			_SetActivePane(0);
		return m_Panes.GetCount();
	}

	bool RemovePane(UINT index)
	{
		ATLASSERT(index < GetPaneCount());
		T* pThis = static_cast<T*>(this);
		if(!pThis->OnRemovePane(index))
			return false;
		UINT nCount = GetPaneCount();
		if(!(index < nCount))
			return false;
		m_Panes.RemoveAt(index);
		if(m_nActivePane == index)
			_SetActivePane(max((int)0, (int)(index-1)));
		// VOyC[h̑Ώۂjꂽꍇ͉܂B
		if(m_nSinglePane == index)
		{
			m_nSinglePane = min((int)m_nSinglePane, (int)(m_Panes.GetCount()-1));
			SetSinglePaneMode(SPLIT_PANE_NONE);
		}
		RecalcLayout();
		return true;
	}

	bool RemovePane(HWND hWnd)
	{
		UINT nCount = GetPaneCount();
		for(UINT i = 0; i < nCount; i++)
		{
			if(GetPane(i) == hWnd)
				return RemovePane(i);
		}
		return false;
	}

	void GetSplitterRect(LPRECT lpRect) const
	{
		ATLASSERT(lpRect != NULL);
		*lpRect = m_rcClientCache;
	}

	int GetPaneSize(UINT nPane) const
	{
		UINT nCount = GetPaneCount();
		if(nPane < nCount-1)
		{	// ԉẼyCȊO
			return m_Panes[nPane].size;
		}
		else
		{
			CRect rc;
			GetSplitterPaneRect(nPane, &rc);
			return m_bVertical ? rc.Width() : rc.Height();
		}
	}

	bool SetPaneSize(UINT nPane, int size, bool bUpdate = true)
	{
		if(!(nPane < GetPaneCount()-1))
			return false;

		int cxyMax = GetPaneSize(nPane) + GetPaneSize(nPane+1);

		if(size < m_cxyMin + m_cxyBarEdge)
			size = m_cxyMin;
		else if(size > (cxyMax - GetSplitBarSize()))
			size = cxyMax - GetSplitBarSize();

		// Set new position and update if requested
		bool bRet = (m_Panes[nPane].size != size);
		m_Panes[nPane].size = size;
		m_Panes[nPane+1].size = cxyMax - size;

		if(m_bUpdateProportionalPos)
		{
			if(IsProportional())
				StoreProportionalPos();
			else if(IsRightAligned())
				StoreRightAlignPos();
		}
		else
		{
			m_bUpdateProportionalPos = true;
		}

		if(bUpdate && bRet)
			UpdateLayout();

		return bRet;
	}

	int GetSplitterPos(int nPane) const
	{
		ATLASSERT(0 <= nPane && nPane < GetPaneCount());
		return m_Panes[nPane].size;
	}

	bool SetSinglePaneMode(int nPane)
	{
		ATLASSERT(nPane == SPLIT_PANE_NONE || (0 <= nPane && nPane < (int)GetPaneCount()));
		if(!(nPane == SPLIT_PANE_NONE || (0 <= nPane && nPane < (int)GetPaneCount())))
			return false;

		if(nPane != SPLIT_PANE_NONE)
		{
			_SetActivePane(nPane);
			// ANeBuyC\
			if(m_Panes[nPane].window)
				m_Panes[nPane].window.ShowWindow(SW_SHOW);
			// ̃yCB
			int nCount = GetPaneCount();
			for(int i = 0; i < nCount; i++)
			{
				if(i != nPane && m_Panes[i].window)
					m_Panes[i].window.ShowWindow(SW_HIDE);
			}
		}
		else
		{
			_SetActivePane((!IsSinglePaneMode()) ? 0 : m_nSinglePane);
			// ׂẴyC\
			int nCount = GetPaneCount();
			for(int i = 0; i < nCount; i++)
			{
				if(m_Panes[i].window)
					m_Panes[i].window.ShowWindow(SW_SHOW);
			}
		}

		m_nSinglePane = nPane;
		UpdateLayout();
		return true;
	}

	bool IsSinglePaneMode() const
	{
		return m_nSinglePane != SPLIT_PANE_NONE;
	}

	int GetSinglePaneMode() const
	{
		return m_nSinglePane;
	}

	DWORD GetSplitterExtendedStyle() const
	{
		return m_dwExtendedStyle;
	}

	DWORD SetSplitterExtendedStyle(DWORD dwExtendedStyle, DWORD dwMask = 0)
	{
		DWORD dwPrevStyle = m_dwExtendedStyle;
		if(dwMask == 0)
			m_dwExtendedStyle = dwExtendedStyle;
		else
			m_dwExtendedStyle = (m_dwExtendedStyle & ~dwMask) | (dwExtendedStyle & dwMask);
#ifdef _DEBUG
		if(IsProportional() && IsRightAligned())
			ATLTRACE2(atlTraceUI, 0, "CMultiSplitterImpl::SetSplitterExtendedStyle - SPLIT_PROPORTIONAL and SPLIT_RIGHTALIGNED are mutually exclusive, defaulting to SPLIT_PROPORTIONAL.\n");
#endif //_DEBUG
		return dwPrevStyle;
	}

// operations
	void RecalcLayout(bool bUpdate = true)
	{
		T* pT = static_cast<T*>(this);
		pT->GetClientRect(&m_rcClientCache);

		if(IsProportional())
			UpdateProportionalPos();
		else if(IsRightAligned())
			UpdateRightAlignPos();

		if(bUpdate)
			UpdateLayout();
	}

	bool SetPane(UINT nPane, HWND hWnd, bool bUpdate = true)
	{
		ATLASSERT(nPane < GetPaneCount());
		if(!(nPane < GetPaneCount()))
			return false;
		T* pT = static_cast<T*>(this);
		if(!pT->OnSetPane(nPane, hWnd))
			return false;
		m_Panes[nPane].window = hWnd;
		if(bUpdate)
			UpdateLayout();
		return true;
	}

	HWND GetPane(UINT nPane) const
	{
		ATLASSERT(nPane < GetPaneCount());
		if(!(nPane < GetPaneCount()))
			return NULL;
		return m_Panes[nPane].window;
	}

	// ׂẴyC𓯂傫ɂ낦
	void TilePanes()
	{
		UINT nCount = GetPaneCount();
		UINT sum = m_bVertical ?  m_rcClientCache.Width() : m_rcClientCache.Height();
		sum -= GetSplitBarSize() * (nCount-1);
		UINT size = sum / nCount;
		SetSinglePaneMode(SPLIT_PANE_NONE);
		for(UINT i = 0; i < nCount; i++)
			SetPaneSize(i, size, false);
		RecalcLayout();
	}

	UINT GetActivePane() const
	{
		return m_nActivePane;
	}

	bool SetActivePane(UINT nPane)
	{
		ATLASSERT(nPane < GetPaneCount());
		if(!(nPane < GetPaneCount()))
			return false;
		if(IsSinglePaneMode() && nPane != m_nSinglePane)
			return false;
		if(m_nActivePane != nPane)
		{
			T* pT = static_cast<T*>(this);
			pT->Invalidate();
			HWND hFocus = ::GetFocus();
			if(pT->m_hWnd == hFocus || pT->IsChild(hFocus))
				m_Panes[nPane].window.SetFocus();
		}
		_SetActivePane(nPane);
		return true;
	}

	bool SetActivePaneWindow(HWND hViewOrChild)
	{
		UINT nCount = GetPaneCount();
		for(UINT i = 0; i < nCount; i++)
		{
			if(m_Panes[i].window == hViewOrChild || m_Panes[i].window.IsChild(hViewOrChild))
			{
				return SetActivePane(i);
			}
		}
		return false;
	}

	bool ActivateNextPane(bool bNextOrPrev)
	{
		if(IsSinglePaneMode())
			return false;
		int nPane = GetActivePane();
		if(nPane < 0)
		{
			nPane = bNextOrPrev ? 0 : GetPaneCount()-1;
		}
		else
		{
			if(bNextOrPrev)
			{
				if(++nPane == GetPaneCount())
					nPane = 0;
			}
			else
			{
				if(--nPane == -1)
					nPane = GetPaneCount()-1;
			}
		}
		return SetActivePane(nPane);
	}

	void UpdateActivePane(POINT pt)
	{
		INT index = HitTest(pt);
		if(index >= 0)
			SetActivePane(index);
	}

	INT HitTest(POINT pt) const
	{
		RECT rcPane;
		UINT nCount = GetPaneCount();
		for(UINT i = 0; i < nCount; i++)
		{
			if(GetSplitterPaneRect(i, &rcPane) && ::PtInRect(&rcPane, pt))
			{
				return i;
			}
		}
		return -1;
	}

	void DrawSplitter(CDCHandle dc)
	{
		ATLASSERT(dc.m_hDC != NULL);
		if(GetPaneCount() == 0)
			return;

		T* pT = static_cast<T*>(this);
		if(!IsSinglePaneMode())
		{
			UINT nCount = GetPaneCount();
			for(UINT nPane = 0; nPane < nCount; nPane++)
			{
				pT->DrawSplitterBar(dc, nPane);
				if(m_Panes[nPane].window == NULL)
					pT->DrawEmptyPane(dc, nPane);
				else
					pT->DrawPaneFrame(dc, nPane);
			}
		}
		else
		{
			if(m_Panes[m_nSinglePane].window == NULL)
				pT->DrawEmptyPane(dc, m_nSinglePane);
		}
	}

// Overrideables
	void DrawSplitterBar(CDCHandle dc, int nPane)
	{
		RECT rect;
		if(GetSplitterBarRect(nPane, &rect))
		{
			dc.FillRect(&rect, COLOR_3DFACE);
			// draw 3D edge if needed
			T* pT = static_cast<T*>(this);
			if((pT->GetExStyle() & WS_EX_CLIENTEDGE) != 0)
				dc.DrawEdge(&rect, EDGE_RAISED, m_bVertical ? (BF_LEFT | BF_RIGHT) : (BF_TOP | BF_BOTTOM));
		}
	}

	// called only if pane is empty
	void DrawEmptyPane(CDCHandle dc, int nPane)
	{
		RECT rect;
		if(GetSplitterPaneRect(nPane, &rect))
		{
			T* pT = static_cast<T*>(this);
			if((pT->GetExStyle() & WS_EX_CLIENTEDGE) == 0)
				dc.DrawEdge(&rect, EDGE_SUNKEN, BF_RECT | BF_ADJUST);
			dc.FillRect(&rect, COLOR_APPWORKSPACE);
		}
	}

	//
	void DrawPaneFrame(CDCHandle dc, int nPane)
	{
		if(m_PaneEdge == 0)
			return;
		RECT rect;
		if(GetSplitterPaneRect(nPane, &rect))
		{
			int syscolor = (nPane == m_nActivePane) ? COLOR_ACTIVECAPTION : COLOR_GRAYTEXT;
//			int syscolor = (nPane == m_nActivePane) ? COLOR_HIGHLIGHT : COLOR_GRAYTEXT;
			dc.FillRect(&rect, ::GetSysColorBrush(syscolor));
		}
	}

// Message map and handlers
	typedef CMultiSplitterImpl<T>	thisClass;
	BEGIN_MSG_MAP(thisClass)
		MESSAGE_HANDLER(WM_CREATE     , OnCreate)
		MESSAGE_HANDLER(WM_PAINT      , OnPaint)
		MESSAGE_HANDLER(WM_PRINTCLIENT, OnPaint)
		MESSAGE_HANDLER(WM_SETFOCUS     , OnSetFocus)
		MESSAGE_HANDLER(WM_MOUSEACTIVATE, OnMouseActivate)
		MESSAGE_HANDLER(WM_SETTINGCHANGE, OnSettingChange)
		if(IsInteractive())
		{
			MESSAGE_HANDLER(WM_SETCURSOR    , OnSetCursor)
			MESSAGE_HANDLER(WM_MOUSEMOVE    , OnMouseMove)
			MESSAGE_HANDLER(WM_LBUTTONDOWN  , OnLButtonDown)
			MESSAGE_HANDLER(WM_LBUTTONUP    , OnLButtonUp)
			MESSAGE_HANDLER(WM_LBUTTONDBLCLK, OnLButtonDoubleClick)
		}
	END_MSG_MAP()

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		GetSystemSettings(false);
		bHandled = FALSE;
		return 1;
	}

	LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		// do painting
		CPaintDC dc(pT->m_hWnd);
		pT->DrawSplitter(dc.m_hDC);
		return 0;
	}

	LRESULT OnSetCursor(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		T* pT = static_cast<T*>(this);
		if((HWND)wParam == pT->m_hWnd && LOWORD(lParam) == HTCLIENT)
		{
			DWORD dwPos = ::GetMessagePos();
			POINT ptPos = { GET_X_LPARAM(dwPos), GET_Y_LPARAM(dwPos) };
			pT->ScreenToClient(&ptPos);
			if(IsOverSplitterBar(ptPos.x, ptPos.y, NULL))
				return 1;
		}

		bHandled = FALSE;
		return 0;
	}

	LRESULT OnMouseMove(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		T* pT = static_cast<T*>(this);
		int xPos = GET_X_LPARAM(lParam);
		int yPos = GET_Y_LPARAM(lParam);
		if((wParam & MK_LBUTTON) && ::GetCapture() == pT->m_hWnd)
		{
			ATLASSERT(m_nDraggingBar != -1);
			int xyNewSplitPos = 0;
			if(m_bVertical)
				xyNewSplitPos = xPos - m_rcClientCache.left - m_cxyDragOffset;
			else
				xyNewSplitPos = yPos - m_rcClientCache.top - m_cxyDragOffset;

			if(m_Panes[m_nDraggingBar].size != xyNewSplitPos)
			{
				if(m_bFullDrag)
				{
					if(pT->SetPaneSize(m_nDraggingBar, xyNewSplitPos, true))
						pT->UpdateWindow();
				}
				else
				{
					DrawGhostBar(m_nDraggingBar);
					pT->SetPaneSize(m_nDraggingBar, xyNewSplitPos, false);
					DrawGhostBar(m_nDraggingBar);
				}
			}
		}
		else		// not dragging, just set cursor
		{
			if(IsOverSplitterBar(xPos, yPos, NULL))
				::SetCursor(m_hCursor[m_bVertical ? 0 : 1]);
			bHandled = FALSE;
		}

		return 0;
	}

	LRESULT OnLButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& bHandled)
	{
		int xPos = GET_X_LPARAM(lParam);
		int yPos = GET_Y_LPARAM(lParam);
		if(IsOverSplitterBar(xPos, yPos, &m_nDraggingBar))
		{
			T* pT = static_cast<T*>(this);
			pT->SetCapture();
			::SetCursor(m_hCursor[m_bVertical ? 0 : 1]);
			if(!m_bFullDrag)
				DrawGhostBar(m_nDraggingBar);
			if(m_bVertical)
				m_cxyDragOffset = xPos - m_Panes[m_nDraggingBar].size;
			else
				m_cxyDragOffset = yPos - m_Panes[m_nDraggingBar].size;
		}
		bHandled = FALSE;
		return 1;
	}

	LRESULT OnLButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		if(!m_bFullDrag)
		{
			DrawGhostBar(m_nDraggingBar);
			T* pT = static_cast<T*>(this);
			UpdateLayout();
			pT->UpdateWindow();
		}
		::ReleaseCapture();
		m_nDraggingBar = -1;
		bHandled = FALSE;
		return 1;
	}

	LRESULT OnLButtonDoubleClick(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
	{
		int xPos = GET_X_LPARAM(lParam);
		int yPos = GET_Y_LPARAM(lParam);
		int nBar;
		if(IsOverSplitterBar(xPos, yPos, &nBar))
		{
			T* pT = static_cast<T*>(this);
			pT->TilePanes();
		}
		return 0;
	}

	LRESULT OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM, BOOL& bHandled)
	{
		if(!IsSinglePaneMode())
		{
			if((UINT)m_nActivePane < GetPaneCount())
				m_Panes[m_nActivePane].window.SetFocus();
		}
		else
		{
			m_Panes[m_nSinglePane].window.SetFocus();
		}
		bHandled = FALSE;
		return 1;
	}

	LRESULT OnMouseActivate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		LRESULT lRet = pT->DefWindowProc(uMsg, wParam, lParam);
		if(lRet == MA_ACTIVATE || lRet == MA_ACTIVATEANDEAT)
		{
			DWORD dwPos = ::GetMessagePos();
			POINT pt = { GET_X_LPARAM(dwPos), GET_Y_LPARAM(dwPos) };
			T* pT = static_cast<T*>(this);
			pT->ScreenToClient(&pt);
			UpdateActivePane(pt);
		}
		return lRet;
	}

	LRESULT OnSettingChange(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		GetSystemSettings(true);
		return 0;
	}

private:
// Implementation - internal helpers

	// ̂߂HDWPgĂ݂A܂ωB
	HDWP InvalidatePane(int nPane, bool drawEdge, HDWP hDWP)
	{
		T* pT = static_cast<T*>(this);
		CRect rect;
		if(GetSplitterPaneRect(nPane, &rect))
		{
			if(m_Panes[nPane].window != NULL)
			{
				if(drawEdge && m_PaneEdge > 0)
				{
					pT->InvalidateRect(&rect);
					rect.DeflateRect(m_PaneEdge, m_PaneEdge);
				}
				if(hDWP)
					return ::DeferWindowPos(hDWP, m_Panes[nPane].window, NULL, rect.left, rect.top, rect.Width(), rect.Height(), SWP_NOZORDER);
				else
					m_Panes[nPane].window.SetWindowPos(NULL, rect.left, rect.top, rect.Width(), rect.Height(), SWP_NOZORDER);
			}
			else
				pT->InvalidateRect(&rect);
		}
		return NULL;
	}
	void InvalidateBar(int nBar)
	{
		T* pT = static_cast<T*>(this);
		RECT rect;
		if(GetSplitterBarRect(nBar, &rect))
		{
			pT->InvalidateRect(&rect);
		}
	}
	void UpdateLayout()
	{
		if(!IsSinglePaneMode())
		{
			int nCount = GetPaneCount();
			if(nCount > 1)
			{
				HDWP hDWP = BeginDeferWindowPos(nCount);
				for(int i = 0; i < nCount; i++)
				{
					hDWP = InvalidatePane(i, true, hDWP);
					InvalidateBar(i);
				}
				EndDeferWindowPos(hDWP);
			}
			else
			{
				InvalidatePane(0, false, NULL);
			}
		}
		else
		{
			InvalidatePane(m_nSinglePane, false, NULL);
		}
	}

	bool GetSplitterBarRect(UINT nPane, LPRECT lpRect) const
	{
		ATLASSERT(nPane < GetPaneCount());
		ATLASSERT(lpRect != NULL);
		if(IsSinglePaneMode() || nPane == GetPaneCount()-1)
			return false;
		if(!GetSplitterPaneRect(nPane, lpRect))
			return false;
		if(m_bVertical)
		{
			lpRect->left = lpRect->right;
			lpRect->right = lpRect->left + GetSplitBarSize();
		}
		else
		{
			lpRect->top = lpRect->bottom;
			lpRect->bottom = lpRect->top + GetSplitBarSize();
		}
		return true;
	}

	bool GetSplitterPaneRect(UINT nPane, LPRECT lpRect) const
	{
		ATLASSERT(lpRect != NULL);
		if(!(nPane < GetPaneCount()))
			return false;
		if(IsSinglePaneMode())
		{
			if(nPane != m_nSinglePane)
				return false;
			*lpRect = m_rcClientCache;
			return true;
		}
		else
		{
			if(m_bVertical)
			{
				lpRect->left   = m_rcClientCache.left;
				lpRect->top    = m_rcClientCache.top;
				lpRect->bottom = m_rcClientCache.bottom;
				for(UINT i = 0; i < nPane; i++)
				{
					lpRect->left += m_Panes[i].size + GetSplitBarSize();
				}
				// Ō̃yC͉E҂
				if(nPane == GetPaneCount()-1)
					lpRect->right = m_rcClientCache.right;
				else
					lpRect->right = lpRect->left + m_Panes[nPane].size;
			}
			else
			{
				lpRect->left   = m_rcClientCache.left;
				lpRect->top    = m_rcClientCache.top;
				lpRect->right  = m_rcClientCache.right;
				for(UINT i = 0; i < nPane; i++)
				{
					lpRect->top += m_Panes[i].size + GetSplitBarSize();
				}
				// Ō̃yC͉҂
				if(nPane == GetPaneCount()-1)
					lpRect->bottom = m_rcClientCache.bottom;
				else
					lpRect->bottom = lpRect->top + m_Panes[nPane].size;
			}
			return true;
		}
	}

	bool IsOverSplitterRect(int x, int y) const
	{
		// -1 == don't check
		return ((x == -1 || (x >= m_rcClientCache.left && x <= m_rcClientCache.right)) &&
			(y == -1 || (y >= m_rcClientCache.top && y <= m_rcClientCache.bottom)));
	}

	bool IsOverSplitterBar(int x, int y, int* pBar) const
	{
		if(IsSinglePaneMode())
			return false;
		if(!IsOverSplitterRect(x, y))
			return false;
		int xy = m_bVertical ? x : y;
		int xyOff = m_bVertical ? m_rcClientCache.left : m_rcClientCache.top;
		int nCount = GetPaneCount();
		for(int i = 0; i < nCount-1; i++)
		{
			int xyOffPrev = xyOff + m_Panes[i].size;
			int xyOffNext = xyOffPrev + GetSplitBarSize();
			if(xyOffPrev-m_PaneEdge <= xy && xy < xyOffNext+m_PaneEdge)
			{
				if(pBar) *pBar = i;
				return true;
			}
			xyOff = xyOffNext;
		}
		return false;
	}

	void DrawGhostBar(int nPane)
	{
		RECT rect = { 0, 0, 0, 0 };
		if(GetSplitterBarRect(nPane, &rect))
		{
			// invert the brush pattern (looks just like frame window sizing)
			T* pT = static_cast<T*>(this);
			CWindowDC dc(pT->m_hWnd);
			CBrush brush = CDCHandle::GetHalftoneBrush();
			if(brush.m_hBrush != NULL)
			{
				CBrushHandle brushOld = dc.SelectBrush(brush);
				dc.PatBlt(rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, PATINVERT);
				dc.SelectBrush(brushOld);
			}
		}
	}

	void GetSystemSettings(bool bUpdate)
	{
		m_cxySplitBar = ::GetSystemMetrics(m_bVertical ? SM_CXSIZEFRAME : SM_CYSIZEFRAME);

		T* pT = static_cast<T*>(this);
		if((pT->GetExStyle() & WS_EX_CLIENTEDGE))
		{
			m_cxyBarEdge = 2 * ::GetSystemMetrics(m_bVertical ? SM_CXEDGE : SM_CYEDGE);
			m_cxyMin = 0;
		}
		else
		{
			m_cxyBarEdge = 0;
			m_cxyMin = 2 * ::GetSystemMetrics(m_bVertical ? SM_CXEDGE : SM_CYEDGE);
		}

		BOOL bFullDrag;
		::SystemParametersInfo(SPI_GETDRAGFULLWINDOWS, 0, &bFullDrag, 0);
		m_bFullDrag = (bFullDrag != 0);
		if(bUpdate)
			UpdateLayout();
	}

	bool IsProportional() const
	{
		return ((m_dwExtendedStyle & SPLIT_PROPORTIONAL) != 0);
	}

	void StoreProportionalPos()
	{
		int cxyTotal = m_bVertical ? (m_rcClientCache.Width() - GetSplitBarSize()) : (m_rcClientCache.Height() - GetSplitBarSize());
		if(cxyTotal > 0)
		{
			int nPane = 0;
			m_nProportionalPos = ::MulDiv(m_Panes[0].size, m_nPropMax, cxyTotal);
		}
		else
			m_nProportionalPos = 0;
	}

	void UpdateProportionalPos()
	{
		int cxyTotal = m_bVertical ? (m_rcClientCache.Width() - GetSplitBarSize()) : (m_rcClientCache.Height() - GetSplitBarSize());
		if(cxyTotal > 0)
		{
			int xyNewPos = ::MulDiv(m_nProportionalPos, cxyTotal, m_nPropMax);
			m_bUpdateProportionalPos = false;
			T* pT = static_cast<T*>(this);
			pT->SetPaneSize(0, xyNewPos, false);
		}
	}

	bool IsRightAligned() const
	{
		return ((m_dwExtendedStyle & SPLIT_RIGHTALIGNED) != 0);
	}

	void StoreRightAlignPos()
	{
		int cxyTotal = m_bVertical ? (m_rcClientCache.Width() - GetSplitBarSize()) : (m_rcClientCache.Height() - GetSplitBarSize());
		if(cxyTotal > 0)
		{
			int nPane = 0;
			m_nProportionalPos = cxyTotal - m_Panes[0].size;
		}
		else
			m_nProportionalPos = 0;
	}

	void UpdateRightAlignPos()
	{
		int cxyTotal = m_bVertical ? (m_rcClientCache.Width() - GetSplitBarSize()) : (m_rcClientCache.Height() - GetSplitBarSize());
		if(cxyTotal > 0)
		{
			m_bUpdateProportionalPos = false;
			T* pT = static_cast<T*>(this);
			pT->SetPaneSize(0, cxyTotal - m_nProportionalPos, false);
		}
	}

	bool IsInteractive() const
	{
		return ((m_dwExtendedStyle & SPLIT_NONINTERACTIVE) == 0);
	}
};

/////////////////////////////////////////////////////////////////////////////
// CMultiSplitterWindowImpl - Implements a splitter window

template < class T, class TBase = CWindow, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CMultiSplitterWindowImpl : 
	public CWindowImpl< T, TBase, TWinTraits >,
	public CMultiSplitterImpl<T >
{
public:
	DECLARE_WND_CLASS_EX(NULL, CS_DBLCLKS, COLOR_WINDOW)

	typedef CMultiSplitterWindowImpl< T, TBase, TWinTraits >	thisClass;
	typedef CMultiSplitterImpl< T >				baseClass;
	BEGIN_MSG_MAP(thisClass)
		MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBkgnd)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
		MESSAGE_HANDLER(WM_PARENTNOTIFY	, OnParentNotify)
		CHAIN_MSG_MAP(baseClass)
		FORWARD_NOTIFICATIONS()
	END_MSG_MAP()

	LRESULT OnEraseBkgnd(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// handled, no background painting needed
		return 1;
	}

	LRESULT OnSize(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
	{
		if(wParam != SIZE_MINIMIZED)
			RecalcLayout();

		bHandled = FALSE;
		return 1;
	}

	LRESULT OnParentNotify(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		UINT what = LOWORD(wParam);
		switch(what)
		{
		case WM_DESTROY:
			if(::GetParent((HWND)lParam) == m_hWnd)
			{	// r[jꂽꍇɃ^u폜B
				RemovePane((HWND)lParam);
			}
			break;
		case WM_LBUTTONDOWN:
		case WM_MBUTTONDOWN:
		case WM_RBUTTONDOWN:
		case WM_XBUTTONDOWN:
			UpdateActivePane(CPoint(lParam));
			break;
		}
		bHandled = FALSE;
		return 0;
	}
};


/////////////////////////////////////////////////////////////////////////////
// CMultiSplitterWindow - Implements a splitter window to be used as is

class CMultiSplitterWindow : public CMultiSplitterWindowImpl<CMultiSplitterWindow>
{
public:
	DECLARE_WND_CLASS_EX(_T("WTL_SplitterWindowEx"), CS_DBLCLKS, COLOR_WINDOW)
};

} // namespace mew
