// Picture.h
#pragma once

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CPicture : IPicture wrapper

class CPicture
{
public:
	CComPtr<IPicture> m_pPicture;
	CSize	m_Size;	// sNZPʂł̃TCY

private:
	void RecalcPixelSize()
	{
		if(m_pPicture)
		{
			CClientDC dc(NULL);
			long width, height;
			m_pPicture->get_Width(&width);		// PʂHIMETRIC_INCHȂ̂ŁAsNZPʂɒB
			m_pPicture->get_Height(&height);	// PʂHIMETRIC_INCHȂ̂ŁAsNZPʂɒB
			int pix = GetDeviceCaps(dc, LOGPIXELSX);
			m_Size.cx = MulDiv(width, pix, HIMETRIC_INCH);
			m_Size.cy = MulDiv(height, pix, HIMETRIC_INCH);
		}
		else
		{
			m_Size.cx = m_Size.cy = 0;
		}
	}

public:
	CPicture(IPicture* pPicture = NULL) : m_pPicture(pPicture)
	{
		RecalcPixelSize();
	}
	CPicture(const CPicture& lhs) : m_pPicture(lhs.m_pPicture)
	{
		RecalcPixelSize();
	}
	const CPicture& operator = (const CPicture& lhs)
	{
		m_pPicture = lhs.m_pPicture;
		RecalcPixelSize();
		return *this;
	}
	~CPicture()
	{
		m_pPicture.Release();
	}

	operator IPicture* ()
	{
		return m_pPicture;
	}

	// sNZPʂ̕擾B
	LONG Width() const	{ return m_Size.cx; }
	// sNZPʂ̍擾B
	LONG Height() const	{ return m_Size.cy; }
	//
	CSize Size() const	{ return m_Size; }
	// t@C摜ǂݍށBBitmap, Icon, GIF, JPEGǂ߂邱ƂmFBPNG͓ǂ߂ȂB
	bool LoadFromFile(LPCTSTR filename)
	{
		ATLASSERT(!m_pPicture);
		CComPtr<IStream> pStream;
		HRESULT hr = ::SHCreateStreamOnFile(filename, STGM_READ | STGM_SHARE_DENY_WRITE, &pStream);
		if FAILED(hr)
			return false;
		return LoadFromStream(pStream);
	}
	// \[Xǂݍ
	bool LoadFromResource(HMODULE hModule, UINT id, LPCTSTR type)
	{
		HRSRC hRes = ::FindResource(hModule, MAKEINTRESOURCE(id), type);
		if(hRes == NULL)
		{
			ATLTRACE("failed FindResource()\n");
			return false;
		}
		void* ptr = LockResource(::LoadResource(hModule, hRes));
		if(ptr == NULL)
		{
			ATLTRACE("failed LoadResource()\n");
			return false;
		}
		DWORD size = SizeofResource(hModule, hRes);
		HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE, size);
		void* data = GlobalLock(hGlobal);
		CopyMemory(data, ptr, size);
		GlobalUnlock(hGlobal);
		return LoadFromHGlobal(hGlobal, TRUE);
	}
	// O[onhǂݍށBautofree==TRUEȂAIGlobalFree()B
	bool LoadFromHGlobal(HGLOBAL hGlobal, BOOL autofree)
	{
		// IStream쐬B
		CComPtr<IStream> stream;
		HRESULT hr = CreateStreamOnHGlobal(hGlobal, autofree, &stream);
		if(FAILED(hr) || stream == NULL)
		{
			ATLTRACE("failed CreateStreamOnHGlobal()\n");
			return false;
		}
		return LoadFromStream(stream);
	}
	// IStreamǂݍށB
	bool LoadFromStream(IStream* stream)
	{
		Destroy();
		HRESULT hr = ::OleLoadPicture(stream, 0, FALSE, IID_IPicture, (void**)&m_pPicture);
		if(FAILED(hr) || m_pPicture == NULL)
		{
			ATLTRACE("failed OleLoadPicture()\n");
			return false;
		}
		return true;
	}
	void Destroy()
	{
		m_pPicture.Release();
	}
	void Draw(HDC hDC, CPoint pos)
	{
		ATLASSERT(m_pPicture);
		long width, height;
		m_pPicture->get_Width(&width);
		m_pPicture->get_Height(&height);
		m_pPicture->Render(hDC, pos.x, pos.y, Width(), Height(), 0, height, width, -height, NULL);
	}
	void Draw(HDC hDC, CPoint pos, CSize size)
	{
		ATLASSERT(m_pPicture);
		long width, height;
		m_pPicture->get_Width(&width);
		m_pPicture->get_Height(&height);
		m_pPicture->Render(hDC, pos.x, pos.y, size.cx, size.cy, 0, height, width, -height, NULL);
	}
};

} // namespace mew
