// ProfileUtil.h
#pragma once

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CProfile

/////////////////////////////////////////////////////////////////////////////

inline bool GetPrivateProfileBool(LPCTSTR section, LPCTSTR key, bool bDefault, LPCTSTR filename)
{
	TCHAR buffer[64];
	::GetPrivateProfileString(section, key, _T(""), buffer, 64, filename);
	if(lstrcmpi(buffer, _T("true")) == 0)
		return true;
	else if(lstrcmpi(buffer, _T("false")) == 0)
		return false;
	else
		return bDefault;
}

inline BOOL WritePrivateProfileBool(LPCTSTR section, LPCTSTR key, bool bValue, LPCTSTR filename)
{
	return ::WritePrivateProfileString(section, key, bValue ? _T("true") : _T("false"), filename);
}

inline BOOL WritePrivateProfileInt(LPCTSTR section, LPCTSTR key, int nValue, LPCTSTR filename)
{
	TCHAR buffer[64];
	wsprintf(buffer, _T("%d"), nValue);
	return ::WritePrivateProfileString(section, key, buffer, filename);
}

/////////////////////////////////////////////////////////////////////////////
// ̃t@CAZNVAL[͓ŃRs[ȂƂɒӁB

class CProfile
{
private:
	LPCTSTR m_FileName;
	LPCTSTR m_Section;
public:
	CProfile(LPCTSTR filename, LPCTSTR section = NULL) : m_FileName(filename), m_Section(section)
	{}

	void SetSection(LPCTSTR section)
	{
		m_Section = section;
	}

	// Bool
	bool ReadBool(LPCTSTR key, bool bDefault) const
	{
		ATLASSERT(m_Section);
		return GetPrivateProfileBool(m_Section, key, bDefault, m_FileName);
	}
	BOOL WriteBool(LPCTSTR key, bool bValue)
	{
		ATLASSERT(m_Section);
		return WritePrivateProfileBool(m_Section, key, bValue, m_FileName);
	}
	// Int
	INT ReadInt(LPCTSTR key, INT nDefault) const
	{
		ATLASSERT(m_Section);
		return GetPrivateProfileInt(m_Section, key, nDefault, m_FileName);
	}
	BOOL WriteInt(LPCTSTR key, INT nValue)
	{
		ATLASSERT(m_Section);
		return WritePrivateProfileInt(m_Section, key, nValue, m_FileName);
	}
	// WindowPlacement
	BOOL LoadWindowPlacement(HWND hWnd)
	{
		WINDOWPLACEMENT place = { sizeof(WINDOWPLACEMENT) };
		int x, y, w, h;
		x = ReadInt(_T("X"), INT_MAX);
		y = ReadInt(_T("Y"), INT_MAX);
		w = ReadInt(_T("Width"), INT_MAX);
		h = ReadInt(_T("Height"), INT_MAX);
		if(x == INT_MAX || y == INT_MAX || w == INT_MAX || h == INT_MAX)
			return FALSE;
		ATLTRACE(_T("LoadWindowPlacement(%s, %d, %d, %d, %d)\n"), m_Section, x, y, w, h);
		::SetRect(&place.rcNormalPosition, x, y, x+w, y+h);
		place.showCmd = SW_NORMAL;
		return ::SetWindowPlacement(hWnd, &place);
	}
	BOOL SaveWindowPlacement(HWND hWnd)
	{
		WINDOWPLACEMENT place = { sizeof(WINDOWPLACEMENT) };
		if(!::GetWindowPlacement(hWnd, &place))
			return FALSE;
		CRect& rc = (CRect&)place.rcNormalPosition;
		WriteInt(_T("X"), rc.left);
		WriteInt(_T("Y"), rc.top);
		WriteInt(_T("Width"), rc.Width());
		WriteInt(_T("Height"), rc.Height());
		ATLTRACE(_T("SaveWindowPlacement(%s, %d, %d, %d, %d)\n"), m_Section, rc.left, rc.top, rc.Width(), rc.Height());
		return TRUE;
	}
	// ReBar
	BOOL LoadReBarState(CReBarCtrl rebar, const REBARBANDINFO* bands, size_t size)
	{
		// łɒǉꂽohёւAVǉقyȂ̂ŁB
		CStdVector<REBARBANDINFO> bands2;
		bands2.resize(size);
		memcpy(bands2.GetData(), bands, sizeof(REBARBANDINFO)*size);
		for(size_t i = 0; i < size; i++)
		{
			DWORD id = ::GetDlgCtrlID(bands[i].hwndChild);
			TCHAR buffer[256];
			wsprintf(buffer, _T("ReBar[#%d].index"), id);
			UINT index = ReadInt(buffer, -1);
			if(index >= size)
				continue;
			memcpy(&bands2[index], &bands[i], sizeof(REBARBANDINFO));
			wsprintf(buffer, _T("ReBar[#%d].cx"), id);
			bands2[index].cx = ReadInt(buffer, bands[index].cx);
			wsprintf(buffer, _T("ReBar[#%d].fStyle"), id);
			bands2[index].fStyle = ReadInt(buffer, bands[index].fStyle);
		}
		for(size_t i = 0; i < bands2.size(); i++)
		{
			rebar.InsertBand(i, &bands2[i]);
			rebar.ShowBand(i, (bands2[i].fStyle & RBBS_HIDDEN) ? FALSE : TRUE);
		}
		return TRUE;
	}
	BOOL SaveReBarState(CReBarCtrl rebar)
	{
		REBARBANDINFO band = { sizeof(REBARBANDINFO), RBBIM_SIZE | RBBIM_STYLE | RBBIM_CHILD };
		for(int i = 0; rebar.GetBandInfo(i, &band); i++)
		{
			DWORD id = ::GetDlgCtrlID(band.hwndChild);
			TCHAR buffer[256];
			wsprintf(buffer, _T("ReBar[#%d].index"), id);
			WriteInt(buffer, i);
			wsprintf(buffer, _T("ReBar[#%d].cx"), id);
			WriteInt(buffer, band.cx);
			wsprintf(buffer, _T("ReBar[#%d].fStyle"), id);
			WriteInt(buffer, band.fStyle);
		}
		return TRUE;
	}
};

/////////////////////////////////////////////////////////////////////////////

} // namespace mew
