// TreeImpl.h
#pragma once

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CTreeImpl<T, TParam, TBase, TWinTratis> : ^tc[Rg[

template < class T, class TParam, class TBase, class TWinTraits >
class ATL_NO_VTABLE CTreeImpl : public CWindowImpl<T, TBase, TWinTraits>
{
public:
	DECLARE_WND_SUPERCLASS(NULL, TBase::GetWndClassName())
	typedef TParam	ParamType;

	struct TreeItem : public TVITEM
	{
		TreeItem(UINT mask)
		{
			this->mask = mask;
		}
		TParam* get_Param() const	{ return (TParam*)lParam; }
		__declspec(property(get=get_Param)) TParam* pParam;
	};

	struct TreeNotify
	{
		NMHDR hdr; 
		UINT action; 
		TreeItem itemOld; 
		TreeItem itemNew; 
		POINT ptDrag; 
	};

public: // Iy[V
	ParamType* GetItemParam(HTREEITEM hItem)
	{
		if(hItem == NULL)
			return NULL;
		TreeItem item(TVIF_HANDLE | TVIF_PARAM);
		item.hItem = hItem;
		if(GetItem(&item))
			return item.pParam;
		else
			return NULL;
	}

	BOOL GetItem(TreeItem* pItem)
	{
		return __super::GetItem(pItem);
	}

	void DeleteChildren(HTREEITEM hParent)
	{
		Expand(hParent, TVE_COLLAPSE | TVE_COLLAPSERESET);
	}

	template < class Op >
	bool ForEachChildren(HTREEITEM hParent, Op& op)
	{
		T* pThis = static_cast<T*>(this);
		for(HTREEITEM hChild = GetChildItem(hParent); hChild; hChild = GetNextSiblingItem(hChild))
		{
			if(!op(pThis, hChild))
				return false;
		}
		return true;
	}

public: // I[o[Ch
	CAutoPtr<ParamType> AllocItem()
	{
		return CAutoPtr<ParamType>(new ParamType());
	}

	void FreeItem(CAutoPtr<ParamType> pItem)
	{
		//pItem.Free();
	}

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		return FALSE;
	}

	void OnItemContextMenu(HTREEITEM hItem, CPoint ptScreen)
	{
	}

	// NM_RETURN ̑ɁÃ\bhI[o[Ch邱ƁB
	void OnItemReturn(HTREEITEM hItem)
	{
		ATLTRACE("CTreeImpl.OnItemReturn()\n");
	}

public: // bZ[W}bv
	BEGIN_MSG_MAP(CTreeImpl)
		MESSAGE_HANDLER(WM_CONTEXTMENU	, OnContextMenu)
		REFLECTED_NOTIFY_CODE_HANDLER(NM_RCLICK			, OnItemRClick)
		REFLECTED_NOTIFY_CODE_HANDLER(NM_RETURN			, OnItemReturn)
		REFLECTED_NOTIFY_CODE_HANDLER(TVN_KEYDOWN		, OnItemKeyDown)
		REFLECTED_NOTIFY_CODE_HANDLER(TVN_DELETEITEM	, OnDeleteItem)
		DEFAULT_REFLECTION_HANDLER()
	END_MSG_MAP()

	LRESULT OnContextMenu(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
	{
		CPoint ptScreen(lParam);
		HTREEITEM hItem = GetSelectedItem();
		if(hItem == NULL)
			return 0;
		if(ptScreen.x == -1 || ptScreen.y == -1)
		{
			CRect rc;
			GetItemRect(hItem, &rc, TRUE);
			CPoint pt(rc.left, rc.bottom);
			ClientToScreen(&pt);
			ptScreen = pt;
		}
		T* pThis = static_cast<T*>(this);
		pThis->OnItemContextMenu(hItem, ptScreen);
		return 0;
	}

	LRESULT OnDeleteItem(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{
		TreeNotify* notify = (TreeNotify*)pnmh;
		CAutoPtr<ParamType> pItem(notify->itemOld.pParam);
		T* pThis = static_cast<T*>(this);
		pThis->FreeItem(pItem);
		return 0;
	}

	LRESULT OnItemRClick(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{	// Ȃ WM_CONTEXTMENU _uNbNɂȂ̂ŁA
		// NM_RCLICK ReLXgj[B
		CPoint pt;
		GetCursorPos(&pt);
		TVHITTESTINFO hit = { pt.x, pt.y };
		ScreenToClient(&hit.pt);
		HTREEITEM hItem = HitTest(&hit);
		if(hItem)
		{
			T* pThis = static_cast<T*>(this);
			pThis->OnItemContextMenu(hItem, pt);
		}
		return 0;
	}

	// Return nonzero to prevent the default processing, or zero to allow the default processing. 
	// ftHǵur[v炷v̂ŁALZB
	LRESULT OnItemReturn(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
	{
		return 1;
	}

	// NM_RETURN ł́ACtrl+Return 擾łȂ߁ATVN_KEYDOWN nhB
	LRESULT OnItemKeyDown(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
	{
		NMTVKEYDOWN* key = (NMTVKEYDOWN*)pnmh;
		if(key->wVKey == VK_RETURN)
		{
			T* pThis = static_cast<T*>(this);
			pThis->OnItemReturn(GetSelectedItem());
			return 0;
		}
		return 1;
	}
};

} // namespace mew
