// WebBrowser.h
#pragma once

#include <ExDispid.h>
#include "../WTL/tlogstg.h"
#include "PathUtil.h"
#include "Keyboard.h"

namespace mew {

/////////////////////////////////////////////////////////////////////////////
// Classes in this file
//
// CWebBrowser
// CWebBrowserCommands<T>
// CWebBrowserEvents<T, nDispID>
// CWebBrowserImpl<T, nDispID, TBase, TWinTraits>

////////////////////////////////////////////////////////////////////////////////

// Undocumented command ids
static const CLSID CGID_IWebBrowser = {0xED016940L,0xBD5B,0x11cf,0xBA, 0x4E,0x00,0xC0,0x4F,0xD7,0x08,0x16};

enum WebBrowserCommand
{
	WBC_FIND		= 1,
	WBC_VIEWSOURCE	= 2,
	WBC_OPTIONS		= 3
};

////////////////////////////////////////////////////////////////////////////////

class CWebBrowser
{
private:
	CComPtr<IWebBrowser2>	m_pBrowser;

public: // property
	__declspec(property(get=get_RegisterAsBrowser, put=put_RegisterAsBrowser)) bool RegisterAsBrowser;
	__declspec(property(get=get_RegisterAsDropTarget, put=put_RegisterAsDropTarget)) bool RegisterAsDropTarget;
	__declspec(property(get=get_Visible, put=put_Visible)) bool Visible;
	__declspec(property(get=get_Offline, put=put_Offline)) bool Offline;
	__declspec(property(get=get_Silent, put=put_Silent)) bool Silent;
	__declspec(property(get=get_TopLevelContainer)) bool IsTopLevelContainer;
	__declspec(property(get=get_Busy)) bool IsBusy;
	__declspec(property(get=get_Title)) CString Title;
	__declspec(property(get=get_URL)) CString URL;
	__declspec(property(get=get_ReadyState)) READYSTATE ReadyState;

public: // operation
	HRESULT AdviseWebBrowser(CAxWindow& axWindow)
	{
		return axWindow.QueryControl(&m_pBrowser);
	}
	HRESULT UnadviseWebBrowser(CAxWindow& axWindow)
	{
		m_pBrowser.Release();
		return S_OK;
	}
	HRESULT QueryWebBrowser(IWebBrowser2** ppWebBrowser)
	{
		return m_pBrowser.CopyTo(ppWebBrowser);
	}
	HRESULT ExecWB(OLECMDID cmdID, OLECMDEXECOPT cmdexecopt = OLECMDEXECOPT_DODEFAULT, VARIANT* pvaIn = NULL, VARIANT* pvaOut = NULL)
	{
		return m_pBrowser->ExecWB(cmdID, cmdexecopt, pvaIn, pvaOut);
	}
	HRESULT ExecCommand(WebBrowserCommand nCmdID, DWORD nCmdExecOpt = 0, VARIANTARG *pvaIn = NULL, VARIANTARG *pvaOut = NULL)
	{
		CComQIPtr<IOleCommandTarget> pCmdTarget(m_pBrowser);
		if(!pCmdTarget)
			return E_FAIL;
		return pCmdTarget->Exec(&CGID_IWebBrowser, nCmdID, nCmdExecOpt, pvaIn, pvaOut);
	}
	HRESULT Navigate(LPCTSTR url)
	{
		return m_pBrowser->Navigate(CComBSTR(url), 0, 0, 0, 0);
	}
	HRESULT Refresh(bool completely = false)
	{
		if(completely)
		{
			CComVariant level((long)REFRESH_COMPLETELY);
			return m_pBrowser->Refresh2(&level);
		}
		else
			return m_pBrowser->Refresh();
	}
	HRESULT Stop()
	{
		return m_pBrowser->Stop();
	}
	HRESULT GoHome()
	{
		return m_pBrowser->GoHome();
	}
	HRESULT GoBack(INT num = 1)
	{
		HRESULT hr;
		for(INT i = 0; i < num; i++)
		{
			if FAILED(hr = m_pBrowser->GoBack())
				return hr;
		}
		return S_OK;
	}
	HRESULT GoForward(INT num = 1)
	{
		HRESULT hr;
		for(INT i = 0; i < num; i++)
		{
			if FAILED(hr = m_pBrowser->GoForward())
				return hr;
		}
		return S_OK;
	}
	HRESULT GoUp(INT num = 1)
	{
		CString url = this->URL;
		LPTSTR buffer = url.GetBuffer();
		ATLASSERT(buffer); // NOTIMPL
		BOOL isURL = PathIsURL(buffer);
		// TODO: t@CURL̏ꍇ̏B
		//BOOL isFileURL = UrlIsFileUrl(buffer);
		if(isURL)
		{	// URL
			if(UrlIsIndex(buffer))
				UrlRemoveLeaf(buffer);
			while(num-- > 0 && UrlRemoveLeaf(buffer)) {}
			UrlAddSlash(buffer);
		}
		else
		{	// Windows Path
			bool isIndex = PathIsIndex(buffer);
			if(isIndex)
				PathRemoveFileSpec(buffer); // index.html ̏ꍇ͂Qi邽
			while(num-- > 0 && PathRemoveFileSpec(buffer)) {}
			// index.html ܂ index.htm 邩ǂׂB
			TCHAR tmp[MAX_PATH];
			lstrcpy(tmp, buffer);
			PathAppend(tmp, _T("index.html"));
			if(PathFileExists(tmp))
				lstrcpy(buffer, tmp);
			else
			{
				lstrcpy(tmp, buffer);
				PathAppend(tmp, _T("index.htm"));
				if(PathFileExists(tmp))
					lstrcpy(buffer, tmp);
			}
		}
		url.ReleaseBuffer();
		return Navigate(url);
	}
	OLECMDF QueryStatusWB(OLECMDID cmdID) const
	{
		OLECMDF result;
		m_pBrowser->QueryStatusWB(cmdID, &result);
		return result;
	}
	HRESULT ShowFavoriteAddDialog()
	{
		CString url = this->URL;
		if(!PathIsURL(url))
			return E_FAIL;
		CComPtr<IShellUIHelper> pShellUIHelper;
		HRESULT hr = pShellUIHelper.CoCreateInstance(CLSID_ShellUIHelper);
		if FAILED(hr)
			return hr;
		CComVariant name(this->Title);
		pShellUIHelper->AddFavorite(CComBSTR(url), &name);
		return S_OK;
	}

	// Op(IEnumTravelLogEntry* entry)
	template < class Op >
	HRESULT QueryTravelLog(Op& op, INT num)
	{
		HRESULT hr;
		CComPtr<IServiceProvider>	pService;
		CComPtr<ITravelLogStg>		pTravelLog;
		CComPtr<IEnumTravelLogEntry>	pEnum;

		if(FAILED(hr = m_pBrowser.QueryInterface(&pService))
		|| FAILED(hr = pService->QueryService(SID_STravelLogCursor, IID_ITravelLogStg, (void**)&pTravelLog))
		|| FAILED(hr = pTravelLog->EnumEntries(num < 0 ? TLEF_RELATIVE_BACK : TLEF_RELATIVE_FORE, &pEnum)))
			return hr;

		CComPtr<ITravelLogEntry> pEntry;
		UINT n = 0;
		UINT numMax = abs(num);
		while(n < numMax && pEnum->Next(1, &pEntry, NULL) == S_OK)
		{
			op(pEntry.p);
			n++;
			pEntry.Release();
		}
		return num > 0 ? S_OK : S_FALSE;
	}

#if 0
	IDispatch* GetApplication() const
	{
		IDispatch* result;
		m_pBrowser->get_Application(&result);
		return result;
	}
	IDispatch* GetParentBrowser() const
	{
		IDispatch* result;
		m_pBrowser->get_Parent(&result);
		return result;
	}
	IDispatch* GetContainer() const
	{
		IDispatch* result;
		m_pBrowser->get_Container(&result);
		return result;
	}
	IDispatch* GetHtmlDocument() const
	{
		ATLASSERT(m_pBrowser-> != NULL);

		IDispatch* result;
		m_pBrowser->get_Document(&result);
		return result;
	}
	void PutProperty(LPCTSTR lpszProperty, const VARIANT& vtValue)
	{
		CComBSTR bstrProp(lpszProperty);
		m_pBrowser->PutProperty(bstrProp, vtValue);
	}
	bool GetProperty(LPCTSTR lpszProperty, CString& strValue)
	{
		CComBSTR bstrProperty(lpszProperty);

		bool bResult = false;
		CComVariant vReturn;
		vReturn.vt = VT_BSTR;
		vReturn.bstrVal = NULL;
		HRESULT hr = m_pBrowser->GetProperty(bstrProperty, &vReturn);

		if (SUCCEEDED(hr))
		{
			strValue = CString(vReturn.bstrVal);
			bResult = true;
		}

		return bResult;
	}
	CComVariant GetProperty(LPCTSTR lpszProperty)
	{
		ATLASSERT(m_pBrowser-> != NULL);

		CComVariant result;
		CComQIPtr<IDispatch> spDisp = m_pBrowser->;

		CComDispatchDriver::GetProperty(spDisp, 0x12f, &result);
		return result;

//		static BYTE parms[] = VTS_BSTR;
//		m_wndBrowser.InvokeHelper(0x12f, DISPATCH_METHOD,
//			VT_VARIANT, (void*)&result, parms, lpszProperty);
	}
#endif

public: // property implementation
	bool get_RegisterAsBrowser() const
	{
		VARIANT_BOOL result;
		m_pBrowser->get_RegisterAsBrowser(&result);
		return (result == VARIANT_TRUE);
	}
	void put_RegisterAsBrowser(bool bValue)
	{
		m_pBrowser->put_RegisterAsBrowser(bValue ? VARIANT_TRUE : VARIANT_FALSE);
	}
	bool get_RegisterAsDropTarget() const
	{
		VARIANT_BOOL result;
		m_pBrowser->get_RegisterAsDropTarget(&result);
		return (result == VARIANT_TRUE);
	}
	void put_RegisterAsDropTarget(bool bValue)
	{
		m_pBrowser->put_RegisterAsDropTarget(bValue ? VARIANT_TRUE : VARIANT_FALSE);
	}
	bool get_Visible() const
	{
		VARIANT_BOOL result;
		m_pBrowser->get_Visible(&result);
		return (result == VARIANT_TRUE);
	}
	void put_Visible(bool bValue)
	{
		m_pBrowser->put_Visible(bValue ? VARIANT_TRUE : VARIANT_FALSE);
	}
	bool get_Offline() const
	{
		VARIANT_BOOL result;
		m_pBrowser->get_Offline(&result);
		return (result == VARIANT_TRUE);
	}
	void pur_Offline(bool bValue)
	{
		m_pBrowser->put_Offline(bValue ? VARIANT_TRUE : VARIANT_FALSE);
	}
	bool get_Silent() const
	{
		VARIANT_BOOL result;
		m_pBrowser->get_Silent(&result);
		return (result == VARIANT_TRUE);
	}
	void put_Silent(bool bValue)
	{
		m_pBrowser->put_Silent(bValue ? VARIANT_TRUE : VARIANT_FALSE);
	}
	bool get_Busy() const
	{
		VARIANT_BOOL result;
		m_pBrowser->get_Busy(&result);
		return (result == VARIANT_TRUE);
	}
	bool get_TopLevelContainer() const
	{
		VARIANT_BOOL result;
		m_pBrowser->get_TopLevelContainer(&result);
		return (result == VARIANT_TRUE);
	}
	CString get_Title() const
	{
		CComBSTR bstr;
		m_pBrowser->get_LocationName(&bstr);
		return CString(bstr);
	}
	CString get_URL() const
	{
		CComBSTR bstr;
		m_pBrowser->get_LocationURL(&bstr);
		return CString(bstr);
	}
	READYSTATE get_ReadyState() const
	{
		READYSTATE result;
		m_pBrowser->get_ReadyState(&result);
		return result;
	}
};

////////////////////////////////////////////////////////////////////////////////

template < class T >
class ATL_NO_VTABLE CWebBrowserCommands
{
public: // bZ[W}bv
	BEGIN_MSG_MAP(CWebBrowserCommands)
		COMMAND_ID_LAMBDA(ID_FILE_SAVE			, { static_cast<T*>(this)->ExecWB(OLECMDID_SAVEAS); })
		COMMAND_ID_LAMBDA(ID_FILE_SAVE_AS		, { static_cast<T*>(this)->ExecWB(OLECMDID_SAVEAS); })
		COMMAND_ID_LAMBDA(ID_FILE_PAGE_SETUP	, { static_cast<T*>(this)->ExecWB(OLECMDID_PAGESETUP); })
		COMMAND_ID_LAMBDA(ID_FILE_PRINT			, { static_cast<T*>(this)->ExecWB(OLECMDID_PRINT); })
		COMMAND_ID_LAMBDA(ID_FILE_PRINT_PREVIEW	, { static_cast<T*>(this)->ExecWB(OLECMDID_PRINTPREVIEW); })
		COMMAND_ID_LAMBDA(ID_FILE_PROPERTY		, { static_cast<T*>(this)->ExecWB(OLECMDID_PROPERTIES); })
		COMMAND_ID_LAMBDA(ID_EDIT_CUT			, { static_cast<T*>(this)->ExecWB(OLECMDID_CUT); })
		COMMAND_ID_LAMBDA(ID_EDIT_COPY			, { static_cast<T*>(this)->ExecWB(OLECMDID_COPY); })
		COMMAND_ID_LAMBDA(ID_EDIT_PASTE			, { static_cast<T*>(this)->ExecWB(OLECMDID_PASTE); })
		COMMAND_ID_LAMBDA(ID_EDIT_FIND			, { static_cast<T*>(this)->ExecCommand(WBC_FIND); })
		COMMAND_ID_LAMBDA(ID_EDIT_SELECT_ALL	, { static_cast<T*>(this)->ExecWB(OLECMDID_SELECTALL); })
		COMMAND_ID_LAMBDA(ID_VIEW_REFRESH		, { static_cast<T*>(this)->Refresh(); })
		COMMAND_ID_LAMBDA(ID_VIEW_STOP			, { static_cast<T*>(this)->Stop(); })
		COMMAND_ID_LAMBDA(ID_VIEW_BACK			, { static_cast<T*>(this)->GoBack(); })
		COMMAND_ID_LAMBDA(ID_VIEW_FORWARD		, { static_cast<T*>(this)->GoForward(); })
		COMMAND_ID_LAMBDA(ID_VIEW_UP			, { static_cast<T*>(this)->GoUp(); })
		COMMAND_ID_LAMBDA(ID_VIEW_HOME			, { static_cast<T*>(this)->GoHome(); })
//		COMMAND_ID_LAMBDA(ID_VIEW_SOURCE		, { static_cast<T*>(this)->ExecCommand(WBC_VIEWSOURCE); })
//		COMMAND_ID_LAMBDA(ID_VIEW_OPTION		, OnViewOption)
		COMMAND_ID_LAMBDA(ID_FAVORITE_ADD		, { static_cast<T*>(this)->ShowFavoriteAddDialog(); })
		COMMAND_RANGE_ID_LAMBDA(ID_VIEW_FONT_SMALLEST, ID_VIEW_FONT_LARGEST, { OnViewFont(LOWORD(wParam) - ID_VIEW_FONT_SMALLEST); })
	END_MSG_MAP()

#if 0
	LRESULT OnViewOption(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL /*bHandled*/)
	{
		T* static_cast<T*>(this) = static_cast<T*>(this);
		static_cast<T*>(this)->ExecCommand(WBC_OPTIONS);
		// this is modeless
		// ::ShellExecute(0, "open", "control.exe", "inetcpl.cpl", ".", SW_SHOW);
		return 0;
	}
#endif

	void OnViewFont(ULONG factor)
	{
		ATLASSERT(factor < 5);
		CComVariant vaFactor(factor);
		static_cast<T*>(this)->ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER, &vaFactor);
	}
};

////////////////////////////////////////////////////////////////////////////////

// ĂяoK / Ԓl / ̐ / { ̌^ }
static _ATL_FUNC_INFO FUNC_Void					= {CC_STDCALL, VT_EMPTY, 0, { NULL } };
static _ATL_FUNC_INFO FUNC_BSTR					= {CC_STDCALL, VT_EMPTY, 1, { VT_BSTR } };
static _ATL_FUNC_INFO FUNC_Dispatch_RefVar		= {CC_STDCALL, VT_EMPTY, 2, { VT_DISPATCH, (VT_BYREF | VT_VARIANT) } };
static _ATL_FUNC_INFO FUNC_BeforeNavigate2		= {CC_STDCALL, VT_EMPTY, 7,	{ VT_DISPATCH, VT_BYREF|VT_VARIANT, VT_BYREF|VT_VARIANT, VT_BYREF|VT_VARIANT, VT_BYREF|VT_VARIANT, VT_BYREF|VT_VARIANT, VT_BYREF|VT_BOOL } };
static _ATL_FUNC_INFO FUNC_NewWindow2			= {CC_STDCALL, VT_EMPTY, 2, { (VT_BYREF | VT_DISPATCH ), (VT_BYREF | VT_BOOL)}};
static _ATL_FUNC_INFO FUNC_CommandStateChange	= {CC_STDCALL, VT_EMPTY, 2, { VT_I4, VT_BOOL } };
static _ATL_FUNC_INFO FUNC_ProgressChange		= {CC_STDCALL, VT_EMPTY, 2, { VT_I4, VT_I4 } };

////////////////////////////////////////////////////////////////////////////////

template < class T, UINT nDispID >
class ATL_NO_VTABLE CWebBrowserEvents : public IDispEventSimpleImpl<nDispID, CWebBrowserEvents, &DIID_DWebBrowserEvents2>
{
private:
	CComPtr<IUnknown>	m_pSink;

public: // I[o[Ch
	void OnTitleChange(const CString& title)		{}
	void OnStatusTextChange(const CString& text)	{}
	void OnProgressChange(LONG Progress, LONG ProgressMax)	{}
	void OnNavigateComplete(IDispatch* pDisp, const CString& url)	{}
	bool OnCancelNavigate(IDispatch* pDisp, BSTR url, BSTR targetFrameName, VARIANT* vtPostData, BSTR header)	{ return false; }
	void OnNewWindow(IWebBrowser2** ppNewBrowser)	{}

public: // Iy[V
	HRESULT AdviseWebBrowserEvents(CAxWindow& axWindow)
	{
		HRESULT hr;
		if FAILED(hr = axWindow.QueryControl(&m_pSink))
			return hr;
		if FAILED(hr = DispEventAdvise(m_pSink, &DIID_DWebBrowserEvents2))
		{
			m_pSink.Release();
			return hr;
		}
		return S_OK;
	}

	HRESULT UnadviseWebBrowserEvents(CAxWindow& axWindow)
	{
		if(!m_pSink)
			return E_FAIL;
		DispEventUnadvise(m_pSink, &DIID_DWebBrowserEvents2);
		m_pSink.Release();
		return S_OK;
	}

public: // DIID_DWebBrowserEvents2
	BEGIN_SINK_MAP(CWebBrowserEvents)
		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_TITLECHANGE		, _OnTitleChange		, &FUNC_BSTR)
		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_STATUSTEXTCHANGE	, _OnStatusTextChange	, &FUNC_BSTR)
//		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_PROPERTYCHANGE		, _OnPropertyChange		, &FUNC_BSTR)
		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_PROGRESSCHANGE		, _OnProgressChange		, &FUNC_ProgressChange)
		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_BEFORENAVIGATE2	, _OnBeforeNavigate2	, &FUNC_BeforeNavigate2)
		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_NAVIGATECOMPLETE2	, _OnNavigateComplete2	, &FUNC_Dispatch_RefVar)
//		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_DOCUMENTCOMPLETE	, _OnDocumentComplete	, &FUNC_Dispatch_RefVar)
		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_NEWWINDOW2			, _OnNewWindow2			, &FUNC_NewWindow2)
//		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_COMMANDSTATECHANGE	, _OnCommandStateChange	, &FUNC_CommandStateChange)
//		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_DOWNLOADBEGIN		, _OnDownloadBegin		, &FUNC_Void)
//		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_DOWNLOADCOMPLETE	, _OnDownloadComplete	, &FUNC_Void)
//		SINK_ENTRY_INFO(nDispID, DIID_DWebBrowserEvents2, DISPID_QUIT				, _OnQuit				, &FUNC_Void)
	END_SINK_MAP()

	void __stdcall _OnTitleChange(BSTR title)
	{
		T* pT = static_cast<T*>(this);
		pT->OnTitleChange(CString(title));
	}

	void __stdcall _OnStatusTextChange(BSTR text)
	{
		T* pT = static_cast<T*>(this);
		pT->OnStatusTextChange(CString(text));
	}

	void __stdcall _OnProgressChange(LONG Progress, LONG ProgressMax)
	{
		T* pT = static_cast<T*>(this);
		pT->OnProgressChange(Progress, ProgressMax);
	}

	void __stdcall _OnNavigateComplete2(IDispatch* pDisp, VARIANT* vtURL)
	{
		ATLASSERT(V_VT(vtURL) == VT_BSTR);
		T* pT = static_cast<T*>(this);
		pT->OnNavigateComplete(pDisp, CString(vtURL->bstrVal));
	}

	void __stdcall _OnBeforeNavigate2(
		/*[in]*/ IDispatch* pDisp,
		/*[in]*/ VARIANT* vtURL,
		/*[in]*/ VARIANT* vtFlags, 
		/*[in]*/ VARIANT* vtTargetFrameName,
		/*[in]*/ VARIANT* vtPostData,
		/*[in]*/ VARIANT* vtHeaders, 
		/*[out]*/ VARIANT_BOOL* Cancel)
	{
		ATLASSERT(V_VT(vtURL) == VT_BSTR);
		ATLASSERT(V_VT(vtTargetFrameName) == VT_BSTR);
		ATLASSERT(V_VT(vtHeaders) == VT_BSTR);
		ATLASSERT(Cancel != NULL);
		T* pT = static_cast<T*>(this);
		bool result = pT->OnCancelNavigate(pDisp, vtURL->bstrVal, vtTargetFrameName->bstrVal, vtPostData, vtHeaders->bstrVal);
		*Cancel = result ? VARIANT_TRUE : VARIANT_FALSE;
	}

	void __stdcall _OnNewWindow2(IDispatch** ppDisp, VARIANT_BOOL* Cancel)
	{
		CComPtr<IWebBrowser2> pBrowser;
		T* pT = static_cast<T*>(this);
		pT->OnNewWindow(&pBrowser);
		if(pBrowser && SUCCEEDED(pBrowser->QueryInterface(ppDisp)))
			*Cancel = VARIANT_FALSE;
		else
			*Cancel = VARIANT_TRUE;
	}
};

////////////////////////////////////////////////////////////////////////////////

template < class T, UINT nDispID, class TBase = CWindow, class CWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CWebBrowserImpl :
	public CWindowImpl<T, TBase, CWinTraits >,
	public CWebBrowser,
	public CWebBrowserCommands<T>,
	public CWebBrowserEvents<T, nDispID>
{
private:
	CAxWindow	m_Browser;

public: // message map
	BEGIN_MSG_MAP(CWebBrowserImpl)
		MESSAGE_HANDLER(WM_CREATE	, OnCreate)
		MESSAGE_HANDLER(WM_DESTROY	, OnDestroy)
		MESSAGE_HANDLER(WM_SETFOCUS	, OnSetFocus)
		MESSAGE_HANDLER(WM_SIZE		, OnSize)
		CHAIN_MSG_MAP(CWebBrowserCommands<T>)
	END_MSG_MAP()

//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		if((pMsg->message < WM_KEYFIRST   || pMsg->message > WM_KEYLAST) &&
		   (pMsg->message < WM_MOUSEFIRST || pMsg->message > WM_MOUSELAST))
			return FALSE;
		// Ctrl+N, O ȂǂHĂ܂߁ACtrlĂꍇ͏ȂB
		if(pMsg->message == WM_KEYDOWN && CKeyboard::IsPressed(VK_CONTROL))
			return FALSE;
		// give HTML page a chance to translate this message
		return (BOOL)m_Browser.SendMessage(WM_FORWARDMSG, 0, (LPARAM)pMsg);
	}

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		if(!m_Browser.Create(m_hWnd, rcDefault, _T("Shell.Explorer.2"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS))
			return -1;
		AdviseWebBrowser(m_Browser);
		AdviseWebBrowserEvents(m_Browser);
		return 0;
	}

	LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		UnadviseWebBrowser(m_Browser);
		UnadviseWebBrowserEvents(m_Browser);
		return 0;
	}

	LRESULT OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
	// ɕKvȂȋCB
/*		CComPtr<IHTMLDocument2> pDoc;
		if FAILED(GetDocument(&pDoc))
			return 0;
		CComPtr<IHTMLWindow2> pWnd;
		if FAILED(pDoc->get_parentWindow(&pWnd))
			return 0;
		pWnd->focus();
		ATLTRACE("OnSetFocus(pWnd->focus())\n");
*/
		m_Browser.SetFocus();
		return 0;
	}

	LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
	{
		m_Browser.MoveWindow(0, 0, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	}
};

////////////////////////////////////////////////////////////////////////////////

} // namespace mew

/*
	HRESULT GetDocument(IHTMLDocument2** ppDocument)
	{
		HRESULT hr;
		CComPtr<IWebBrowser2> m_pBrowser;
		if FAILED(hr = GetBrowser(&m_pBrowser))
			return hr;
		CComPtr<IDispatch> disp;
		if FAILED(hr = m_pBrowser->get_Document(&disp))
			return hr;
		if FAILED(hr = disp.QueryInterface(ppDocument))
			return hr;
		return S_OK;
	}

	bool IsEqualBrowser(IDispatch* pDisp)
	{	//Knowledge Base Q180366
		CComPtr<IWebBrowser2> m_pBrowser;
		GetBrowser(&m_pBrowser);
		return m_pBrowser.IsEqualObject(pDisp);
	}

*/
